# This script will generate a public and private key and place them
# in the directory $SPLUNK_HOME/etc/auth/audit/private.pem|public.pem


import os, sys, getopt, subprocess, time

def usage():
    print """
Usage: genAuditKeys.py

    -p privateKeyPath
    -k publicKeyPath
    -l keyLenth Default: 1024
    -d directory store

    $SPLUNK_HOME must also be set
"""
    sys.exit(1)

SPLUNK_HOME = os.environ.get("SPLUNK_HOME")
if not SPLUNK_HOME:
	usage()

AUDIT_DIR = SPLUNK_HOME + os.path.sep + "etc" + os.path.sep + "auth" +  os.path.sep + "audit"
PRIV_PATH = AUDIT_DIR + os.path.sep + "private.pem"
PUB_PATH  = AUDIT_DIR + os.path.sep + "public.pem"
KEYLEN   = "1024"

try:
    opts, args = getopt.getopt(sys.argv[1:], "d:p:k:l:", [])
    
    for o, a in opts:
        if o == '-p':
            PRIV_PATH = a
        elif o == '-k':
            PUB_PATH = a
        elif o == '-d':
            AUDIT_DIR = a
        elif o == '-l':
            KEYLEN = a
except getopt.GetoptError, exc:
    usage()
            

print PRIV_PATH
print PUB_PATH

if os.path.exists( PRIV_PATH ) or os.path.exists( PUB_PATH ):
    print "There already appears to be keys generated in %s dir. Please remove before proceeding" % AUDIT_DIR
    sys.exit(1)
else:
    try:
        os.makedirs( AUDIT_DIR )
    except:
        print "audit directory already exists '%s'" % AUDIT_DIR
            
    genKeyArgs = [
        "openssl",
        "genrsa",
        "-out",
        PRIV_PATH,
        KEYLEN
        ]
    
    print genKeyArgs
    subprocess.call( genKeyArgs )
    print AUDIT_DIR + os.path.sep + "private.pem generated."
    extractPubArgs = [
        "openssl",
        "rsa",
        "-in",
        PRIV_PATH,
        "-out",
        PUB_PATH,
        "-outform",
        "PEM",
        "-pubout"
        ]
    print AUDIT_DIR + os.path.sep + "public.pem generated."
    subprocess.call( extractPubArgs )
    
