import os, sys, getopt, subprocess, socket, shutil

def usage():
	print """
Usage: getWebCert.py

    -n <COMMONNAME> The CommonName of the cert.  This should match the DNS
                    name, if DNS is not available then the IP will suffice.
		    (optional)

    -l <KEYLEN>     Length of RSA key to generate. Default 1024
                    (optional)   

    SPLUNK_HOME environment variable must be set before running.
"""
	sys.exit(1)

SPLUNK_HOME = os.environ.get("SPLUNK_HOME")
if not SPLUNK_HOME:
    usage()

openssl_cnf = None
if sys.platform == "win32":
    # on windows the config file is stored in different location from UNIX
    openssl_cnf = os.path.join(SPLUNK_HOME, "openssl.cnf")
    os.environ["OPENSSL_CONF"] = openssl_cnf
else:
    openssl_cnf = os.path.join(SPLUNK_HOME, "openssl", "openssl.cnf")

COMMONNAME = ""
KEYLEN    = "1024"
try:
    opts, args = getopt.getopt(sys.argv[1:], "n:l:", [])
    for o, a in opts:
        if o == '-n':
            COMMONNAME = a
        if o == '-l':
            KEYLEN = a

except getopt.GetoptError, exc:
    usage()

if COMMONNAME == "":
    COMMONNAME = socket.gethostname()

os.chdir(os.path.join(SPLUNK_HOME, "etc", "auth"))

subprocess.call([
	"openssl",
	"req",
	"-newkey",
	"rsa:" + KEYLEN,
	"-sha1",
	"-passout",
	"pass:password",
	"-batch",
	"-subj",
	"/commonName=" + COMMONNAME + "/organizationName=SplunkUser/",
	"-keyout",
	"privkeySecure.pem",
	"-out",
	"req.pem",
	"-config",
	openssl_cnf,
])

subprocess.call([
	"openssl",
	"x509",
	"-req",
	"-in",
	"req.pem",
	"-sha1",
	"-passin",
	"pass:password",
	"-extensions",
	"usr_cert",
	"-CA",
	"ca.pem",
	"-CAkey",
	"ca.pem",
	"-CAcreateserial",
	"-out",
	"cert.pem",
	"-days",
	"1095",
])

# get rid of the password on the cert otherwise the server will prompt on startup.
subprocess.call([
	"openssl",
	"rsa",
	"-in",
	"privkeySecure.pem",
	"-passin",
	"pass:password",
	"-out",
	"privkey.pem",
])

for file in [ "privkey.pem", "cert.pem" ]:
    shutil.move(file, os.path.join(SPLUNK_HOME, "share", "splunk", "certs" ) + os.sep)
