# Copyright (C) 2005-2010 Splunk Inc.  All Rights Reserved.  Version 4.0
import re,sys,os,math
import splunk.Intersplunk as si
import logging as logger


def scorePath(depth, pathinfo, isDir):
    count = pathinfo['count']
    total = pathinfo['total']

    return total


    # !!! Unclear how to score directories.  benefits vs cost.
    # display saving vs information lost. 
    #
    # return total/count                     # get dense directories.
    # return math.log(depth * total / count) # get dense directories by punishing top level dirs as too simplistic
    # return depth * math.log(total / count) # log variation so total isn't takes so importantly.        
    # return depth * count * math.log(total) # favor deep directories with many files, favoring larger log(total)'s
    # return depth * count + math.log(total / count) # favor deep directories with many files, a favor dense dirs
    # KEEP IT SIMPLE. cover as many files as rapidly as possible. add depth to favor only child rather than its parent
    return count + depth
    
def floatSort(v1, v2):
    x = v1[1]
    y = v2[1]
    if y > x:
        return 1
    elif x > y:
        return -1
    else:
        return 0

def listDiff(list1, list2):
    for v in list2:
        if v in list1:
            list1.remove(v)

def getFileGroungs(results):
    try:
        badcounts = False
        try:
            maxcount   = int(options.get('maxcount',  '20'))
            if maxcount <= 0: 
                badcounts = True
        except:
            badcounts = True
        if badcounts:
            si.generateErrorResults("Error: invalid required 'maxcount' (1-INF) setting.")
            return
   
        sizefield  = options.get('sizefield',  None)
        pathfield  = options.get('pathfield',  None)
        if sizefield == None or pathfield == None:
            si.generateErrorResults("Error: both pathfield and sizefield must be specified.")
            return
            
        totalfield = options.get('totalfield', 'total-' + sizefield)
        countfield = options.get('countfield', 'count')
        delimiter = options.get('sep', os.sep)

        total = 0
        for result in results:
            total += int(result.get(sizefield, "0"))

        files = {}
        for result in results:
            size = int(result.get(sizefield, "0"))
            path = result.get(pathfield, "")
            files[path] = size

        # FIX BOGUS PATH VARIATIONS    
        deleted = []
        for path, size in files.items():
            # FIX TRAILING DELIMITERS -- /foo/ -> /foo adding /foo/'s count to /foo
            if path!=delimiter and path.endswith(delimiter):
                deleted.append(path)
                path = path[:-1]
                files[path] = size + files.get(path, 0)
            # FIX "" PATH TO BE "/"
            if path=="":
                deleted.append(path)
                path = delimiter
                files[path] = size + files.get(path, 0)
        for d in deleted:
            #print "doomed:", d
            del files[d]

        
        pathobjs = []
        notCovered = []
        deleted = []
        for path, size in files.items():
            pathobjs.append({ 'path': path, 'size': size})
            if len(path) == 0:
                print "path: '%s'" %  path
            notCovered.append(path)


        # CREATE MAP OF PATH TO ALL FILES IN IT
        dirobjs = {}
        for pathobj in pathobjs:
            path = pathobj['path']
            myparts = path.split(delimiter)
            mypath = ""
            if path == delimiter:
                myparts = [delimiter]
            if myparts[-1] == "":
                myparts.pop() # get rid of trailing /
            lastpos = len(myparts)-1
            for i, val in enumerate(myparts):
                mypath += val
                ellipse = ""
                if i < lastpos:
                    mypath += delimiter
                    ellipse = "..."

                pathinfo = dirobjs.get(mypath + ellipse, { 'count':0, 'total':0, 'files':set()})
                pathinfo['count'] += 1
                pathinfo['total'] += pathobj['size']
                pathinfo['files'].add(path)
                dirobjs[mypath + ellipse] = pathinfo

        # SCORE EACH PATH
        pathsAndScores = []
        resultcount = len(results)
        
        for mypath,val in dirobjs.items():
            depth = mypath.count(delimiter) + 1
            isDir = mypath.endswith("...")
            score = scorePath(depth, val, isDir)
            count = dirobjs[mypath]['count']
            if isDir and count > resultcount * 0.9:
                continue
            
            pathsAndScores.append((mypath, score))
            #print "score: %s\tpath: %s " % (score, mypath)

        # SORT PATHS BY SCORE
        pathsAndScores.sort(floatSort)

        # PICK TOP N AS INITIAL RESULTS, REST AS CANDIDATES
        clusters = pathsAndScores[:maxcount]
        for cluster in clusters:
            mypath  = cluster[0]
            myscore = cluster[1]
            files = dirobjs[mypath]['files']
            # REMOVE FILES COVERED BY THIS OATH FROM THE SET OF PATHS WE HAVE YET TO COVER
            listDiff(notCovered, files)

        results = []
        for i, cluster in enumerate(clusters):
            mypath  = cluster[0]
            myscore = cluster[1]
            count = dirobjs[mypath]['count']
            total = dirobjs[mypath]['total']
            #print "%s\tscore: %s\tcount: %s\ttotal: %s\tpath: %s" % (i, myscore, count, total, mypath)
            results.append({ pathfield: mypath, totalfield:total, countfield:count})

        othertotal = 0
        for otherpath in notCovered:
            othertotal += dirobjs[otherpath]['total']
        if len(notCovered) > 0:
            results.append( { pathfield: 'other', totalfield: othertotal, countfield: len(notCovered) } )
            
        si.outputResults(results)
    except Exception, e:
        import traceback
        stack =  traceback.format_exc()
        si.generateErrorResults("Error '%s'. %s" % (e, stack))
        si.generateErrorResults("Error '%s'." % e) #(e, stack))

if __name__ == '__main__':
    keywords,options = si.getKeywordsAndOptions()
    results,dummyresults,settings = si.getOrganizedResults()
    getFileGroungs(results)
    

    
