# Copyright (C) 2005-2010 Splunk Inc.  All Rights Reserved.  Version 4.0
import os, sys, datetime
import splunk.Intersplunk, splunk.mining.dcutils as dcu
from xml.sax import saxutils
from stat import *

# Create an rss file with the appropriate path
logger = dcu.getLogger()
graceful = 0


results,dummyresults,settings = splunk.Intersplunk.getOrganizedResults()
keywords, args = splunk.Intersplunk.getKeywordsAndOptions() # args = dcu.getArgValues()

if(len(args) < 5):
    results = dcu.getErrorResults(results, graceful, 'Usage: createrss <path=rss path> <name = name/title of the rss item> <link = rss item link> <descr = rss item description> <count = maximum number of items> [graceful = (1|0)]')
    splunk.Intersplunk.outputResults( results )
    sys.exit()
  
path   = args["path"]
name   = args["name"]
link   = args["link"]
desc   = args["descr"]
thresh = int(args["count"])

graceful  = 0
if("graceful" in args):
    graceful = int(args["graceful"])

# ensure nothing dangerous
if ".." in path or "/" in path or "\\" in path:
    results = dcu.getErrorResults(results, graceful, 'createrss Path location cannot contain "..", "/", or "\\".  Path was "' + str(path) + '".')
    splunk.Intersplunk.outputResults( results )
    sys.exit()

# prefix enforced path 
path = os.path.join(splunk.Intersplunk.splunkHome(), "var", "run", "splunk","rss", path)

foundFirstItem = 0
itemCount = 1 
skipItem = 0;

rss = """<?xml version="1.0" encoding="UTF-8"?>
<rss version="2.0">
    <channel>
        <title>Alert: %s</title>
        <link>%s</link>
        <description>Saved Searches Feed for saved search %s</description>
        <item>
            <title>%s</title>
            <link>%s</link>
            <description>%s</description>
            <pubDate>%s</pubDate>
        </item>
    </channel>
</rss>
""" % (	saxutils.escape(name),	# <title>
		saxutils.escape(link),	# <link>
		saxutils.escape(name),	# <description>
		saxutils.escape(name),	# <item><title>
		saxutils.escape(link),	# <item><link>
		saxutils.escape(desc),	# <item><description>
		datetime.datetime.now()) # <item><pubDate>

item = """        <item>
            <title>%s</title>
            <link>%s</link>
            <description>%s</description>
            <pubDate>%s</pubDate>
        </item>
""" % (	saxutils.escape(name),	# <item><title>
		saxutils.escape(link),	# <item><link>
		saxutils.escape(desc),	# <item><desc>
		datetime.datetime.now()) # <item><pubDate>


# if file exists, start looking for existing items
if os.path.exists(path) and os.stat(path)[ST_SIZE] > 0:
    rssfile = open(path, "r")
    tmprssfile = open(path+".tmp", "w+")
    
    for line in rssfile.readlines():
    	lineTmp = line.strip()
    	if lineTmp == '<item>':
    		# print "itemcount: %d, thresh: %d" % (itemCount, thresh)
    		if itemCount < thresh:
    			itemCount =	itemCount+1 
    			if foundFirstItem == 0: 
    				# found the first item, add the new one first
    				tmprssfile.write(item)
    				tmprssfile.write(line)
    				foundFirstItem = 1
    			else:
    				tmprssfile.write(line)
    		else:
    			skipItem = 1
    	elif lineTmp == '</item>':
    		if skipItem == 0:
    			tmprssfile.write(line)
    		else:
    			skipItem = 0
    	else:
    		if skipItem == 0:
    			tmprssfile.write(line)
    rssfile.close()
    tmprssfile.close()
    
    # replace old file with the new tmp file - need to remove then 
    # rename to ensure platform functional protability 
    try:
        os.unlink(path)
    except OSError, e: 
        pass
        
    logger.info("Atomically updating RSS feed: " + str(path))
    os.rename(path+".tmp", path)
else:
    # make parent dirs if they don't exist
    try:
        os.makedirs(os.path.dirname(path))
    except OSError, e: 
        pass
        
    rssfile = open(path, "w")
    rssfile.write(rss)
    rssfile.close()

splunk.Intersplunk.outputResults( results )
