# Copyright (C) 2005-2010 Splunk Inc.  All Rights Reserved.  Version 4.0
import sys,splunk.Intersplunk
import difflib,time
import splunk.mining.dcutils as dcu

logger = dcu.getLogger()

##  COMPARE TWO RESULTS
##  ARGS [pos1 pos2] [attribute to compare]
##
##  DEFAULTS = 1 2 _raw
##

help = """------------------------------------------------------------------------------------
diff x y compares x to y
  - indicates a line present in x but missing in y
  + indicates a line present in y but missing in x
  ! indicates a line that exists in both x and y, but contains different information 
------------------------------------------------------------------------------------"""

contexthelp = """------------------------------------------------------------------------------------
diff x y compares x to y
  *** a,b **** precedes results from x, lines a through b
  --- a,c ---- precedes results from y, lines a through c
  - indicates a line present in x but missing in y
  + indicates a line present in y but missing in x
  ! indicates a line that exists in both x and y, but contains different information 
------------------------------------------------------------------------------------"""




def diff(results, pos1, pos2, attr, showHeader, context):
    resultcount = len(results)

    if pos1 > resultcount or pos1 < 1 or pos2 < 1 or pos2 > resultcount:
        raise Exception("Index out of range. Must be between 1 and %s" % resultcount)
    
    val1 = results[pos1-1].get(attr, "")
    val2 = results[pos2-1].get(attr, "")
    maxlen = 9000
    outresults = []
    cutoff = False
    if val1.strip() == val2.strip():
        diff = "\n** Results are the Same **\n"
    else:
      if len(val1) > maxlen or len(val2) > maxlen:
          # cut text off at maxlen
          val1 = val1[:maxlen]
          val2 = val2[:maxlen]
      
      if html:
          htmldiffer = difflib.HtmlDiff()
          diff = htmldiffer.make_file(val1.splitlines(), val2.splitlines())
          if not tofile == '':
            f=open(tofile,'w')
            f.write(diff)
            f.close()
          
      elif context or unified:
          if context:
            difflist = list(difflib.context_diff(val1.splitlines(), val2.splitlines(), lineterm=""))
          else:
            difflist = list(difflib.unified_diff(val1.splitlines(), val2.splitlines(), lineterm=""))
          difflist.pop(0)
          difflist.pop(0)
          difflist.pop(0)
          diff = "\n".join(difflist)
          if showHeader:
              #diff = diff.replace("***************",help , 1)
              diff = contexthelp + "\n\n" + diff
      else:
          d = difflib.Differ()
          difflist = list(d.compare(val1.splitlines(), val2.splitlines()))
          diff = "\n".join(difflist)
          if showHeader:
              #diff = diff.replace("***************",help , 1)
              diff = help + "\n\n" + diff
      
    result = {}
    # fill in attributes with default values from top result
    for key, otherval in results[pos1-1].items():
        result[key] = otherval

    # set diff
    result["_raw"] = diff
    result["_decoration"] = "diff"
   
    outresults.append(result)
    return outresults


results = []
messages = {}
try:
    results,dummyresults,settings = splunk.Intersplunk.getOrganizedResults()
    html = False
    unified = True 
    header = False
    context = False
    attribute = "_raw"
    tofile = ''
    index1 = 1
    index2 = 2


    # poor mans opt
    for a in sys.argv:

        # Commenting out for now since the header has been put into the decorations stuff.
        #if a.startswith("header="):
        #    where = a.find('=')
        #    value = a[where+1:len(a)]
        #    if value.startswith('t') or value.startswith("T"):
        #        header = True

        if a.startswith("context="):
            where = a.find('=')
            value = a[where+1:len(a)]
            if value.startswith('t') or value.startswith("T"):
                context = True
        
        elif a.startswith("html="):
            where = a.find('=')
            value = a[where+1:len(a)]
            if value.startswith('t') or value.startswith("T"):
                html = True

        elif a.startswith("unified="):
            where = a.find('=')
            value = a[where+1:len(a)]
            if value.startswith('t') or value.startswith("T"):
                unified = True

        elif a.startswith("tofile="):
            where = a.find('=')
            tofile = a[where+1:len(a)]

        elif a.startswith("attribute="):
            where = a.find('=')
            attribute = a[where+1:len(a)]

        elif a.startswith("position1=") or a.startswith("pos1="):
            where = a.find('=')
            try:
                index1 = int(a[where+1:len(a)])
            except:
                splunk.Intersplunk.addWarnMessage(messages, "Unable to parse position1 value as an integer. Using default.")

        elif a.startswith("position2=") or a.startswith("pos2="):
            where = a.find('=')
            try:
                index2 = int(a[where+1:len(a)])
            except:
                splunk.Intersplunk.addWarnMessage(messages, "Unable to parse position2 value as an integer. Using default.")


    results = diff(results, index1,index2, attribute, header, context)
except Exception, e:
    import traceback
    stack =  traceback.format_exc()
    results = splunk.Intersplunk.generateErrorResults(str(e))
    logger.warn("invalid arguments passed to 'diff' search operator. Traceback: %s" % stack)

splunk.Intersplunk.outputResults(results, messages)
