#
# mock nodegraph data generator
#

import sys, random, time
import logging as logger
import splunk.Intersplunk as isp
import splunk.util as util


DEFAULT_ARGS = {

    'nodecount':     25,
    'levelcount':   10,
    'maxsiblings':  5,
    'seed': None,
    'interval':     60 * 17,
    'basetime':     time.time()
    

}

ARG_SEPARATOR = ';'



#
# event generators
#


def getConnectEvent(startNode, endNode=None):
    raw = 'nodegraph action=CONNECT start_node="%s"' % startNode
    output = {
        '_time': time.time(),
        'action': 'CONNECT',
        'start_node': startNode
    }
    if endNode:
        raw += ' end_node=%s' % endNode
        output['end_node'] = endNode
    output['_raw'] = raw
    return output
    
def getDisconnectEvent(startNode):
    return {
        '_time': time.time(),
        'action': 'DISCONNECT',
        'start_node': startNode,
        '_raw': 'nodegraph action=DISCONNECT start_node="%s"' % startNode
    }
    
def getSetEvent(startNode, endNode=None, kvpairs={}):
    raw = 'nodegraph action=SET start_node="%s"' % startNode
    output = {
        '_time': time.time(),
        'action': 'SET',
        'start_node': startNode
    }
    if endNode:
        raw += ' end_node="%s"' % endNode
        output['end_node'] = endNode
    output['_raw'] = raw
    output.update(kvpairs)
    
    return output
    


#
# data generators
#

def getIp():
    return '.'.join(map(lambda x: str(random.randrange(1,255)), ['']*4))
    
def getName():
    names = [ "2MB/second","about","actual)","after","airwaves","allowing","analyzing","applications","approaches","architecture","areas","aspects","attention","backup","become","becoming","behavior","being","beyond","blogs","BlueDot","Bluetooth","bookmarking","bottom","broadband","bubble","budgets","Business","business","businesses","buzzing","buzzword","buzzwords","centers","ChoicePoint","Cingular","collaboration","collected","combination","common","company","compliance","computers","consultants","content","costs","create","creation","crucial","customer","Delicious","delivery","developmental","device",
    "different","digital","efficient","elsewhere","engine","Enterprises","everyone","everywhere","experience","extending","faceless","false","favorite","finally","finding","flexible","Flikr","format","formats","found","giving","green","hasn't","haven't","headache","human","immensely","important","improve","increasingly","independent","information","innovative","integrated","interactive","interested","intranets","investigator","JavaScript","jumped","keeping","launch","launched","leading","learn","lines","listen","listening","major","management","mandatory","mash-ups","mature","media","medium",
    "Microsoft's","millions","mining","missing","model","moved","network","November","offer","officically","online","operating","optimization","organize","oriented","others","overhyped","paying","People","perhaps","phenomenon","phones","place","players","podcast","Podcasting","podcasts","popular","powerful","practice","predictions","prevalent","printers","private","probably","process","purchase","putting","question","quite","reality","realize","recorded","relied","remain","resources","results","retrieval","Search","search","second","server","service","service-oriented","services","sexiest",
    "share","shops","Skype","smartphones","Social","something","speeds","Sprint","Spurl","States","storage","support","system","systematically","technologies","testing","there's","though","tighten","tools","topic","track","tremendously","underlying","United","useful","users","using","Verizon","version","virtual","Virtualization","Vista","wants","WhatIscom","Whether","which","Wikipedia","wikis","wireless","worked","World","world","wrinkles","year's","years","you're","you've"]
    return names[random.randint(0,len(names)-1)]
    
def getProperty():
    props = [
        ('status', ['up', 'down']),
        ('throughput', [random.gammavariate(2,2)]),
        ('kbps', [random.betavariate(5,1)]),
        ('delay', [random.gammavariate(1,2)])
    ]
    
    thing = random.choice(props)
    key = thing[0]
    value = random.choice(thing[1])
    
    return {key: value}
 
 
    
#
# sequence iterator
#   

def treeIterator():
    
    # get participating nodes
    nodes = []
    for i in xrange(args['nodecount']):
        choice = random.randint(0,1)
        if choice == 0:
            nodes.append(getName())
        elif choice == 1:
            nodes.append(getIp())
            
    # get node pairs
    pairs = []
    for i in xrange(args['levelcount']):
        
        if i == 0: 
            siblingCount = 1
        else:
            siblingCount = random.randint(0,args['maxsiblings'])
            
        localStartNodes = []
        for j in xrange(siblingCount):
            if len(nodes):
                localStartNodes.append(nodes.pop())
        
        if len(nodes):
            for start in localStartNodes:
                pairs.append((start, random.choice(nodes)))
            
    # get properties
    toDisconnect = []
    for pair in pairs:
        yield getConnectEvent(pair[0], pair[1])
        
        for i in xrange(random.randrange(0,4)):
            yield getSetEvent(pair[0], pair[1], getProperty())
            
        if random.randrange(0,5) == 0:
            toDisconnect.append(pair[0])

        if toDisconnect and random.randrange(0,2) == 0:
            yield getDisconnectEvent(toDisconnect.pop())
            
            
    
#
# main
#

# merge any passed args
args = DEFAULT_ARGS
for item in sys.argv:
    kv = item.split('=')
    if len(kv) > 1:
        val = item[item.find('=') + 1:]
        try:
            val = int(val)
        except:
            pass
        args[kv[0]] = util.normalizeBoolean(val)

# run generator
try:
    if args['seed']:
        random.seed(args['seed'])
    results = []
    for x in treeIterator():
        results.append(x)
        
except Exception, e:
    logger.exception(e)
    results = isp.generateErrorResults(str(e))

# dump out to Intersplunk
isp.outputResults(results)
