# Copyright (C) 2005-2010 Splunk Inc.  All Rights Reserved.  Version 4.0
import re,sys,logging,splunk.Intersplunk, splunk.mining.dcutils as dcu
import smtplib, sys, StringIO, base64, os, socket
from xml.sax import saxutils
import cStringIO, csv
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase

importanceMap = {
    "highest": "1 (Highest)",
    "high"   : "2 (High)",
    "low"    : "4 (Low)",
    "lowest" : "5 (Lowest)",
    "1" : "1 (Highest)",
    "2" : "2 (High)",
    "4" : "4 (Low)",
    "5" : "5 (Lowest)"
}

logger = dcu.getLogger()

def toBool(strVal):
   if strVal == None:
       return False

   lStrVal = strVal.lower()
   if lStrVal == "true" or lStrVal == "t" or lStrVal == "1" or lStrVal == "yes" or lStrVal == "y" :
       return True 
   return False


def mail(argvals, settings, bodytext=''):

    serverURL  = argvals.get("server", "localhost")
    sender     = argvals.get("from", "splunk")
    to         = argvals.get("to" , None)
    cc         = argvals.get("cc" , None)
    bcc        = argvals.get("bcc", None)
    subject    = argvals.get("subject" , "Splunk Results")
    format     = argvals.get("format"  , "html")
    importance = argvals.get("priority", None)
    inline     = argvals.get("inline"  , "true").lower()
    plainText  = format != "html"
    sendresults = toBool(argvals.get("sendresults"  , "false")) 

    message = MIMEMultipart()
    message['Subject'] = subject

    recipients = []
    if to != None:
        message['To'] = to
        recipients.extend(to.split(','))

    if sender != None:
        message['From'] = sender
        
    if cc != None:
	for addr in cc.split(','):
           message['Cc'] = addr
           recipients.append(addr)
    if bcc != None:
	for addr in bcc.split(','):
           message['Bcc'] = addr
           recipients.append(addr)

    if importance != None:
        # look up better name
        val = importanceMap.get(importance.lower(), None)
        # unknown value, use value user supplied
        if val == None:
            val = importance
        message['X-Priority'] = val


    intro = "Saved search results.\n\n"
   
    if settings != None:
	user  = settings.get("user", None)
        if user != None:
            intro += "User: \'" + user + "\'\n"
    
    ssName = argvals.get("ssname", None)
    if ssName != None:
        intro += "Name: \'" + ssName + "\'\n"
    
    query = argvals.get("ssquery", None)
    if query != None:
        intro += "Query Terms: \'" + query + "\'\n"

    ssLink = argvals.get("sslink", None)
    if ssLink != None and not plainText:
        ssLink = "<a href=\"" + ssLink + "\">" + ssLink + "</a>"
    
    if ssLink != None:
      	intro += "Link to results: " + ssLink + "\n";
    
    ssSummary = argvals.get("sssummary", None)
    if ssSummary != None:
        intro += "Alert was triggered because of: \'" + ssSummary + "\'\n"
    
        
    bodyformat = "html"
    if plainText:
        bodyformat = "plain"
        

    #######################################################################################
    # create the body of the email and attach or inline results if required. Make sure to #
    # adhere to the requested format and proper tag balancing.                            #
    #######################################################################################

    body = StringIO.StringIO()

    if not plainText:
        body.write("<HTML><BODY>\n")
        intro = intro.replace("\n", "\n<BR/>\n") + "<BR/><BR/>\n"
    body.write(intro)
    
    if toBool(inline)  or inline == "none" or not sendresults:
    
        # inline the results if required to 
        if inline != "none" and sendresults:
            body.write("\n\n")
            body.write(bodytext)
            
        # correctly close the html if we're not in plaintext mode
        if not plainText:
            body.write("</BODY></HTML>")
        message.attach(MIMEText(body.getvalue(), bodyformat))
    else:
        attachStr = "\nSearch results attached:\n\n"; 
        if not plainText:
           attachStr = attachStr.replace("\n", "\n<BR/>\n") + "</BODY></HTML>"
        body.write(attachStr)

        message.attach(MIMEText(body.getvalue(), bodyformat))

        # now attach the results as a separate file
        
        filename = "splunk-results.csv"

        mimetype = "text/csv"

        attachment = MIMEBase(mimetype, None)


        attachment.set_payload(bodytext)
        attachment.add_header('Content-Disposition', 'attachment', filename=filename)
        message.attach(attachment)
            
    mail_log_msg = 'Sending email. subject="%s", results_link="%s", recepients="%s"' % (subject, ssLink, str(recipients)) 

    try:
        # make sure the sender is a valid email address
        if sender.find("@") == -1:
           sender = sender + '@' + socket.gethostname()
           if sender.endswith("@"):
              sender = sender + 'localhost'
     
        # send the mail
        smtp = smtplib.SMTP(serverURL)
        smtp.sendmail(sender, recipients, message.as_string())
        smtp.quit()
        #log an info message only if eveything passes
        logger.info(mail_log_msg)
    except Exception, e:
        #else log the same message at an error level
        logger.error(mail_log_msg)
        raise 
    
    
def numsort(x, y):
    if y[1] > x[1]:
        return -1
    elif x[1] > y[1]:
        return 1
    else:
        return 0

# sort columns from shortest to largest
def getSortedColumns(results):
    columnMaxLens = {}
    for result in results:
        for k,v in result.items():
            # ignore attributes that start with "_"
            if k.startswith("_") and k!="_raw" and k!="_time":
                continue
            newLen = len(str(v))
            oldMax = columnMaxLens.get(k, -1)
            
            #initialize the column width to the length of header (name)
            if oldMax == -1:
                columnMaxLens[k] = oldMax = len(k)
            if newLen > oldMax:
                columnMaxLens[k] = newLen

    colsAndCounts = columnMaxLens.items()
    colsAndCounts.sort(numsort)
    return colsAndCounts

def pad(count):
    padding = ""
    for i in range(0, count):
        padding += ' '
    return padding
    
def generateTextResults(results):
    columnMaxLens = getSortedColumns(results)
    text = ""
    space = " "*4
    
    # output column names
    for col, maxlen in columnMaxLens:
        val = col
        padsize = maxlen - len(val)
        text += val + pad(padsize) + space
    text += "\n" + "-"*len(text) + "\n"
    # output each result's values
    for result in results:
        for col, maxlen in columnMaxLens:
            val = result.get(col, "")
            padsize = maxlen - len(val)
            # left justify ALL the columns
            text += val + pad(padsize) + space
        text += "\n"
    return text

def generateHTMLResults(results):
    columnMaxLens = getSortedColumns(results)
    text = "<table border=1>\n<tr>"
    # output column names
    for col, maxlen in columnMaxLens:
        text += "<th>" + col + "</th>"
    text += "</tr>\n"
    # output each result's values
    for result in results:
        text += "<tr valign=top>"
        for col, maxlen in columnMaxLens:
            val = result.get(col, "")
            escval = saxutils.escape(val)
            text += "<td><pre>" + escval + "</pre></td>"
        text += "</tr>\n"
    text += "</table>"
    return text

def esc(val):
    return val.replace("\n", "\\n").replace("\r", "\\r").replace("\t", "\\t")

def generateCSVResults(results):
    columnMaxLens = getSortedColumns(results)
    s = cStringIO.StringIO()
    w = csv.writer(s)
    row = []
    # output column names
    row = [col for col, maxlen in columnMaxLens]    
    w.writerow(row)
    # output each result's values
    for result in results:
        row = [esc(result.get(col,"")) for col, maxlen in columnMaxLens]
        w.writerow(row)
    return s.getvalue()

def generateRawResults(results):
    str = splunk.Intersplunk.rawresultsToString(results)
    if(len(str) == 0):
        str = "The results contain no '_raw' field. Please choose another result emailing format (csv, plain or html)."
        
    return str


def sendEmail(results, settings):
    keywords, argvals = splunk.Intersplunk.getKeywordsAndOptions() 

    if not ('graceful' in argvals):
	argvals['graceful'] = 0

    if argvals.get("to") == None:
        return dcu.getErrorResults(results, argvals['graceful'], "sendemail operator received no recipient value, which can be specified by, for example \"to=some@emailaddress.com\"")

    if len(results) == 0:
        msgText = "No results."
    else:
        format = argvals.get("format", "html").lower()
        
        # always attach in CSV format
        if not toBool(argvals.get("inline", "true") ):
           format = "csv"
        
        if format == "raw":
            msgText = generateRawResults(results)
        elif format == "html":
            msgText = generateHTMLResults(results)
        elif format =="csv":
            msgText = generateCSVResults(results)        
        else:
            msgText = generateTextResults(results)

    try:
#       if not toBool(argvals.get("sendresults", "false") ):
#           msgText=''

        mail(argvals, settings, msgText)
    except Exception, e:
        #import traceback
        #stack   = traceback.format_exc()
        results = dcu.getErrorResults(results, argvals['graceful'], str(e) + ' while sending mail to: ' + argvals.get("to"))
    return results
        
results, dummyresults, settings = splunk.Intersplunk.getOrganizedResults()
results = sendEmail(results, settings)
splunk.Intersplunk.outputResults(results)


