# Copyright (C) 2005-2010 Splunk Inc.  All Rights Reserved.  Version 4.0
#
# This file contains possible attribute and value pairs for creating multikv rules.
# Multikv is the process of extracting events from table-like events, such as the output of top, ps, 
# ls, netstat, etc.
#
# There is NO DEFAULT multikv.conf.  To set custom configurations, place a multikv.conf in 
# $SPLUNK_HOME/etc/system/local/. For examples, see multikv.conf.example. 
# You must restart Splunk to enable configurations.
#
# To learn more about configuration files (including precedence) please see the documentation 
# located at http://www.splunk.com/base/Documentation/latest/Admin/Aboutconfigurationfiles
# 
# NOTE:  Configure multikv.conf only if you are unhappy with Splunk's automatic multikv
# behavior.  If you use the multikv search command with successful outcome, there is no reason to 
# create this file.

# A table-like event includes a table, which in turn consists of four parts or sections: 
#
#---------------------------------------------------------------------------------------
# Section Name | Description
#---------------------------------------------------------------------------------------
# pre          | optional: info/description (eg the system summary output in top)
# header       | optional: if not defined, fields are named Column_N
# body         | required: this is the body of the table from which child events are constructed
# post         | optional: info/description
#---------------------------------------------------------------------------------------

# NOTE: Each section up to and including the section for processing must have both a section 
# definition (below) and processing (also below) set.

[multikv_config_name]
	* Name your stanza to use with the mulitkv search command: 
		ex: '.... | multikv conf=$STANZA_NAME rmorig=f | ....'
	* Follow this stanza name with any number of the following attribute/value pairs.

#####################
# Section Definition
#####################
# Define where each section begins and ends.
# 

section_$NAME.start = <regex>     
	* A line matching this regex denotes the start of this section (inclusive).
	
OR

section_$NAME.start_offset = <int>     
	* Line offset from event-start or end of previous section where this section starts (inclusive).
	* Use this if you cannot define a regex for the start of the section.
	
section_$NAME.member = <regex>     
	* A line membership test.
	* Member iff lines match the regex.

section_$NAME.end = <regex>     
	* A line matching this regex denotes the end of this section (exclusive).

OR

section_name.linecount = <int>    
	* Specify the number of lines in this section.
	* Use this if you cannot specify a regex for the end of the section.

#####################
# Section processing
#####################
# Set processing for each section.
#

section_$NAME.ignore = <string-matcher>  
	* Member lines matching this string matcher will be ignored and thus NOT processed further
	* <string-matcher> = _all_ | _none_ | _regex_ <regex-list>
	
section_$NAMEe.replace = <quoted-str> = <quoted-str>, <quoted-str> = <quoted-str>.... 
	* List of the form toReplace = replaceWith.
	* Can have any number of toReplace = replaceWith.
	* Example: "%" = "_", "#" = "_"
	
section_$NAME.tokens	 = <chopper> | <tokenizer> | <aligner> | <token-list>
	* See below for definitions of each possible $VAL.

<chopper>   = _chop_, <int-list>               
	* Transform each string into a list of tokens specified by <int-list>.
	* <int-list> is a list of (offset, length) tuples.


<tokenizer> = _tokenize_ <max_tokens (int)> <delims>
	* <delims> = comma-separated list of delimiting chars.
	* Tokenize the string using the delim characters. 
	* This generates at most max_tokens tokens 
	* Set max_tokens to: 
	 	* -1 for complete tokenization
		* 0 to inherit from previous section (usually header)
		* Or to a non-zero number for a specific token count
	* If tokenization is limited by the max_tokens the rest of the string is added onto the last token.
	* Note: consecutive delimiters treated as an empty field.


<aligner> = _align_, <header_string>, <side>, <max_width>
	* Generates tokens by extracting text aligned to the specified header fields.
	* header_string: a complete or partial header field value the columns are aligned with.
	* side: either L or R (for left or right align, respectively).
	* max_width: the maximum width of the extracted field.
		* Set max_width to -1 for automatic width (this expands the field until any of the 
		following delimiters are found : " ", "\t")

<token_list> = _token_list_ <comma-separated list>
	* Defines a list of static tokens in a section. 
	* This is useful for tables with no header, for example in the output of 'ls -lah' 
	which misses a header altogether. 
