import os
import splunk.admin as admin
import splunk.windows as win
import splunk.entity as en

DEFAULT_WIN_COLLECTION = "localhost"
WIN_EVENT_LOG_PREFIX = "WinEventLog:"
WMI_PREFIX = "WMI:"
WMI_INTERVAL_VAL = 5

DISPLAY_LOGS_STR 			= "logs"
DISPLAY_HOSTS_STR			= "hosts"
DISPLAY_COLLECTION_NAME_STR = "name"

# These are strings of keys from wmi.conf and inputs.conf files
KEY_SERVER		= "server"
KEY_EVT_LOG_FILE= "event_log_file"

WIN_ALLEVENTLOGS_ENDPOINT = "admin/win-alleventlogs"
WMI_ENUM_EVENTLOGS = "admin/win-wmi-enum-eventlogs"

DEFAULT_STANZA_NAME = "_new"

class WinEventLogCollections(admin.MConfigHandler):

	def setup(self):
		if self.requestedAction in (admin.ACTION_CREATE, admin.ACTION_EDIT):
			self.supportedArgs.addOptArg("hosts")
			self.supportedArgs.addOptArg("logs")
			self.supportedArgs.addReqArg("lookup_host")
		if self.requestedAction in (admin.ACTION_NEW,):
			self.supportedArgs.addOptArg("lookup_host")
			self.supportedArgs.addOptArg("hosts")
			self.supportedArgs.addOptArg("logs")
		if self.requestedAction == admin.ACTION_LIST:
			self.supportedArgs.addOptArg("lookup_host")

	def setConfInfoData(self, confInfo, stanza, key, val):
		if key in confInfo[stanza]:
			confInfo[stanza][key] = val
		else:
			confInfo[stanza].append(key, val)

	"""
	Looking up windows event log channels for a given host.
	@param confInfo - once the list of channels is obtained, it's appeded to this object 
	@param host - provide the host to lookup the channels at
	@collName - name of the collection to which we're looking up channels for 
	@addToStanza - stanza under confInfo under which it should add log channel info;
	  if None, will default to collName
	"""
	def getLogChannels(self, confInfo, host, collName, selectedLogs, addToStanza = None):

		if addToStanza is None:
			addToStanza = collName

		if host == "":
			confInfo[addToStanza].append(DISPLAY_LOGS_STR, [])
		else:
			'''
			# deprecate this as we don't list all entities here anymore            
			evtChannels = []
			if collName != DEFAULT_WIN_COLLECTION:
				try:
					evtChannels = en.getEntities( WMI_ENUM_EVENTLOGS,
						sessionKey = self.getSessionKey(),
						server=host )[host]["logs"]
				except:
					# this may throw easily if, for example,
					# existing configured server id not longer
					# available
					pass
			else:
				evtChannels = en.getEntities( WIN_ALLEVENTLOGS_ENDPOINT,
					sessionKey = self.getSessionKey(), count = -1)

			if len(evtChannels) == 0:
				evtChannels = selectedLogs
			'''
			#encoded = [win.createJsonEncodedItem(channel, selectedLogs) for channel in evtChannels]
			encoded = selectedLogs
			self.setConfInfoData(confInfo, addToStanza, DISPLAY_LOGS_STR, encoded)

	def handleNew(self, confInfo):
		collName = self.callerArgs.get(DISPLAY_COLLECTION_NAME_STR, [""])[0]
		confInfo[DEFAULT_STANZA_NAME].append(DISPLAY_COLLECTION_NAME_STR, collName)

		hosts = self.callerArgs.get(DISPLAY_HOSTS_STR, [""])[0]
		confInfo[DEFAULT_STANZA_NAME].append(DISPLAY_HOSTS_STR, hosts)

		lookup_host = self.callerArgs.get("lookup_host", [""])[0]
		confInfo[DEFAULT_STANZA_NAME].append("lookup_host", lookup_host)

		self.getLogChannels(confInfo, lookup_host, collName, self.callerArgs.get("logs", []), DEFAULT_STANZA_NAME)

	"""
	This handle will only be called when a remote server is specified in the lookup_host field 
	"""
	def handleCreate(self, confInfo):
		isCreate = False
		
		if self.requestedAction == admin.ACTION_CREATE:
			isCreate = True

		#logsList = self.callerArgs.get("logs", [])
		logsList = []
        
		if "logs" in self.callerArgs:
			logsList = self.callerArgs["logs"]
            
		hostsStr = self.callerArgs.get(DISPLAY_HOSTS_STR, [None])[0]
		lookup_host = self.callerArgs.get("lookup_host", [None])[0]
		
		id = self.callerArgs.id

		# The default win collection can only be edited, not created
		if id == DEFAULT_WIN_COLLECTION:
			prefixLen = len(WIN_EVENT_LOG_PREFIX)
			
			# edit inputs config
			inputsDict = self.readConf("inputs")

			# interate throw each selected log and make sure they're
			# enabled or added
			for log in logsList:
				if log is None:
					continue
				log = WIN_EVENT_LOG_PREFIX + log
				if log not in inputsDict:
					inputsDict[log] = {}
				tmpDict = {"disabled" : 0}
				self.writeConf("inputs", log, tmpDict)

			# interate through each event log input and if they're not
			# in the selected list of logs, they are disabled
			for input in inputsDict:
				if input[:prefixLen] == WIN_EVENT_LOG_PREFIX:
					if input[prefixLen:] not in logsList:
						tmpDict = {"disabled" : 1}
						self.writeConf("inputs", input, tmpDict)
			self.reloadEventLogProcessor()
		else:
			wmiDict = self.readConf("wmi")

			wmi_name = WMI_PREFIX + id

			if wmi_name in wmiDict and not win.isDisabled(wmiDict[wmi_name].get("disabled", "0")) and isCreate == True:
				raise admin.ArgValidationException, "handleCreate: Splunk cannot create '%s' because " \
								"it already exists." % id # TK mgn 06/19/09

			# edit wmi config if wmi_name exists
			if wmi_name not in wmiDict:
				wmiDict[wmi_name] = {}

			event_log_field = ", ".join(logsList)
			# cannot write a zero-list event log stanza
			if len(logsList) == 0:
				raise admin.ArgValidationException, \
					"You cannot save a collection for remote polling without specifying any event logs."  # TK mgn 06/19/09

			hostsStr = ", ".join([i for i in [lookup_host, hostsStr] if i is not None])
			if hostsStr == "":
			    hostsStr = None

			tmpDict = { KEY_EVT_LOG_FILE : event_log_field,
						KEY_SERVER : hostsStr,
						"disabled" : 0,
						"interval" : WMI_INTERVAL_VAL }
		
			self.writeConf("wmi", wmi_name, tmpDict)
			win.reloadWmi(self.getSessionKey())

	def handleEdit(self, confInfo):
		self.handleCreate(confInfo)

	def handleList(self, confInfo):
		# load conf info from file
		self.getConf(confInfo)

		id = self.callerArgs.id

		if id != None:
			if id not in confInfo:
				raise admin.NotFoundException, \
					"Splunk cannot list a collection which does not exist or is not enabled."  # TK mgn 06/19/09
			# figure out the lookup host
			lookup_host = self.callerArgs.get("lookup_host", [None])[0]
			if id == DEFAULT_WIN_COLLECTION:
				lookup_host = "localhost"
			elif lookup_host is None:
				hosts = [i.strip() for i in confInfo[id][DISPLAY_HOSTS_STR].split(",")]
				if len(hosts) > 0:
					lookup_host = hosts[0]
					hosts = hosts[1:]
					confInfo[id].append(DISPLAY_HOSTS_STR, ", ".join(hosts))
				else:
					lookup_host = "localhost"

			self.getLogChannels(confInfo, lookup_host, id, confInfo[id][DISPLAY_LOGS_STR])

			confInfo[id].append("lookup_host", lookup_host)
			confInfo[id].append(DISPLAY_COLLECTION_NAME_STR, id)

	def handleRemove(self, confInfo):
		inputsDict = self.readConf("inputs")
		wmiDict = self.readConf("wmi")
		prefixLen = len(WIN_EVENT_LOG_PREFIX)
		
		id = self.callerArgs.id

		if id == DEFAULT_WIN_COLLECTION:
			raise admin.ArgValidationException, "Event logs collection \"%s\" cannnot be " \
				"removed" % DEFAULT_WIN_COLLECTION
		else:
			wmi_name = WMI_PREFIX + id 
			disabled = wmiDict[wmi_name].get("disabled", 0)
			if win.isDisabled(disabled):
				raise admin.ArgValidationException, "Event logs collection \"%s\" has already " \
					"been removed or disabled" % DEFAULT_WIN_COLLECTION

			tmpDict = {"disabled" : win.setDisabled(disabled, 1)}
			self.writeConf("wmi", wmi_name, tmpDict) 

			win.reloadWmi(self.getSessionKey())

	def getConf(self, confInfo):
		inputsDict = self.readConfCtx("inputs")
		wmiDict = self.readConfCtx("wmi")

		# the localmachine collection does not exists and we need to create it
		confInfo[DEFAULT_WIN_COLLECTION].append(DISPLAY_LOGS_STR, [])
		confInfo[DEFAULT_WIN_COLLECTION].append(DISPLAY_HOSTS_STR, "localhost")

		prefixLen = len(WIN_EVENT_LOG_PREFIX)
		def xformInput(name):
			return name[prefixLen:]
		for input in inputsDict:
			if input[:prefixLen] == WIN_EVENT_LOG_PREFIX:
				if not win.isDisabled(inputsDict[input].get("disabled", 0)):
					confInfo[DEFAULT_WIN_COLLECTION].append(DISPLAY_LOGS_STR, [xformInput(input)])
					# this is less than ideal, but this is an odd input.
					confInfo[DEFAULT_WIN_COLLECTION].copyMetadata(inputsDict[input])

		# use a separate confInfo to avoid name collisions when copying metadata.
		wmiConfInfo = admin.ConfigInfo()
		prefixLen = len(WMI_PREFIX)
		def xformWmi(name):
			return name[prefixLen:]
		for stanza in wmiDict:
			if stanza[:prefixLen] == WMI_PREFIX:
				name = xformWmi(stanza)
				if KEY_EVT_LOG_FILE in wmiDict[stanza]:
					if not win.isDisabled(wmiDict[stanza].get("disabled", 0)):

						logs = [log.strip() for log in wmiDict[stanza][KEY_EVT_LOG_FILE].split(",")]

						wmiConfInfo[name].append(DISPLAY_LOGS_STR, logs)
						if KEY_SERVER in wmiDict[stanza]:
							wmiConfInfo[name].append(DISPLAY_HOSTS_STR, wmiDict[stanza][KEY_SERVER])
						else:
							# if no server specified for some reason, we default to locahost
							wmiConfInfo[name].append(DISPLAY_HOSTS_STR, "localhost")
		wmiConfInfo.copyMetadata(wmiDict, xformWmi)

		# merge the two ConfigInfos, now that the metadata has been merged in.
		confInfo.mergeFrom(wmiConfInfo)

	def reloadEventLogProcessor(self):
		win.reloadConf(self, ["admin/win-eventlogs/_reload"])

admin.init(WinEventLogCollections, admin.CONTEXT_NONE)

