import splunk
import splunk.admin as admin
import splunk.appbuilder as appbuilder
import splunk.entity as en

HTTP_POST_TEMPLATE   = "template"
HTTP_POST_INSTALL    = "install"
HTTP_POST_LABEL      = "label"
HTTP_POST_ENABLED    = "enabled"
HTTP_POST_DESC       = "description"
HTTP_POST_CONFIGURED = "configured"
HTTP_POST_VISIBLE    = "visible"
HTTP_POST_MANAGEABLE = "manageable"
HTTP_POST_AUTHOR     = "author"
HTTP_POST_DESCR      = "description"


DEFAULT_TEMPLATE = "barebones"

class LocalAppsHandler(admin.MConfigHandler):
    '''
    Set up supported arguments
    '''
    def setup(self):
            
        if self.requestedAction == admin.ACTION_CREATE:
            for arg in [HTTP_POST_TEMPLATE, HTTP_POST_INSTALL, HTTP_POST_LABEL, HTTP_POST_VISIBLE, HTTP_POST_MANAGEABLE, HTTP_POST_AUTHOR, HTTP_POST_DESCR]:
                self.supportedArgs.addOptArg(arg)

        elif self.requestedAction == admin.ACTION_EDIT:
            for arg in [HTTP_POST_VISIBLE, HTTP_POST_MANAGEABLE, HTTP_POST_ENABLED, HTTP_POST_LABEL, HTTP_POST_AUTHOR, HTTP_POST_DESCR]:
                self.supportedArgs.addOptArg(arg)
                
    '''
    Create a new application
    
    This method handles both install and create commands. 
    When provided template argument, create is invoked, if
    install path is provided, install command is called.
    '''
    def handleCreate(self, confInfo):
        try:
            appName = self.callerArgs.id
            args = self.callerArgs.data

            if HTTP_POST_TEMPLATE in args and HTTP_POST_INSTALL in args:
                raise admin.ArgValidationException('Ambiguous request: provide either template or install argument')

            if len(appName) > 0:
                template = args[HTTP_POST_TEMPLATE][0].lower() if HTTP_POST_TEMPLATE in args else DEFAULT_TEMPLATE
                
                appLabel = args[HTTP_POST_LABEL][0] if HTTP_POST_LABEL in args else appName
                visible = args[HTTP_POST_VISIBLE][0].lower() if HTTP_POST_VISIBLE in args else 'true'
                manageable = args[HTTP_POST_MANAGEABLE][0].lower() if HTTP_POST_MANAGEABLE in args else 'true'
                
                url = appbuilder.createApp(appName, appLabel, visible, manageable, template)
                
                confInfo['Created'].append('name', appName)
                confInfo['Created'].append('template', template)
                confInfo['Created'].append('url', url)

                appbuilder.addUploadAssets(appName)
                
            elif HTTP_POST_INSTALL in args:
                location = args[HTTP_POST_INSTALL][0]
                appbuilder.installApp(location)
            
                confInfo['Installed'].append('name',appName)
                confInfo['Installed'].append('source_location',location)
                
            else:
                raise admin.ArgValidationException('Missing or wrong arguments provided for the new app')
        # translate exceptions to EAI exceptions...
        except splunk.RESTException, e:
            
            if e.statusCode == 409:
                raise admin.AlreadyExistsException(str(e.msg))
            raise

    '''
    Lists locally installed applications
    '''
    def handleList(self, confInfo):
        for appInfo in appbuilder.appInfoIterator():
            self.appName = appInfo['name']

            confInfo[self.appName].append(HTTP_POST_LABEL, appInfo['label'])
            confInfo[self.appName].append(HTTP_POST_ENABLED, appInfo['enabled'])
            # stub for SPL-17385 
            confInfo[self.appName].append(HTTP_POST_CONFIGURED, 'True')

            confInfo[self.appName].append(HTTP_POST_VISIBLE, appInfo['visible'])
            
    '''
    Controls local applications
    '''
    def handleEdit(self, confInfo):
        appName = self.callerArgs.id
        args = self.callerArgs
        action = None
        appbuilder.addUploadAssets(appName)

    '''
    Handles other commands
    '''
    def handleCustom(self, confInfo):
        action = self.customAction

        actionType = self.requestedAction
        
        # Create a package of an application
        if self.customAction == 'package':
            appName = self.callerArgs.id
            pkgPath = appbuilder.packageApp(appName)

            confInfo['Package'].append('name',appName)
            confInfo['Package'].append('path',pkgPath)
        

# initialize the handler, and add the actions it supports.    
admin.init(LocalAppsHandler, admin.CONTEXT_NONE)
