import os
import splunk.admin as admin
import wmi, re, splunk.windows as win, urllib
import splunk.auth, os
import splunk.entity as en

DEFAULT_STANZA_NAME = "_new"

# validates argument val using functor func; throws errmsg on error
def validateArg(func, val, errmsg):
	try:
		if func(val) == False:
			raise Exception
	except:
                raise admin.ArgValidationException, errmsg

class WMICollections(admin.MConfigHandler):

    def setup(self):
        if self.requestedAction in (admin.ACTION_CREATE, admin.ACTION_EDIT):
            self.supportedArgs.addReqArg("classes")
            self.supportedArgs.addReqArg("lookup_host")
            self.supportedArgs.addReqArg("interval")
            self.supportedArgs.addOptArg("fields")
            self.supportedArgs.addOptArg("instances")
            self.supportedArgs.addOptArg("server")
            self.supportedArgs.addOptArg("disabled")
        if self.requestedAction in (admin.ACTION_NEW,admin.ACTION_LIST):
            self.supportedArgs.addOptArg("lookup_host")
            self.supportedArgs.addOptArg("server")
            self.supportedArgs.addOptArg("interval")
            self.supportedArgs.addOptArg("class")
            self.supportedArgs.addOptArg("field")
            self.supportedArgs.addOptArg("disabled")
            self.supportedArgs.addOptArg("instance")

    def getCurrentSelection(self, name, confInfo):
        if name not in confInfo:
            raise KeyError, "Item with attribute '%s' must be present" % name

        configuredServer = confInfo[name]["server"]
        configuredClass = confInfo[name]["class"]

        # comma-separated list of fields and instances
        configuredFields = [i for i in confInfo[name]["fields"].split(", ")]
        configuredInstances = []
        if confInfo[name]["instances"] != "":
                configuredInstances = [i for i in confInfo[name]["instances"].split(", ")]

        configuredInterval = confInfo[name]["interval"]
        configuredDisabled = confInfo[name]["disabled"]
        configuredWql = confInfo[name]["wql"]

        if configuredDisabled == "":
            configuredDisabled = win.setDisabled("", 0)

        return [configuredServer, configuredClass, configuredFields, configuredInstances, configuredInterval, configuredDisabled, configuredWql]

    def setValueFromArgsAndConfig(self, confInfo, stanzaName, argName, dic, dicKey, isList = False, missingValue = None):
        if dicKey not in dic:
            raise KeyError, "Item with attribute '%s' must be present" % dicKey

        val = None
        if argName in self.callerArgs:
            if isList:
                val = self.callerArgs[argName]
            else:
                val = self.callerArgs[argName][0]
        elif argName not in confInfo[stanzaName]:
            if missingValue is not None:
                val = missingValue
            elif isList:
                val = []
            else:
                val = ""

        if argName not in confInfo[stanzaName]:
            confInfo[stanzaName].append(argName, val)
        else:
            confInfo[stanzaName][argName] = val

    def setConfInfoData(self, confInfo, stanza, key, val):
        if key in confInfo:
            confInfo[stanza][key] = val
        else:
            confInfo[stanza].append(key, val)

    def overrideValueIfNeedBe(self, val, argName):
        if argName in self.callerArgs:
            if isinstance(val, list):
                return self.callerArgs[argName]
            else:
                return self.callerArgs[argName][0]
        return val

    def findConfigValues(self, confInfo):
        id = self.callerArgs.id

        configuredStanza = configuredServer = configuredClass = configuredInterval = configuredDisabled = configuredWql = ""
        configuredFields = configuredInstances = []

        lookup_host = self.callerArgs.get("lookup_host", [None])[0]

        name = ""
        stanzaName = DEFAULT_STANZA_NAME
        if id is not None:
            name = id
            stanzaName = name

            if "name" in self.callerArgs:
                raise admin.ArgValidationException, \
                    "Cannot specify \"name\" argument when editing existing configuration"
        elif "name" in self.callerArgs:
            name = self.callerArgs["name"][0]
            stanzaName = name

        interval = ""
        if id is not None:
            tmpConfInfo = admin.ConfigInfo()
            win.getWmiConf(self, tmpConfInfo)

            try:
                [configuredServer, configuredClass, configuredFields, configuredInstances, configuredInterval, configuredDisabled, configuredWql] = self.getCurrentSelection(name, tmpConfInfo)
            except KeyError, e:
                # the stanza refered to by "name" does not exist in the config file
                raise admin.NotFoundException, "WMI collection '%s' does not exist." % name


        configuredServer = self.overrideValueIfNeedBe(configuredServer, "server")
        configuredClass = self.overrideValueIfNeedBe(configuredClass, "class")
        configuredFields = self.overrideValueIfNeedBe(configuredFields, "field")
        configuredInstances = self.overrideValueIfNeedBe(configuredInstances, "instance")
        configuredInterval = self.overrideValueIfNeedBe(configuredInterval, "interval")
        configuredDisabled = self.overrideValueIfNeedBe(configuredDisabled, "disabled")

        encodedClasses = []
        encodedFields = []
        wql = ""
        allFields = []

        # "lookup_host" and "server" settings get put together when saving the config.
        # here we are making sure that 1) lookup_host is not duplicated when editting,
        # and that 2) we acquire the first value in "server" if not provided by user.
        if configuredServer != "":
            serverList = [i.strip() for i in configuredServer.split(",")]
            if lookup_host is None:
                lookup_host = serverList[0]
                configuredServer = ", ".join(serverList[1:])
            else:
                try:
                    idx = serverList.index(lookup_host)  # raises ValueError if not found

                    # lookup_host is countained inside configuredServer: remove it from there
                    configuredServer = ", ".join(serverList[:idx] + serverList[idx+1:])
                except ValueError:
                    # lookup_host not in serverList: no need to change configuredServer
                    pass

        # default to "localhost" if unable to set otherwise
        if lookup_host is None or lookup_host == "":
            lookup_host = "localhost"

        if configuredClass != "":
            try:
                allFields = win.getWmiFields(configuredClass, lookup_host)
            except admin.InternalException, e:
                # cannot obtain fields of a class: assume custom WQL
                configuredClass = ""


        if configuredClass != "":
            # if specified field is not in the fields list, add it so that we can still show it
            if len(configuredFields) == 1 and configuredFields[0].strip() == "*" and len(allFields) > 0:
                # select all fields
                configuredFields = allFields.keys()

            encodedFields = configuredFields
            
            # generate WQL given the specs, if possible
            if configuredFields is not None and len(configuredFields) > 0:

                if len(configuredFields) == len(allFields):
                    # all fields selected: simply use "*" then
                    fldsParam = ["*"]
                else:
                    fldsParam = configuredFields

                fldsParam = configuredFields
                p = win.WQLParser(fields = fldsParam, clas = configuredClass)
                if len(configuredInstances) > 0:
                        p.setOrWqlWhereClauseValues("Name", configuredInstances)
                wql = str(p)
    
            encodedClasses = [configuredClass]
            
        if wql == "" and configuredWql != "":
                # it appears we have un-parsable WQL: put it verbatim as in the config file
                wql = configuredWql

        self.setConfInfoData(confInfo, stanzaName, "name", name)
        self.setConfInfoData(confInfo, stanzaName, "lookup_host", lookup_host)
        self.setConfInfoData(confInfo, stanzaName, "classes", encodedClasses)
        self.setConfInfoData(confInfo, stanzaName, "fields", encodedFields)
        self.setConfInfoData(confInfo, stanzaName, "instances", configuredInstances)
        self.setConfInfoData(confInfo, stanzaName, "wql", wql)
        self.setConfInfoData(confInfo, stanzaName, "interval", configuredInterval)
        self.setConfInfoData(confInfo, stanzaName, "server", configuredServer)
        self.setConfInfoData(confInfo, stanzaName, "disabled", configuredDisabled)

        
    def handleNew(self, confInfo):

        self.findConfigValues(confInfo)

    def handleCreate(self, confInfo):
        isEdit = False
        [id, confDict, stanzaName] = self.getConfInfo()

        if self.requestedAction == admin.ACTION_EDIT:
            isEdit = True

        # TODO: Should we check if the stanza that already there has been disabled
        # in that case we shouldn't throw an exception but go ahead allow an edit of
        # that disabled 
        if isEdit:
            if stanzaName is not None and stanzaName not in confDict:
                raise admin.ArgValidationException, "Cannot edit '%s', it does not exist." % id
            elif self.callerArgs.id is None:
                raise admin.ArgValidationException, "Collection to edit has not been specified."  # TK mgn 06/19/09
        else:
            # ACTION_CREATE
            if stanzaName is not None and stanzaName in confDict:
                raise admin.ArgValidationException, "Cannot create '%s', it already exists." % id

        lookup_host = self.callerArgs.get("lookup_host", [None])[0]
        server		= self.callerArgs.get("server", [None])[0]
        interval 	= self.callerArgs["interval"][0]
        wmiClass 	= self.callerArgs["classes"][0]
        wmiFields 	= self.callerArgs.get("fields", None)
        disabled	= self.callerArgs.get("disabled", ["0"])[0]
        wmiInstances	= self.callerArgs.get("instances", None)

        server = ", ".join([i for i in [lookup_host, server] if i is not None])
        if server == "":
            server = None

        if not wmiFields or wmiFields == [None]:
            wmiFields = ['*']
            
        if not wmiInstances or wmiInstances == [None]:
            wmiInstances = []
            
        wql = win.WQLParser(wmiFields, wmiClass)
        if not wmiInstances == [None]:
                # if win.ALL_INSTANCES is present, assume that user wants all of them
                wql.setOrWqlWhereClauseValues("Name", wmiInstances)

        # check if we're not exceeding the max length of wql query
        if len(str(wql)) > 16384:
            raise admin.ArgValidationException, "The WQL query exceeded its max length (16384 characters). Consider selecting All properties or instances."  # TK mgn 06/19/09

	# validate that interval is a valid integer with a reasonable value
	validateArg(lambda i: int(i) > 0 and int(i) <= 32000000, interval, "The interval must be a valid positive integer.")

        tmpDict = { "wql"	: str(wql),
            "disabled"	: disabled,
            "server"	: server,
            "interval"	: interval }
        self.writeConf("wmi", stanzaName, tmpDict)
        self.handleReload(confInfo)

    def handleEdit(self, confInfo):
        self.handleCreate(self)

    def handleRemove(self, confInfo):
        [id, confDict, stanzaName] = self.getConfInfo()

        if stanzaName not in confDict or win.isDisabled(confDict[stanzaName].get("disabled", 0)):
            raise admin.ArgValidationException, "Cannot remove '%s', it does not exist or it has been disabled." % id  # TK mgn 06/19/09

        stanza = confDict[stanzaName]
        stanza["disabled"] = "1"
        self.writeConf("wmi", stanzaName, stanza)
        self.handleReload(confInfo)

    def handleList(self, confInfo):
        id = self.callerArgs.id
        if id is None:
            win.getWmiConf(self, confInfo)
        else:
            self.findConfigValues(confInfo)

    def handleReload(self, confInfo):
        win.reloadWmi(self.getSessionKey())

    # returns the id, a dict of config items, and the full stanza name
    def getConfInfo(self):
        id = stanzaName = None
        if self.callerArgs.id is not None:
            id = self.callerArgs.id
        elif "name" in self.callerArgs:
            id = self.callerArgs["name"]

        if id is not None:
            stanzaName = "WMI:" + id

        return [id, self.readConf("wmi"), stanzaName]

admin.init(WMICollections, admin.CONTEXT_NONE)

