import splunk.admin as admin
import splunk.windows as win
import wmi, json

DEFAULT_STANZA = "WMIResult"
NAMEID = "nameid"
SERVER = "server"

class WMIFindCollection(admin.MConfigHandler):

    def setup(self):
        if self.requestedAction in (admin.ACTION_LIST,):
            self.supportedArgs.addOptArg(NAMEID)
            self.supportedArgs.addOptArg("server")
            self.supportedArgs.addOptArg("lookup_host")
            self.supportedArgs.addOptArg("class")
            self.supportedArgs.addOptArg("interval")
            self.supportedArgs.addOptArg("field")
            self.supportedArgs.addOptArg("disabled")

    def getCurrentSelection(self, name, confInfo):
        if name not in confInfo:
            raise KeyError, "Item with attribute '%s' must be present" % NAMEID

        configuredServer = confInfo[name]["server"]
        configuredClass = confInfo[name]["class"]

        # comma-separated list of fields
        configuredFields = [i for i in confInfo[name]["fields"].split(", ")]

        configuredInterval = confInfo[name]["interval"]
        configuredDisabled = confInfo[name]["disabled"]

        return [configuredServer, configuredClass, configuredFields, configuredInterval, configuredDisabled]

    def handleList(self, confInfo):
        configuredStanza = configuredServer = configuredClass = configuredInterval = configuredDisabled = ""
        configuredFields = []

        name = DEFAULT_STANZA

        #if NAMEID in self.callerArgs:
        if self.callerArgs.id is not None:
            name = self.callerArgs.id
            configuredStanza = name
            tmpConfInfo = admin.ConfigInfo()
            win.getWmiConf(self, tmpConfInfo)

            try:
                [configuredServer, configuredClass, configuredFields, configuredInterval, configuredDisabled] = self.getCurrentSelection(name, tmpConfInfo)
            except KeyError, e:
                # the stanza refered to by "name" does not exist in the config file
                pass

        lookOnServer = None
        if SERVER in self.callerArgs:
            lookOnServer = self.callerArgs[SERVER][0]

        if lookOnServer is None:
            confInfo[name].append("server", "")
        else:
            confInfo[name].append("server", lookOnServer)

        # if specified as an argument, replace the value
        if "class" in self.callerArgs:
            configuredClass = self.callerArgs["class"][0]

        allClasses = win.getWmiClasses(host = lookOnServer)
        
        encodedClasses = [u'--- Select Class ---']  # if you change this, make sure widgets/select.html#11 also changed
        encodedClasses.extend(sorted(allClasses))

        # there may be multiple "field" items returned
        if "field" in self.callerArgs:
            configuredFields = self.callerArgs["field"]

        fields = []
        instances = []
        if configuredClass is not None and len(configuredClass) > 0:
            # throws an admin exception on error
            allFields = win.getWmiFields(configuredClass, lookOnServer)

            fields = sorted(allFields.keys())
            # run all instances of a class
            instances = win.getWmiInstances(configuredClass, host = lookOnServer)

        interval = configuredInterval
        if "interval" in self.callerArgs:
            interval = self.callerArgs["interval"][0]

        disabled = configuredDisabled
        if "disabled" in self.callerArgs:
            disabled = self.callerArgs["disabled"][0]

        confInfo[name].append("classes", encodedClasses)
        confInfo[name].append("fields", fields)
        confInfo[name].append("interval", interval)
        confInfo[name].append("disabled", disabled)
        confInfo[name].append("instances", instances)

        wql = ""
        if configuredClass != "" and configuredFields is not None and len(configuredFields) > 0:
            wql = str(win.WQLParser(fields = configuredFields, clas = configuredClass))
        confInfo[name].append("wql", wql)

admin.init(WMIFindCollection, admin.CONTEXT_NONE)

