import cherrypy
from splunk.appserver.mrsparkle import *
from splunk.appserver.mrsparkle.lib import jsonresponse
import splunk, splunk.auth, splunk.util, splunk.entity, splunk.rest
import splunk.appserver.mrsparkle.lib.i18n as i18n
import logging

logger = logging.getLogger('splunk.appserver.controllers.licensing')

PURCHASE_LICENSE_ENDPOINT = 'http://www.splunk.com/r/getlicense' 
SPLUNKD_LICENSING_ENDPOINT = '/services/license/revert'

SPLUNKWEB_LICENSING_EDPOINT = '/licensing'
SPLUNKWEB_EDIT_LICENSE_ENDPOINT = '/manager/launcher/admin/license/license?action=edit'
RESTART_SPLUNK_ENDPOINT = '/manager/launcher/control'

FREE_LICENSE_NAME = 'splunk-free.license'

class LicensingController(BaseController):
    """
    Handle licensing messaging and modifications
    """
    
    @expose_page(must_login=True, methods=['GET', 'POST'])
    def index(self, switch_license=None, **kw):
        
        templateArgs = {'user_message': None}
        
        if cherrypy.request.method == 'GET':
            return self.render_template('licensing/index.html', templateArgs)

        elif cherrypy.request.method == 'POST':
            
            if switch_license == 'new_license':
                return self.redirect_to_url(SPLUNKWEB_EDIT_LICENSE_ENDPOINT)

            elif switch_license == 'purchase_license':
                return self.redirect_to_url(PURCHASE_LICENSE_ENDPOINT)
            
            elif switch_license == 'free_license':
                postArgs = {'name': FREE_LICENSE_NAME}
                
                try:
                    splunk.rest.simpleRequest(SPLUNKD_LICENSING_ENDPOINT, method='POST', postargs=postArgs, raiseAllErrors=True)
                    return self.redirect_to_url(RESTART_SPLUNK_ENDPOINT)
                # 401 errors
                except splunk.AuthorizationFailed, e:
                    logger.warn('Cannot change splunkd license as a non-admin user.')
                    templateArgs['user_message'] = _('You must be logged in as a user with administrative privileges to change the license.')
                    return self.render_template('licensing/index.html', templateArgs)
                # 500 errors
                except splunk.InternalServerError, e:
                    logger.error('An error occurred while attempting to change the splunkd license.')
                    if not e.extendedMessages == None and len(e.extendedMessages) > 0:
                        for msg in e.extendedMessages:
                            logger.error('splunkd returned the following message "%s"' % msg.get('text'))
                    templateArgs['user_message'] = _('There was an error switching your splunkd license to the free license. %(supportLinkOpen)s Please contact support.%(supportLinkClose)s') % { 'supportLinkOpen':'<a href="http://www.splunk.com/support">', 'supportLinkClose':'</a>' }
                    return self.render_template('licensing/index.html', templateArgs)
        
        # In case the endpoint is hit by some other req, eg put, delete, etc.
        # This should prevent accidental license changes.
        return self.redirect_to_url('/')
