# coding=UTF-8

import cherrypy
import logging
import splunk
import splunk.entity
import splunk.util
from splunk.appserver.mrsparkle import *
from splunk.appserver.mrsparkle.lib.jsonresponse import JsonResponse

from splunk.appserver.mrsparkle.lib.msg_pool import MsgPoolMgr, UI_MSG_POOL
import json

logger = logging.getLogger('splunk.appserver.controllers.messages')

class MessagesController(BaseController):
    # Map select splunkd messages to UI messages
    msg_map = {
        'Splunk must be restarted for changes to take effect.':  'Splunk must be restarted for changes to take effect.  [[/manager/search/control|Click here to restart from the Manager]].'
    }

    @route('/:action=uipool')
    @expose_page(must_login=True, handle_api=ONLY_API, methods='GET')
    def uipool(self, action, **args):

       cherrypy.response.headers['content-type'] = MIME_JSON

       # unauthed calls get nothing
       if not cherrypy.session.get('sessionKey'):
           return '/* unauthorized */'

       output = json.dumps(MsgPoolMgr.get_poolmgr_instance()[UI_MSG_POOL].list())

       return output

    @expose_page(must_login=True, handle_api=ONLY_API, methods='GET')
    @lock_session
    def uiindex(self, **kwargs):
        '''
        JSONResponse envelope of message data.
        '''
        resp = JsonResponse()

        try:
            msg = MsgPoolMgr.get_poolmgr_instance()[UI_MSG_POOL].pop(kwargs['id'])

            if msg:
                resp.addMessage(msg.severity.upper(), msg.text)

        except splunk.SplunkdConnectionException, e:
            logger.exception(e)
            resp.success = False
            resp.addFatal(_('The appserver was unable to connect to splunkd. Check if splunkd is still running. (%s)') % e.message)

        except Exception, e:
            logger.exception(e)
            resp.success = False
            resp.addError(e)
                
        return self.render_api(resp, use_etag=True)

    @route('/:action=index')
    @expose_page(must_login=True, handle_api=ONLY_API, methods='GET')
    @set_cache_level('never')
    @lock_session
    def index(self, action, **args):
        '''
        JSONResponse envelope of message data.

        URL: /api/messages/index/
        '''
        resp = JsonResponse()
        uri = '/admin/messages'
        try:
            entries = splunk.entity.getEntities(uri)
            msgs = lib.message.get_session_queue().get_all()

            for idx, entry in entries.iteritems():
                resp.addInfo(self.msg_map.get(entry[idx], entry[idx]))

            for msg in msgs:
                resp.addMessage(msg['level'].upper(), msg['message'])

        except splunk.SplunkdConnectionException, e:
            logger.exception(e)
            resp.success = False
            resp.addFatal(_('The appserver was unable to connect to splunkd. Check if splunkd is still running. (%s)') % e.message)

        except splunk.RESTException, e:
            logger.exception(e)
            resp.success = False
            resp.parseRESTException(e)
            
        except Exception, e:
            logger.exception(e)
            resp.success = False
            resp.addError(e)
                
        return self.render_api(resp, use_etag=True)
