import cherrypy, sys, re
from splunk.appserver.mrsparkle import *
import splunk.appserver.mrsparkle.lib.module
import splunk.search
import logging
import traceback

import splunk
import splunk.rest as rest

logger = logging.getLogger('splunk.appserver.controllers.module')

class ModuleController(BaseController):
    """/module"""

    @route('/')
    @expose_page(methods='GET')
    def listModules(self):
        return 'TODO: this page will list all of the modules that are registered in python'


    @route('/:host_app/:module/:action=render')
    @expose_page(methods='GET')
    def renderModule(self, host_app, module, action, **args):
        '''
        Provides generic module content rendering endpoint.
        
        The base module JS dispatcher will always append a 'client_app' param
        that contains the app context from which this request was made.  It is
        entirely different from the host_app where this module resides.
        '''

        # strip out the cache defeater
        args.pop('_', None)
        
        if not self._isModuleClassInApp(module, host_app):
            logger.warn('Could not load module "%s" in host app "%s"' % (module, host_app))
            return _('Could not find module "%(module)s" in the following host app "%(host_app)s"') % {'module':module, 'host_app':host_app}
            
        args['host_app'] = host_app
        
        # put in default client_app if incoming request did not have one
        args.setdefault('client_app', splunk.getDefault('namespace'))
        
        # TODO: change me when the modules are registered under a module
        try:
            module_name = module.split('.').pop()
            handler = getattr(sys.modules[__name__], module_name)
        except Exception, e:
            logger.exception(e)
            traceback.print_exc(e)
            return _('Splunk could not find a controller to import for the following module: "%s".') % module
        
        # get module content
        cherrypy.request.headers['content-type'] = MIME_HTML
        instance = handler(self)
        
        try:
            pageContent = instance.generateResults(**args)
        except Exception, e:
            logger.exception(e)
            traceback.print_exc(e)
            errorString = _("[%(module_name)s module] %(error_msg)s") % {'module_name':module_name, 'error_msg':e}
            pageContent = '<p class="moduleException">%s</p>' % errorString
        
        if cherrypy.response.headers.get('Cache-Control') == None:
            return util.set_cache_level('etag', pageContent)
        return pageContent        

    @route('/:module/:action=config')
    @expose_page(methods='GET')
    def configModule(self, module, action, **args):
        '''
        Provides generic module configuration information
        '''
        
        return 'configuration for module=%s' % module

        
    def _isModuleClassInApp(self, moduleClass, host_app):
        '''
        Determines if requested moduleClass has been defined in host_app.
        NOTE: This method will only return true if requests are made from
        the host app of 'system', as we are not supporting modules that
        exist in other apps (though they can be packaged as so).
        '''

        return host_app == SYSTEM_NAMESPACE
        
        #moduleMap = splunk.appserver.mrsparkle.lib.module.moduleMapper.getInstalledModules()
        
        #if moduleMap.has_key(moduleClass):
        #    return moduleMap[moduleClass]['appName'].lower() == host_app.lower()
        #else:
        #    return False


class ModuleHandler(object):
    
    def __init__(self, controller):
        self.controller = controller
        
        
    def getTemplatePath(self, modulePath):
        return '../../modules/%s' % modulePath.strip(' /')
        
    def generateResults(self, **args):
        '''
        Generates module HTML content.
        
        Override this method to provide module-specific content information.
        The 'args' dict is a mapping of all the URI querystring params that
        may be passed from the UI.
        '''
        
        return _('This module does not have a registered renderer.')
        
    def getConfig(self, viewId=None):
        '''
        Returns configuration information for a module.
        
        Override this method to provide module-specific configuration information.
        '''
        
        return _('This module does not have registered configuration information.')
        
