import cherrypy
from splunk.appserver.mrsparkle import *
from splunk.appserver.mrsparkle.lib import i18n
import splunk.searchhelp.searchhelper as sh
import splunk.searchhelp.utils as shutils
import logging

logger = logging.getLogger('splunk.appserver.controllers.searchhelper')

class SearchHelperController(BaseController):
    """/searchhelper"""

    @route('/')
    @expose_page(methods='GET', handle_api=True)
    @set_cache_level('etag')
    def index(self, search="", insertpos=None, minimize=False, namespace='search', snippet=False, earliest_time=None, latest_time=None,
              count=50, max_time=1, servers=None, useTypeahead=False, **kwargs):

        username   = cherrypy.session['user']['name']
        sessionKey = cherrypy.session['sessionKey']

        cherrypy.response.headers['content-type'] = MIME_HTML
   
        templateArgs = sh.help(sessionKey, namespace, username, search, insertpos, earliest_time, latest_time, count, max_time, servers, useTypeahead=useTypeahead)

        templateArgs['error'] = getSearchError(search, self.splunkd_urlhost, sessionKey)

        easter = getEasterEgg(search)
        
        template = 'searchhelper/index.html'
        if minimize:
            template = 'searchhelper/runningsearch.html'
        elif easter != None:
            template = easter[0]
            templateArgs['easter'] = easter[1]
        elif snippet:
            template = 'searchhelper/snippet.html'
        return self.render_template(template, templateArgs)


def getSearchError(search, urlhost, sessionKey):
    error = ""
    try:
        search = search.strip()
        # if user has moved on to next operator (i.e., lasst character is |[]), warn user about syntax errors
        if len(search) > 0 and search[-1] in "|[]":
            splunk.search.Parser.parseSearch(search, hostPath=urlhost, sessionKey=sessionKey)
    except Exception, e:
        # import traceback # traceback.print_exc()
        msg = e.message
        logger.debug(e)
        ignoredParseErrors = ['Unknown search operation', 'list index out of range']
        for ignoredError in ignoredParseErrors:
            if ignoredError in msg:
                break
        else:
            import re
            error = msg
            # For example \"inputs\"         --> For example "inputs"
            error = error.replace('\\"', '"')
            # Error in 'SearchOperator:rex': --> In rex:
            # Error in 'rex':                --> In rex:
            # Error in 'rex' command:        --> In rex:
            error = re.sub("Error in '(\\w+:)?(\\w+)'(?: command)?:", "In \\2:", error)            
    return error

def getEasterEgg(search):
    imagetexttemplate = '<center><table width="100%%" height="100%%"><tr><td align=center background="%s"><font size="30px" color="black">%s</font></td></tr></table></center>'

    credits = ( 'searchhelper/creditframe.html', '')
    eastermapping = { 'zork': ( 'searchhelper/rawhtml.html', '<iframe src ="http://thcnet.net/zork/index.php" width="100%%" height="100%%"><p>Go West, Young Man.</p></iframe>'),
                      'boss': ('searchhelper/rawhtml.html',
                               imagetexttemplate % ('http://img.skitch.com/20090612-kd2i9ru8e164tre4y2a9h2u3y.jpg',
                                                    '<blink>Boss is the <u>best</u>! Top 25 CTO! </blink>')),
                      'ssb' : ('searchhelper/rawhtml.html',
                               imagetexttemplate % ('http://img.skitch.com/20090612-8cc11y7kpts6ms6serry316pk2.jpg',
                                                    "<br/><br/><font color='red'><b>Bomberman, FTW</b></font><br/><br/>")),
                      'credits' : credits,
                      'about' : credits
                      }
    result = None
    comm = shutils.getLastCommand(search, None)
    if comm != None:
        lastcommand,arg = comm
        result = eastermapping.get(lastcommand, None)
    return result

    
    
    
if __name__ == '__main__':

    def unit_test():
        import sys

        sessionKey = splunk.auth.getSessionKey('admin', 'changeme')
        try:
            cherrypy.session['sessionKey'] = sessionKey
        except AttributeError:
            setattr(cherrypy, 'session', {})
            cherrypy.session['sessionKey'] = sessionKey
        cherrypy.session['user'] = { 'name': 'admin' }
        cherrypy.config['module_dir'] = '/'
        cherrypy.config['build_number'] = '123'
        cherrypy.request.lang = 'en-US'

        # roflcon
        class elvis:
            def ugettext(self, msg):
                return msg
        cherrypy.request.t = elvis()
        # END roflcon

        shc = SearchHelperController()

        argc = len(sys.argv)
        if argc == 2:
            query = sys.argv[1]
            print shc.index(search=query, snippet=True, minimize=False)
        else:
            print "Usage: %s 'search'" % sys.argv[0]


    unit_test()
