import cherrypy
import splunk.entity
import logging
import cherrypy.process.plugins as plugins
import time

logger = logging.getLogger('splunk.appserver.mrsparkle.cached')


# define the duration (in seconds) to cache the function output
MAX_CACHE_AGE = 5


class memoized(object):
    '''
    Decorator that caches a function's return value each time it is called.
    If called later with the same arguments, the cached value is returned, and
    not re-evaluated.
    
    The lifetime of the cache is defined in MAX_CACHE_AGE.
    '''

    def __init__(self, func, cache_age=MAX_CACHE_AGE):
        self.func = func
        self.cache = {}
        self.timers = {}
        self.cache_age = cache_age
        
        logger.debug('Initialized memoize on function=%s max_age=%s' % (self.func.__name__, self.cache_age))
        

    def __call__(self, *args, **kwargs):

        # create an unmutable cache key from sessionKey, positional and keyword args
        cachekey = [cherrypy.session.get('sessionKey'), args]
        for k in kwargs:
            cachekey.append((unicode(k), unicode(kwargs[k])))
        cachekey = tuple(cachekey)
        
        # check if we need to expire
        try:
            t = self.timers[cachekey]
            if time.time() - t > self.cache_age:
                del self.cache[cachekey]
        except KeyError:
            pass
        except Exception, e:
            logger.exception(e)
                
        # lookup in cache
        try:
            return self.cache[cachekey]
        except KeyError:
            self.cache[cachekey] = value = self.func(*args, **kwargs)
            self.timers[cachekey] = time.time()
            logger.debug('MEMOIZE: inserted cache: %s' % str(cachekey))
            return value
        except TypeError:
            # uncachable -- for instance, passing a list as an argument.
            # Better to not cache than to blow up entirely.
            return self.func(*args, **kwargs)


    def __repr__(self):
        """Return the function's docstring."""
        return self.func.__doc__
        

@memoized
def getEntity(*args, **kwargs):
    '''
    Memoized version of splunk.entity.getEntity
    '''
    return splunk.entity.getEntity(*args, **kwargs)
    
    
@memoized    
def getEntities(*args, **kwargs):
    '''
    Memoized version of splunk.entity.getEntities
    '''
    return splunk.entity.getEntities(*args, **kwargs)
    

@memoized
def getLocalAppData():
    '''
    Memoized version of appserver.lib.apps
    '''
    return apps.getLocalApps()

