# override CherryPy's default log manager to redirect it's internal messages to splunk's logging infrastructure
import logging
import cherrypy
from cherrypy import _cplogging
from cherrypy import _cperror

class SplunkedLogManager(_cplogging.LogManager):
    """ Subclass LogManager to pass access and error messages to splunks logging """
    # make sure this gets set 
    logger = None

    def __call__(self, *args, **kwargs):
        """Write to the error log."""
        return self.error(*args, **kwargs)

    def error(self, msg='', context='', severity=logging.INFO, traceback=False):
        """Write to the splunk error log."""
        if traceback:
            # if it's a traceback, format it and print it out to stdout
            msg += _cperror.format_exc()
            #print msg       <- Because this thing throws IOError: [Errno 9] Bad file descriptor
        self.logger.error(' '.join((self.time(), context, msg)))
