#
# Cherrypy boot-time helper methods
#

import splunk.entity, splunk.util
import cherrypy
import logging

logger = logging.getLogger('splunk.appserver.lib.startup')

VERSION_INIT_KEY = 'is_version_initted'

def initVersionInfo(force=False):
    '''
    Initializes splunk product version and license info by asking splunkd.
    Returns true if successful, false otherwise
    '''
    
    if cherrypy.config.get(VERSION_INIT_KEY) and not force:
        return True
        
    logger.debug('Checking for product version information')

    buildNumber = '000'
    versionLabel = 'UNKNOWN_VERSION'
    isTrialLicense = True
    isFreeLicense = False
    licenseState = 'OK'
    
    try:
        serverInfo = splunk.entity.getEntity('/server', 'info', namespace=None, owner='anon')
        buildNumber = int(serverInfo.get('build'))
        versionLabel = serverInfo.get('version')
        isTrialLicense = splunk.util.normalizeBoolean(serverInfo.get('isTrial'))
        isFreeLicense = splunk.util.normalizeBoolean(serverInfo.get('isFree'))
        licenseState = serverInfo.get('licenseState')
        cherrypy.config[VERSION_INIT_KEY] = True
                
    except Exception, e:
        logger.error('Unable to read in product version information; %s' % e)

    # always set something
    cherrypy.config['build_number'] = buildNumber
    cherrypy.config['version_label'] = versionLabel
    cherrypy.config['is_free_license'] = isFreeLicense
    cherrypy.config['is_trial_license'] = isTrialLicense
    cherrypy.config['license_state'] = licenseState
    
    logger.info(
        'Splunk appserver version=%(version_label)s build=%(build_number)s isFree=%(is_free_license)s isTrial=%(is_trial_license)s' % cherrypy.config
    )
    