# Copyright (C) 2005-2010 Splunk Inc.  All Rights Reserved.  Version 4.0
import logging as logger
import splunk.clilib.cli_common as comm
import validate
import glob, shutil, os
from control_exceptions import *

optionMapFile = {
    "hostname"      : "host",
    "index"         : "index",
    "sourcetype"    : "sourceType"
}

DELETE_FILE = "delete"


def addFile(args, fromCLI):
  """
  Index a single file by copying it into the spool directory.
  """
  # separate from the global one because of .splunk files
  paramsReq = ("source", )
  paramsOpt = (DELETE_FILE, "hostname", "index", "sourcetype")

  comm.validateArgs(paramsReq, paramsOpt, args)

  # no errors

  returnDict = {}
  spoolFunc  = shutil.copy # called later
  if DELETE_FILE in args.keys():
    deleteVal = str(args.pop(DELETE_FILE)).lower()
    if not deleteVal in ("true", "false"):
      raise ArgError, "'%s' is not an invalid value for '%s'." % (deleteVal, DELETE_FILE)
    if "true" == deleteVal: # it's either 'true' or 'false' at this point
      spoolFunc = shutil.move

  fileList = glob.glob(args["source"]) # glob (glawb): 1. verb: expand shell wildcards [and return matches].

  if 0 == len(fileList):
    raise ArgError, "The pattern '%s' does not match any existing files." % args["source"]

  # phase 1 - for anything in the file list, first do verification.  if anything is wrong, fail before doing anything.
  logger.info("Checking file validity...")
  for path in fileList:
    path = os.path.abspath(path)
    path = os.path.normpath(path)
    if fromCLI:
      validate.checkPath("source", path)

    try:
      #non invokeAPI version of this function...SPL-21979
      comm.checkValidFileEAICLI(path)
    except IOError:
      raise FilePath, "The path '%s' is not a valid file.  Please make sure the path exists, is accessible by the user, and is a file." % path

  # phase 2 - now that we know there should be no errors, actually spool all the files.
  for path in fileList:
    logger.info("Adding file to be indexed: %s" % path)
    spoolDir = os.path.join(comm.splunk_home, "var", "spool", "splunk")

    # write out the .splunk file first
    basename = path.split(os.path.sep)[-1]
    splunkFileName = os.path.join(comm.uploadDir(), "%s.splunk" % basename)
    splunkFile = open(splunkFileName, "w")
    splunkFile.writelines("<metainfo>\n")
    splunkFile.writelines("\t<source>%s</source>\n" % os.path.join(spoolDir, basename))
    for param in paramsOpt:
      if param in args.keys():
        splunkFile.writelines("\t<%s>%s</%s>\n" % (optionMapFile[param], args[param], optionMapFile[param]))
    splunkFile.writelines("</metainfo>\n")
    splunkFile.close()

    # now drop the actual .splunk and data file in
    #python2.6 will thow an error if u try and move a file when the dst already exists
    try:
       shutil.move(splunkFileName, spoolDir)
    except shutil.Error:
       os.remove(os.path.join(spoolDir, "%s.splunk" % basename)) 
       shutil.move(splunkFileName, spoolDir)
      
    try:
       spoolFunc(path, spoolDir)
    except shutil.Error:
       os.remove(os.path.join(spoolDir, "%s" % basename)) 
       spoolFunc(path, spoolDir)

  return returnDict
