import sys,glob,re,os
from xml.sax import saxutils
import splunk.mining.dcutils as dcutils

DIR_PATH = os.path.join('wiki', 'readme')
OUTPUT_PATH = os.path.join(DIR_PATH, '%s.conf.wiki')

dirchoices = {
    'madonna':"P4/splunk/branches/madonna/cfg/bundles/README/",
    '32': "../../../../../3.2/cfg/bundles/README/",
}


WIKI_CATEGORIES = ['4.0','4.0beta','4.0.1','4.0.2','4.0.3','4.0.4','4.0.5','4.0.6','4.0.7','4.0.8','4.0.9']
WIKIFORMATCATEGORIES = "[[Category:V:%s]]"

WIKI_TEMPLATE = """
=%(filename)s.conf=
The following are the spec and example files for %(filename)s.conf.

==%(filename)s.conf.spec==
<pre>
%(spec)s
</pre>

==%(filename)s.conf.example==
<pre>
%(example)s
</pre>

%(categories)s
"""

def writeWikiFile(filename, spectext, exampletext):
    
    categoriestext = ""
    for category in WIKI_CATEGORIES:  # should add to searchbnf
        categoriestext += WIKIFORMATCATEGORIES % category

    form = {}
    form['filename'] = filename
    form['spec'] = saxutils.escape(spectext)
    form['example'] = saxutils.escape(exampletext)
    form['categories'] = categoriestext
    text = WIKI_TEMPLATE % form
    filename = OUTPUT_PATH % filename
    dcutils.writeText(filename, text)

def writeWiki(dirname):
    # delete old specs
    os.system('rm -r %s' % DIR_PATH)
    if not os.path.isdir(DIR_PATH): os.makedirs(DIR_PATH)

    specs = glob.glob(dirname + "*spec*")
    # for each spec file, get matching example file
    for spec in specs:
        if spec.endswith("~") or spec.endswith("#"):
            continue
        try:
            root = re.search("README/(.*?)\.(?:conf|example|spec).*", spec).group(1)
        except:
            print "Failed to match conf/example/spec for:", spec
            continue
        exname1 = dirname + root + ".*example.in"
        exname2 = dirname + root + ".*example"
        examples = glob.glob(exname1)
        examples.extend(glob.glob(exname2))
        if len(examples) == 0:
            print "Missing example files for %s" % spec
            exampletext = "No example"
        else:
            if len(examples) > 1:
                print "Two example files.  Using %s" % exname2
            exampletext = dcutils.readText(examples[0])
        exampletext = exampletext.replace("@OsDirSep@","/")
        spectext = dcutils.readText(spec)
        spectext = spectext.replace("@OsDirSep@","/")
        writeWikiFile(root, spectext, exampletext)


def printUsage():
    print "Usage: python %s %s" % (sys.argv[0], dirchoices.keys())
    
if __name__ == '__main__':
    argc = len(sys.argv)
    argv = sys.argv
    if argc == 2:
        dirname = argv[1]
        if not dirname in dirchoices:
            printUsage()
        else:
            writeWiki(dirchoices[dirname])
    else:
        printUsage()
