import re,sys,os
import splunk.searchhelp.utils as utils
import splunk.searchhelp.describer as describer
import subprocess
# splunk isn't running.  get it the manly way
import splunk.mining.conf as conf

def validateCommands(onlypublic=True):
    msg = ""
    errors = []
    #stanzas = conf.ConfParser.parse(os.environ['SPLUNK_HOME'] + "/etc/system/default/searchbnf.conf", True, errors)
    path = os.path.join('..','..','..','cfg','bundles','default','searchbnf.conf')
    stanzas = conf.ConfParser.parse(path, True, errors)
    if len(stanzas) < 2:
        return "ERROR: stanzas are empty"
    for error in errors:
        msg  += "ERROR: %s\n" % error
    stanzanames = stanzas.keys()
    stanzanames.sort()
    for key in stanzanames:
        val = stanzas[key]
        if key.endswith("-command") and (not onlypublic or utils.isPublic(val)):
            name = key[0:-8]
            out = validateCommand(stanzas, name)
            if len(out) > 0:
                msg += out + "\n"            

    return msg

okErrors = ["does not exist",
"not found",
"valid session",
"authorization",
"authentication",
"Unable to access",
"Unable to learn",
"Permission denied",
"could not be opened",
"Invalid lookup table",
"not find any artifacts",
"does not exist",
"No record",
"Unable to find",
"Edit limits.conf to change limits.",
"Cannot find job",
"Invalid events",
"No qualifying fields exist",
"Index out of range",
"Did not find value",
"key found",
"cannot find the path specified",
"No connection could be made",
"while sending mail",
"did not return events in descending time order",   # <----- HACK TO SHUT UP PROBLEM WITH XPATH THAT IS PROBABLY AN EXTERNPROCESSOR BUG? 
"No events or fields to analyze"]

def validateExample(example):
    if "appendcols" in example: # ahh, too slow
        return ''
    originalExample = example
    example = example.strip()

    if example.startswith("..."):
        example = "search * | head 10 | where 0==1 " + example[3:]
    elif not example.startswith("|"):
        example = "search " + example

    example = 'dispatch [%s]' % example
    print "Validating example: %s" % example                

    output = subprocess.Popen(["searchtest", example], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
    msg = ''
    for io in output:
        if "ERROR" in io:
            for ok in okErrors:
                if ok in io:
                    break
            else:
                print "\tPROBLEM!"
                msg += "Example: %s\nERROR:%s\n" % (originalExample, io)
    return msg

def validateCommand(stanzas, command):

    maintainer = "unknown"
    try:
        msg = ""        
        stanzaname   = command + "-command"
        stanza       = stanzas[stanzaname]
        msg +=         syntaxErrors(stanzas, stanza, True)
        msg +=         syntaxErrors(stanzas, stanza, False)
        maintainer = stanza.get('maintainer','unknown')
        desc = stanza.get('description','')
    
        requiredfields = ["description", "syntax", "usage", 'maintainer', 'category', 'related', 'tags']
        optionalfields = ["aliases", "related", 'appears-in', 'tags', 'alias', 'note', 'simplesyntax', 'generating', 'shortdesc', 'supports-multivalue', 'default']
        knownfields    = requiredfields + optionalfields

        badfirstterms = re.findall("\n(\S+)", desc)
        if len(badfirstterms) > 0:
            msg += 'ERROR: description is multiline but is not indented, indicating there may be an extra "\" near %s\n' % badfirstterms

        for r in requiredfields:
            if not r in stanza:
                if r not in ['related', 'category']:
                    msg += "ERROR: "
                msg += "%s not found\n" % r
        description = stanza.get("description", "")
        if len(description) > 100 and "shortdesc" not in stanza:
            msg += "ERROR: need shortdesc field because description more than 100 chars (%s)\n" % len(description)
        validateExamples = True
        if validateExamples:
            sawExample = False
            for attr in stanza.keys():
                if attr.startswith("example"):
                    msg += validateExample(stanza[attr])
                    sawExample = True
                    suffix = attr[len("example"):]
                    commentid = "comment" + suffix
                    if not commentid in stanza:
                        msg += "%s for doesn't have a %s\n" % (attr, commentid)
                elif attr.startswith('comment'):
                    suffix = attr[len("comment"):]
                    exampleid = "example" + suffix
                    if not exampleid in stanza:
                        msg += "%s for doesn't have as %s\n" % (attr, exampleid)
                elif attr not in knownfields:
                    msg += "unknown field %s\n" % attr
            if not sawExample:
                msg += "No examples\n"
        if len(msg) > 0:
            msg = "%s (%s) \n" % (command,maintainer) + msg
            msg = msg.replace('\n','\n\t')
    except Exception, e:
            msg += "%s (%s) Error -- %s\n" % (command, maintainer, e)
            import traceback
            msg += traceback.format_exc()
    return msg


leaf_datatypes = ["sp", "field-and-value", "field-and-value-list",
                  "no-ws", "int", "num", "wc-str", "string", "bool", "field",
                  "field-list", "sqlite-expression", "wc-field", "wc-field-list",
                  "wc-string", "tag", "tag-list", "filename", "true", "false", "term",
                  "sed-expression", "search-pipeline", 'subsearch', 'char',
                  'eval-func-args', 'quoted-search-expression'
                  ]

def syntaxErrors(stanzas, stanza, simplesyntax = True):
    syntax = describer.getValue(stanza,"syntax")
    if simplesyntax and "simplesyntax" in stanza:
        syntax = stanza["simplesyntax"]
    msg = ""
    terms = re.findall("<([a-zA-Z][a-zA-Z0-9_/:=-]*)>", syntax)
    REGEX = re.compile("([a-zA-Z0-9_/-]+)(:[^=]+)?(=.*)?")

    try:
        keys = stanzas.stanzas
    except:
        keys = stanzas.keys()
    
    for term in terms:
        breakdown = REGEX.findall(term)
        datatype   = breakdown[0][0]
        variable   = breakdown[0][1]
        defaultval = breakdown[0][2]
        #print "!datatype %s variable %s defaultval %s " % (datatype, variable, defaultval)

        if datatype in leaf_datatypes or datatype.endswith("_syntax") or datatype.endswith("expression"):
            continue
        if datatype not in keys:
            msg += "Ignoring undefined stanza: %s\n" % datatype
            continue
        substanza = stanzas[datatype]
        if substanza == stanza:
            msg += "Ignoring recursive definition: %s\n" % datatype
            continue
        msg += syntaxErrors(stanzas, substanza, simplesyntax)
    return msg


    
if __name__ == '__main__':
    try:
        msg = validateCommands()
    except Exception, e:
        msg = "ERROR: %s" % e
    print
    print msg
    if "ERROR:" in msg:
        print "FAILED"
        exit(1)
    exit(0)
