import os
import re
import json
import urllib
import cherrypy
import splunk
from cherrypy.test import test, helper
from splunk.appserver.mrsparkle.controllers import parser, debug
from splunk.appserver.mrsparkle.lib import i18n


def setup_server():
    cfg = {
        '/': { 
            'request.dispatch' : i18n.I18NDispatcher(),
            'root_endpoint': '/'
         }
    }
    p = parser.ParserController()
    cherrypy.config.update({'environment': 'test_suite'})
    cherrypy.tree.mount(p, '', cfg)

def exitingAssert(cond):
    if not cond:
        print "FAILED ASSERT"
        import traceback
        traceback.print_stack()
        exit(1)
    


class ParserControllerTest(helper.CPWebCase):

    def setUp(self):
        self.applyEndpoint = '/en-US/parse'
        self.decomposeEndpoint = '/en-US/decompose'

        # Spoof a session
        sessionKey = splunk.auth.getSessionKey('admin', 'changeme')
        setattr(cherrypy, 'session', {})
        cherrypy.session['sessionKey'] = sessionKey

    def tearDown(self):
        # don't leave our fake session around else it'll screw up other tests
        # that use real sessions
        del cherrypy.session

    def _applyIntents(self, q, intents):
        if intents == "[]":
            return
        args = "q=%s&intentions=%s" % ( urllib.quote(q), intents )
        self.getPage(self.applyEndpoint, method='POST', body=args)

    def _decompose(self, search):
        args = "q=%s" % urllib.quote(search)
        self.getPage(self.decomposeEndpoint, method='POST', body=args)


    def _runApplyCases(self, cases):
        for case in cases:
            self._applyIntents( case['search'], json.dumps(case['intents']) )
            self.assertStatus(200)
            body = json.loads(self.body)

            if body['search'] == case['expected']:
                print body['search'], "\t==\t" , case['expected']
            else:
                print body['search'], "\t!=\t" , case['expected']

            exitingAssert(body['search'] == case['expected'])

    def test_shouldExceptOut(self):
        '''
            Test that these searches fail out with exceptions
        '''
        searches = [
            'search eventtype="jira-snap" sprint="Sprint 8" type!=documentation | eval istrue=t | dedup field | eval is_resolved="UNRESOLVED"',
            'search eventtype="jira-snap" sprint="Sprint 8" type!=documentation | dedup key | dedup else',
            'search referer="-" NOT gif | head 10 | stats count | fields count | sort -count',
            'search referer=- NOT gif | head 10 | stats count | fields count',
            'search eventtype=download OR eventtype=pageview | transaction fields=cookie | eval entrypath = mvindex(uri,0) + ":" + mvindex(uri,1) + ":" + mvindex(uri,2) | timechart count(_raw) by entrypath',
            'search index="sampledata" | search "http"',
            'search eventtype=download | stats count by clientip',
        ]
        for search in searches:
            body = None
            self.body = None
            messages = []
            self._decompose( search )
            self.assertStatus(200)
            body = json.loads(self.body)
            exitingAssert(not body.get('success') )

    def test_roundtrip(self):
        '''
            Test roundtrip intention parsing.
        '''

        searches = [
            # commented out until timechart gives better arguments (currently doesn't return usenull, for example
            #'search source="syslog" | timechart span=55m count by twikipage usenull=f ',
            #'search source="syslog" | timechart span=1m bins=60 count by twikipage',
            'search foo bar baz',
            'search foo bar baz | timechart avg(quux)',
            'search source="syslog" | timechart bins=12 count by twikipage',
            'search source="syslog" | timechart span=1m count by twikipage',
            'search source="syslog" | timechart span="1m" count by twikipage',
            'search source="syslog" | timechart span="55m" count by twikipage',
            'search source="syslog" | timechart span=55m count by twikipage',
            'search source="syslog" | chart count by twikipage,twikiuser',
            'search index=_internal group=per_index_thruput | timechart per_second(kb)',
            '| crawl | search NOT *personal*',
            'search sourcetype=access_combined | timechart dc(clientip) by clientip',
            '| windbag | search "<random>" these words come after',
            '| windbag | search \\"<random>\\" these words come after',
            '| windbag | search \\"<random these words come after',
            '| windbag | search \<',
            '| windbag | search <',
            '| windbag | search "<"',
            'search "<random>" these words come after',
            'search \\"<random>\\" these words come after',
            'search \\"<random these words come after',
            'search \<',
            'search "<"',
            'search <',
            'search "next=/../.."',
            'search sourcetype="linux_secure" nate Feb 11 11:47:42 spacecake sshd\[14113\]: Accepted publickey for nate from 10.1.1.203 port 59130 ssh2',
            '| windbag rowcount=100000 | search lang="Albanian" OR lang="Hindi" OR lang="Hawaiian" | timechart count by lang',
            'search eventtype="jira-snap" resolution="UNRESOLVED" | chart count by assigneeID,sprint',
            #TODO: respect order of clauses in de/composition: 'search index=main sourcetype=syslog | rex "(?i)\.apple\.(?P<FIELDNAME>\w%2B)\[" | fields FIELDNAME | dedup FIELDNAME',
            #'search error | sort',
            #'search error | fields',
            #'search eventtype="jira-snap" sprint="Sprint 8" type!=documentation | dedup key | eval is_resolved = if(resolution="UNRESOLVED", 0, 1) | chart sum(eval(if(is_resolved = 0, 1, 0))) as unresolved_count sum(eval(if(is_resolved = 0, exp_effort, 0))) as unresolved_effort sum(eval(if(is_resolved = 0 AND isnull(exp_effort), 1, 0))) as unresolved_uncosted sum(eval(if(is_resolved = 0, 0, 1))) as resolved_count sum(eval(if(is_resolved = 0, 0, exp_effort))) as resolved_effort by assigneeID'
            'search * | timechart count by clientip',
            'search badness%3E100',
            'search source = *apache*',
            'search source>10',
        ]

        failFlag = 0
        quoteMismatches = 0
        extraStars = 0
        for search in searches:
            body = None
            self.body = None
            messages = []
            self._decompose( search )
            self.assertStatus(200)
            try:
                body = json.loads(self.body)
            except Exception, e:
                failFlag = failFlag + 1
                print "!"*100
                print "\n*PARSE EXCEPTION"
                print "\tSearch: %s" % search
                print "\tResponse body %s" % self.body
                exitingAssert(False)

            
            if body.has_key('messages'):
                failFlag = failFlag + 1
                print "!"*100
                print "Parser failure on", search
                print body['messages']
                exitingAssert(False)
                continue

            self._applyIntents(body['search'], json.dumps(body['intentions']) )
            try:
                newbody = json.loads(self.body)
            except Exception:
                newbody = None
                print "Attempted search: %s\n" % search

            if sorted(newbody['search'].split() ) != sorted( search.split() ):
                sortedSearch   = sorted( [ urllib.unquote(re.sub(r'"', '', x)) for x in newbody['search'].split()] )
                sortedExpected = sorted( [ urllib.unquote(re.sub(r'"', '', y)) for y in search.split() ] )

                star = '*'
                if star in sortedExpected and not star in sortedSearch:
                    sortedExpected.remove(star)
                    extraStars = extraStars + 1
                    messages.append("WARN: Extra * removed")

                if sortedSearch == sortedExpected:
                    quoteMismatches = quoteMismatches + 1
                    ## messages.append("WARN: Quotes normalized")
                    ## print "; ".join(messages)
                    ## print '\tOriginal\t %s' % search
                    ## print '\tComposed\t %s' % newbody['search']
                    continue

                failFlag = failFlag + 1
                print "!"*180
                print "\n** ERROR: Mismatch"
                print '\tOriginal\t %s' % search
                print '\tComposed\t %s' % newbody['search']
                print '\tOriginal Sorted\t %s' % sorted(search.split())
                print '\tComposed Sorted\t %s' % sorted(newbody['search'].split())
                print '\t',body['intentions'], "--", body['search']
                exitingAssert(False)
                continue

        print
        if failFlag: print "%s roundtrips failed." % failFlag
        if quoteMismatches: print "%s searches with quotes normalized." % quoteMismatches
        exitingAssert(not failFlag)

    def test_alternateTopLimitSyntax(self):
        '''
            Test the alternate way to define a limit in top/rare commands
        '''
        print "\n"
        equivSearches = ['search * | top 100 host', 'search * | top limit=100 host', 'search * | top host limit=100']
        equivDecomps = []

        for i,search in enumerate(equivSearches):
            self._decompose(search)
            self.assertStatus(200)

            try:
                equivDecomps.append( json.loads(self.body) )
            except Exception, e:
                print "\n*Decompose Exception"
                print "\tSearch: %s" % search
                print "\tResponse body: %s" % self.body
                print "\tException    : %s" % e

        for decomp in equivDecomps[1:]:
            if not str(decomp) == str(equivDecomps[1]):
                print "\n*MISMATCH"
                print "\tResponse body: %s" % decomp
                print "\tExpected     : %s" % equivDecomps[0]
                exitingAssert(str(decomp) == str(equivDecomps[1]))

    def test_addtermgt(self):
        ''' SPL-21160 '''
        q = 'search index=_internal metrics source="*metrics.log" NOT debug'
        intents = '[{"name":"addterm","arg":{"group":"per_sourcetype_thruput"}},{"name":"addterm","arg":{"series":"audittrail" }},{"name":"addtermgt","arg":{"eps":"126"}}]'
        self._applyIntents(q, intents)
        resp = json.loads(self.body)
        for k in ['clauses','search']:
            exitingAssert(k in resp.keys())

    def test_mixedintentions(self):
        '''
            Test string replacement mixed with regular intents.
        '''
        return True

        cases = [
            {
                'search'    : 'search foo bar $foo$',
                'intents'   : [{'name':'stringreplace', 'arg': {'foo':{'value':'hello world'} } }, {'name':'addterm', 'arg':'baz'}],
                'expected'  : 'search foo bar hello world baz'
            },

            # this will fail due to startTimeTerm having no 'transform' method
            {
                'search'    : 'search * | eval Daily_Indexing_Volume_in_MBs = todaysBytesIndexed/1024/1024',
                'intents'   : [ {"name":"startTimeTerm","arg":{"unit":"days","quantity":"7"}},
                                {"name":"addterm","arg":{"index":"_internal"}},
                                {"name":"addterm","arg":"todaysBytesIndexed"},
                                {"name":"addterm","arg":"LicenseManager-Audit"},
                                {"name":"addterm","arg":{"NOT+source":"*web_service.log"}},
                                {"name":"addterm","arg":{"NOT+source":"*web_access.log"}},
                                {"name":"addterm","arg":{"readlevel":"2"}},
                                {"name":"plot","arg":{"splitby":"host","fields":[["avg","Daily_Indexing_Volume_in_MBs"]],"normalizeOutliers":False,"suppressNull":False,"mode":"timechart","suppressOther":False,"defer":"False"}},
                                {"name":"plot","arg":{"splitby":"host","fields":[["avg","Daily_Indexing_Volume_in_MBs"]],"normalizeOutliers":False,"suppressNull":False,"mode":"timechart","suppressOther":False,"defer":"False"}}
                                ],
                'expected'  : 'search startdaysago=7 NOT source="*web_access.log" index="_internal" todaysBytesIndexed LicenseManager-Audit | eval Daily_Indexing_Volume_in_MBs = todaysBytesIndexed/1024/1024 | timechart avg(Daily_Indexing_Volume_in_MBs) by host'
            },
        ]

        self._runApplyCases(cases)

    def test_stringreplacement(self):
        '''
            Test string replacement intents.
        '''
        return True
 
        cases = [
            { 'search'    : 'search foo bar $foo$',
                'intents'   : [{'name':'stringreplace', 'arg': {'foo':{'value':'hello world'} } }],
                'expected'  : 'search foo bar hello world' },
            { 'search'    : 'search foo bar $foo$',
                'intents'   : [{'name':'stringreplace', 'arg': {'foo':{'value':'hello world', 'prefix':'greeting='} } }],
                'expected'  : 'search foo bar greeting=hello world' },
            { 'search'    : 'search foo bar $foo$',
                'intents'   : [{'name':'stringreplace', 'arg': {'foo':{'value':'hello world', 'prefix':'greeting=', 'suffix':'shamwow'} } }],
                'expected'  : 'search foo bar greeting=hello worldshamwow' },
            { 'search'    : 'search foo bar $foo$',
                'intents'   : [{'name':'stringreplace', 'arg': {'foo':{'default':'fallbackonme'} } }],
                'expected'  : 'search foo bar fallbackonme' },
            { 'search'    : 'search foo bar $foo$',
                'intents'   : [{'name':'stringreplace', 'arg': {'foo':{'default':'defaultFilled', 'fillOnEmpty':True} } }],
                'expected'  : 'search foo bar defaultFilled' },
            { 'search'    : 'search foo bar $foo$',
                'intents'   : [{'name':'stringreplace', 'arg': {'foo':{'default':'defaultFilled', 'fillOnEmpty':False} } }],
                'expected'  : 'search foo bar defaultFilled' },
            { 'search'    : 'search foo bar $foo$',
                'intents'   : [{'name':'stringreplace', 'arg': {'foo':{'fillOnEmpty':False} } }],
                'expected'  : 'search foo bar $foo$' },
            { 'search'    : 'search foo bar $foo$',
                'intents'   : [{'name':'stringreplace', 'arg': {'foo':{'fillOnEmpty':True} } }],
                'expected'  : 'search foo bar' },
            { 'search'    : 'search $Search Tokens$',
                'intents'   : [{'name':'stringreplace', 'arg': {'Search Tokens':{'value': 'some search tokens'} } }],
                'expected'  : 'search some search tokens' },
            { 'search'    : 'search foo bar $foo$',
                'intents'   : [{'name':'stringreplace', 'arg': {'foo':{} } }],
                'expected'  : 'search foo bar $foo$' },
        ]
 
        self._runApplyCases(cases)

        
if __name__ == '__main__':
    setup_server()
    helper.testmain()
