import splunk.search.Parser as Parser
import splunk.search.Transformer as Transformer
import splunk.auth
import copy

splunk.auth.getSessionKey('admin', 'changeme')

def applyIntention(expected, search, name, arg, flags=None):
    s = Parser.parseSearch(search)
    sbefore = str(s)
    s = Transformer.applyIntention(s, name, arg, flags)
    if normalize(s, False) != normalize(expected, False):
        print "\n"
        print "Search:", search
        print "\tApplying:", name, arg, flags
        print "\tAfter:   ", s
        print "\tExpected:", expected
        print "\tParsed:  ", str(sbefore)
        print "FAIL", "!"*80
        print "\n"
        exit(1)
        return s
    #print ":) Success! Applied intention ", name, arg, flags, "to", search, "-->", s
    return None

def normalize(v, normalizeQuotes=True):
    v = str(v).strip().replace(' ', '')
    if normalizeQuotes:
         v = v.replace('"', '')
    return v


def resurrect(search, normalizeQuotes=True, acceptableRecomposition=None):
    if not search.strip().startswith("|"):
        search = "search " + search
    p = Parser.parseSearch(search)
    try:
        s,i = Transformer.decomposeSearch(copy.deepcopy(p))
    except Exception, e:
        if normalize(search, normalizeQuotes) != normalize(p, normalizeQuotes):
            print "\n\n"
            print "!Cannot decompose:", e
            print "\tSearch: ", search 
            print "\tParsed: ", str(p)
            print "FAIL", "!"*80
            print "\n\n"
            exit(1)
            return search
        #print ":) Success! Cannot decompose, but parse is same:", e,"Search: ", search 
        return None
    recomposed = copy.deepcopy(s)
    for intent in copy.deepcopy(i): 
        name = intent['name']
        arg  = intent['arg']
        flags = intent.get('flags', None)
        #print '\tApplying Name: %s \nArg: %s' % (name, arg)
        recomposed = Transformer.applyIntention(recomposed, name, arg, flags)
        #print '\tRecomposed: %s' % recomposed
    if normalize(recomposed, normalizeQuotes) != normalize(search, normalizeQuotes) and normalize(recomposed, normalizeQuotes) != normalize(acceptableRecomposition, normalizeQuotes):
        print "\n"
        print "Search:         %s" % search
        print '\tRecomp: %s' % recomposed
        if acceptableRecomposition != None:
            print '\tAcceptable: %s' % acceptableRecomposition
        print "\tDCSrch: %s" % s
        print "\tIntent: %s" % i
        print "!"*80
        print "\n\n"
        return recomposed
    #print ":) Success! Search: %s" % search
    return None


def test():

    # SPL-25738 punct values not quote escaped on interior quotes
    
    val = applyIntention('search punct="..._-_-_[//:::]_\\"_/-//@//._/.\\"___\\"://:/-///?=.&=\\"_"', 'search *', 'toggleterm', {"punct":"..._-_-_[//:::]_\"_/-//@//._/.\"___\"://:/-///?=.&=\"_"})
    
    #     SPL-23211 INTENTIONS: error placing quotes with values in parens on
    #     the main dashboard, search for NOT (source=foo) the search gets
    #     converted to: NOT (source="foo)" 
    
    val = resurrect("NOT (source=foo)")
    
    
    #     SPL-22322 INTENTIONS: NOT included within "" when query gets a new look after resurrection 
    #     sourcetype="access_combined" source="foo" NOT 10.1.1.43 | tail 5 
    #     sourcetype="access_combined" source="foo" "NOT 10.1.1.43" | tail 5
    
    val = resurrect('sourcetype="access_combined" source="foo" NOT 10.1.1.43 | tail 5 ')
    
    
    #     SPL-21760 INTENTIONS: Report builder hjorks my search
    #     The issue there is '| rex "somestring"' becomes '| "somestring"', losing the rex.
    
    val = resurrect('| rex "(?P<foo>somestring)"')
    
    #     SPL-21706 BETA: don't mangle my search
    #     referer="-" NOT gif | head 10 | stats count | fields count | sort -count
    #     becomes 
    #     NOT gif | head 10 | sort - count | search referer="-" | fields count | stats count
    
    val = resurrect('referer="-" NOT gif | head 10 | stats count | fields count | sort -count')
    
    #     SPL-21639 BETA: reloading complex search job mangles search string
    #     quickdraw | rex field=referer "\/(?<licensetype>(pro|free))\/(?<productversion>[^\/]+)\/[^\/]+\/(?<trial_or_prod>[^\/]+)" | eval licensetype = licensetype + trial_or_prod | stats count by licensetype, clientip, cookie | stats sum(count) as sessions, dc(cookie) as cookie_count by clientip, licensetype | stats sum(sessions) as sessions, avg(cookie_count) as avg_cookie_count, dc(clientip) as distinct_ips by licensetype | eval avg_sessions_per_ip=sessions/distinct_ips
    #     is becoming
    #     quickdraw | stats | rex field=referer "\/(?<licensetype>(pro|free))\/(?<productversion>[^\/]+)\/[^\/]+\/(?<trial_or_prod>[^\/]+)" | eval licensetype = licensetype + trial_or_prod | eval avg_sessions_per_ip = sessions/distinct_ips | stats count by licensetype clientip cookie
    
    val = resurrect('quickdraw | rex field=referer "\/(?<licensetype>(pro|free))\/(?<productversion>[^\/]+)\/[^\/]+\/(?<trial_or_prod>[^\/]+)" | eval licensetype = licensetype + trial_or_prod | stats count by licensetype, clientip, cookie | stats sum(count) as sessions, dc(cookie) as cookie_count by clientip, licensetype | stats sum(sessions) as sessions, avg(cookie_count) as avg_cookie_count, dc(clientip) as distinct_ips by licensetype | eval avg_sessions_per_ip=sessions/distinct_ips')
    
    #     SPL-21088 ExtendedSearchField module doesn't support searches with 'timechart' and 'chart'
    #     I tried to create a form search with ExtendedFieldSearch such as "*|timechart count by $st$", where $st$ is the variable I'm trying to replace. I'm then using HiddenChartFormatter to render the chart but the UI just punts.
    #     I isolated the problem to the 'timechart' and 'chart' commands causing a problem. If I substitute 'stats' it works just fine.
    
    
    #     SPL-20072 searching for '* | top 100 host' is translated into: * | top 100
    val = resurrect('* | top 100 host', True, 'search * | top host limit="100"')
    
    #     SPL-19998 splunkweb throws stacktrace on search errors search for: |sort
    
    val = resurrect('|sort foo')
    
    
    #     SPL-19997 splunkweb throws stacktrace on "|fields" 
    
    val = resurrect('|fields')
    val = resurrect('foo|fields')
    
    
    #     SPL-18847 Search bar has strange behavior when starting with a pipe
    #     1) any '| ... | [charting command]' search removes the leading pipe when views are switched, so the search fails next time
    #     2) also, leading spaces before starting with a | fails
    
    val = resurrect('| crawl | chart avg(foo)')
    val = resurrect(' | crawl | chart avg(foo)')
    
    
    #     SPL-18492 Altered query when we click report on results
    #     Run a search for: index="sampledata" | search "http" in splunk search.
    #     Click "report on results".
    #     Query is now index="sampledata" *
    #     "sourcename=com" Application
    #     Search returned results and I wanted to create a report. String in first step of report wizard:
    #     Application "sourcename="com"   
    
    val = resurrect('index="sampledata" | search "http"')
    val = resurrect('"sourcename=com" Application')
    
    
    #     SPL-24957 subsearch clauses are not ressurrected
    #     sourcetype=p4change [search sourcetype=p4change | head 100 | top user | fields user]
    #     resurrected
    #     sourcetype=p4change
    
    val = resurrect('[search sourcetype=p4change | head 100 | top user | fields user] sourcetype=p4change')
    
    
    #     SPL-22691 INTENTIONS: 'addterm' and 'addtermgt' intents both end up decomposing the same term.
    #     the linecount>200 term is decomposed by both 'addterm' and 'addtermgt'
    #     intents. Need to fix behavior so that quoted terms with specials are
    #     handled by 'addterm' and unquoted terms that contain the ><-signs are
    #     handed by the 'addtermgt' and 'addtermlt' intents.
    #     {"fullSearch": "search linecount>200 sourcetype=\"config_file\"", "intentions": [{"name": "addtermgt", "arg": {"linecount": 200}}, {"name": "addterm", "arg": "linecount>200"}, {"name": "addterm", "arg": {"sourcetype": "config_file"}}], "sid": "1241720987.92", "createTime": 1241720987.0, "baseSearch": "search *"}
    #     Note that linecount>200 now gets extracted twice so if i take baseSearch and intentions and reparse it, i get search linecount>200 "linecount>200" sourcetype="config_file"
    #     basically another roundtrip cas
    
    val = resurrect('linecount>200')
    # val = decompose('linecount>200')
    
    
    
    #     SPL-22591 INTENTIONS: stacktrace in web_service on search Exception
    #     in resurrectSearch when trying to parse the following search - search
    #     index=main sourcetype=osx_window_server | rex
    #     "\(?i)^[^\(]*\((?P<FIELDNAME>\d+)\)" | fields FIELDNAME | dedup
    #     FIELDNAME
    
    val = resurrect('index=main sourcetype=osx_window_server | rex "(?i)^[^\(]*\((?P<FIELDNAME>\d+)\)" | fields FIELDNAME | dedup FIELDNAME')
    
    
    #     SPL-21760 INTENTIONS: Report builder hjorks my search
    #     This is failing now because my search :
    #        source="/splunk/testlogs/testlogs/depot/splunk/maillog" | rex ".*?\]:\s(?<action>\S+)\s"
    #         gets translated to :
    #        source="/splunk/testlogs/testlogs/depot/splunk/maillog" | ".*?\]:\s(?<action>\S+)\s" 
            
    val = resurrect('source="/splunk/testlogs/testlogs/depot/splunk/maillog" | rex ".*?\]:\s(?<action>\S+)\s"')
            
            
    #     SPL-23502 INTENTIONS: saved searches get mangled in flashtimeline view
    #     source="*\windowsupdate.log" kb_num="*" Success Install "Installation Successful" | chart count,max(_time) as latest_install_time by kb_num | sort - latest_install_time | convert ctime(latest_install_time)
    #     becomes
    #     source="*\windowsupdate.log" kb_num="*" Success Install "Installation Successful" | convert ctime(latest_install_time) | sort - latest_install_time | chart count,max(_time) as latest_install_time by kb_num
    
    val = resurrect('source="*\windowsupdate.log" kb_num="*" Success Install "Installation Successful" | chart count,max(_time) as latest_install_time by kb_num | sort - latest_install_time | convert ctime(latest_install_time)')
    
    
    #     SPL-21740 Intentions: Decomposition does not sufficiently respect clause ordering.
    #          referer="-" NOT gif | head 10 | stats count | fields count | sort -count
    #         becomes
    #          NOT gif | head 10 | sort - count | search referer="-" | fields count | stats count
            
    val = resurrect('referer="-" NOT gif | head 10 | stats count | fields count | sort -count')
            
            
    #     SPL-23687 INTENTIONS: BETA: report builder mangles searches with "eval" command
    #     * | rex ".*?\[(?<pid>\d+)\]"| transaction pid maxspan="100m"
    #     becomes
    #     * | ".*?\[(?<pid>\d+)\]" | transaction pid maxspan="100m" 
    
    val = resurrect('* | rex ".*?\[(?<pid>\d+)\]"| transaction pid maxspan="100m"')
    
    
    #     SPL-23598 INTENTIONS: '| top foo by bar' handled ungracefully by report builder
    #     index=_audit action=search search=* | top 10 search by user
    #     becomes
    #     '... | top search limit=1000' 
            
    val = resurrect('index=_audit action=search search=* | top 10 search by user', True, 'search index="_audit" action="search" search="*" | top search by user limit="10"')
            
    val = resurrect('sourcetype=access_combined | top status by useragent', True, 'search sourcetype="access_combined" | top status by useragent limit="10"')
            
            
            
    #     SPL-17285 INTENTIONS: in the UI, clicking on keywords that have
    #     special characters does not get quoted in search bar See screen
    #     capture, basically when I clicked on Duration=372 in UI, Duration=372
    #     showed up on searchbar, but it should actually be quoted:
    #         "Duration=372" 
                
    val = applyIntention('search "Duration=372"', 'search *', 'toggleterm', 'Duration=372')
                
    #     SPL-21310    INTENTIONS: clicking on fields in event viewer should not just add "field=value" to the starting search command
    #     TOGGLETERM
    #     search * | eval foo = bar
    #     If I then click on the foo field for some value, currently the search will look like
    #     search * foo=somevalue | eval foo = bar
    #     which is wrong cause foo doesn't exist yet.
            
    val = applyIntention('search * | eval foo = bar | search Duration="372"', 'search * | eval foo = bar', 'toggleterm', {'Duration':'372'})

    # test > < quoting problems do to fix of above
    # known fail -- val = applyIntention('', 'search *', "addtermgt", {"linecount": '@#$#@$'})
    val = applyIntention('search linecount>200', 'search *', "addtermgt", {"linecount": 200})


test()            
            
            
            
