# Copyright (C) 2005-2010 Splunk Inc.  All Rights Reserved.  Version 4.0
# 
import os, re, sys, time
import utils, describer
import logging as logger


# adds count maping command to next command
def addCount(nextmap, command, next):
    if not command in nextmap.keys():
        nextmap[command] = {}
    nextcommmap = nextmap[command]
    if not next in nextcommmap.keys():
        nextcommmap[next] = 0
    nextcommmap[next] += 1

# adds count for how many times command is seen
def addCommonCount(commands, command):
    commands[command] = commands.get(command,0) + 1

def normalizeArgs(knownkeywords, arg):
    REGEX = re.compile("[a-zA-Z0-9_.-]+")
    matches = [match for match in REGEX.finditer(arg)]

    offset = 0
    for match in matches:
        keyword = match.group()
        start = match.start() + offset
        end = start + len(keyword)
        if not keyword in knownkeywords:
            arg = arg[0:start] + "VALUE" + arg[end:]
            offset += len("VALUE") - len(keyword)

    longfieldlists = re.findall("VALUE(?:,\s*VALUE){1,}", arg)
    for v in longfieldlists:
        arg = arg.replace(v, "VALUELIST")
    longfieldlists = re.findall("VALUE(?:\s+VALUE){1,}(?:\s+|$)", arg)
    for v in longfieldlists:
        arg = arg.replace(v, "VALUELIST ")
    arg = arg.replace(", ", " ")
    arg = arg.replace("  ", " ")
    arg = arg.replace("= ", "=")
    arg = arg.replace(" =", "=")
    arg = arg.replace("::", "=")

    arg = arg.replace("VALUELIST", "<value_list>")
    arg = arg.replace("VALUE", "<value>")
    
    return arg.strip()


def getPastSearches(user, sessionKey, namespace):

    userSearches = []
    bootstrapSearches = []

    try:
        bootsearchlog = os.path.join(os.environ['SPLUNK_HOME'],'etc','system','static','bootstrapsearches.txt')
        lines = utils.readText(bootsearchlog).split('\n')
        bootstrapSearches.extend(lines)
    except:
        logger.warn("Unable to get bootstrap search history")

    try:
        searchlog = os.path.join(os.environ['SPLUNK_HOME'],'var','log','splunk','searches.log')
        lines = utils.readText(searchlog).split('\n')
        REGEX = re.compile(' - %s\t(.*)' % user)
        for line in lines:
            # 2008-12-02 09:29:15,295 - admin	search elvis | was | here
            matches = REGEX.findall(line)
            if len(matches) > 0:
                userSearches.append(matches[0])
    except:
        logger.warn("Unable to get queries from searches.log")
    return bootstrapSearches, userSearches

END = "<RUN>"

cache = {}
MAX_CACHE_AGE = 60 # cache nextdata for up to one minute

def getNextData(user, stanzas, sessionKey, namespace):

    # if we have a cache of next data
    if sessionKey in cache:
        data_time, commandResults, searches = cache[sessionKey]
        # and it hasn't expired, return it.
        if data_time + MAX_CACHE_AGE > time.time():
            return commandResults, searches
    
    commandResults = []

    nextmap = {}
    commoncounts = {}
    argsmap = {}
    literals = describer.getLiterals(stanzas)
    bootstrapSearches, userSearches = getPastSearches(user, sessionKey, namespace)
    searches = bootstrapSearches + userSearches
    
    aliasMap = utils.getAliasMap(stanzas)
    badCommands = set()
    # for each search in file
    for search in searches:
        commandseqs = utils.getCommands(search, aliasMap)
        # for each sequency of commands for that search
        for j, commands in enumerate(commandseqs):
            commands.append((END,""));
            # for each command
            for i, commandarg in enumerate(commands):
                command, arg = commandarg
                if command not in literals:
                    if command != END:
                        badCommands.add(command)
                    keywords = []
                else:
                    keywords = literals[command]
                arg = arg.strip()
                #arg = normalizeArgs(keywords, arg)
                if command == END:
                    break
                addCount(argsmap, command, arg)
                addCount(nextmap, command, commands[i+1][0])
                addCommonCount(commoncounts, command)

    if len(badCommands) > 0:
        logger.warn("No searchbnf for these commands: %s!" % list(badCommands))

    commandAndCounts = commoncounts.items()
    commandAndCounts.sort( lambda x, y: y[1] - x[1] )
    for command,count in commandAndCounts:
        thisdata = {}
        commandResults.append(thisdata)
        thisdata['command'] = command
        thisdata['count'] = count

        thisargs = thisdata['args'] = []
        thisnexts = thisdata['nextcommands'] = []

        addSortedValueAndCounts(thisargs, argsmap[command])
        addSortedValueAndCounts(thisnexts, nextmap[command])

    # put calculated nextdata into cache
    cache[sessionKey] = time.time(), commandResults, userSearches
        
    return commandResults, userSearches

def addSortedValueAndCounts(result, valmap):
    
    valsAndCount = valmap.items()
    valsAndCount.sort( lambda x, y: y[1] - x[1] )
    total = 0
    for arg, count in valsAndCount:
        total += count
    for arg, count in valsAndCount:
        percent = "%.5s" % (100 * float(count) / total)
        result.append((arg, percent))
    
if __name__ == '__main__':
    if len(sys.argv) > 1:
        search = sys.argv[1]
        commands = utils.getCommands(search, None)
        comms = [c.strip() for search in commands for c,a in search ]
        args = [a.strip() for search in commands for c,a in search ]
        print "Commands:", commands
        print "Commands: %s  Args: %s" % (comms, args)
    else:
        user = "admin"
        sessionKey = utils.TEST_SESSION()
        namespace = utils.TEST_NAMESPACE()
        #print getPastSearches(user, None, sessionKey, namespace)
        bnf = utils.getStanzas("searchbnf", sessionKey, namespace)
        data, searches = getNextData(user, bnf, sessionKey, namespace)
        for command in data:
            print "\t%s" % command

        
##     for command,count in commandAndCounts:
##         print "command: %s (%s)" % (command, count)
##
##         print "\targs:"
##         argsAndCount = argsmap[command].items()
##         argsAndCount.sort( lambda x, y: y[1] - x[1] )
##         for arg, count in argsAndCount:
##             print "\t\t%s (%s)" % (arg, count)
##
##         nextCommAndCounts = nextmap[command].items()
##         nextCommAndCounts.sort( lambda x, y: y[1] - x[1] )
##         print "\tnext command:"
##         for comm, count in nextCommAndCounts:
##             print "\t\t%s (%s)" % (comm, count)
## 
## 
##                 
############################# MOVED FROM SEARCHHELPER
########BELOW################ MOVED FROM SEARCHHELPER
############################# MOVED FROM SEARCHHELPER
## 
## def getNextToken(syntax):
##     print "syntax:", syntax
##     subs = re.findall("\(([^()]*)\)([?*+]?)", syntax)
##     nexts = []
## 
##     for sub in subs:
##         subexp = sub[0]
##         modifier = sub[1]
##         print  "sub:", subexp, "mod:", modifier
##         if modifier == '+' or modifier == '': # if required, just return
##             nexts.extend(getNextToken(subexp))
##             #print "NEXTA:", nexts            
##             return nexts
##         else: # it's optional, append on to previous nexts
##             nexts.extend(getNextToken(subexp))
##             #print "NEXTB:", nexts
##     if len(subs) == 0:
##         ors = syntax.split("|")
##         nexts = [o.strip() for o in ors]
##         #return syntax
##         #return subs
##     return nexts
##     
## def getNextArg(stanzas, command, args):
##     stanza = stanzas[command+"-command"]
##     syntax = bnf.recurseSyntax(stanzas, stanza)
## 
##     # (<num>)? ((("-"|"+")?( )?((auto|str|ip|num)?<field>) , )+ (d|desc)?)
##     return getNextToken(syntax)
## 
##     
## def getNext(search, insertpos = None):
##     # default to 
##     if insertpos == None:
##         insertpos = len(search)
##     search = search[:insertpos]
##     search = search.strip()
## 
##     stanzas = utils.getStanzas("searchbnf", sessionKey, namespace)
## 
##     lastCommand = getLastCommand(search)
##     if lastCommand == None:
##         return getAllCommands(stanzas)
##     command,args = lastCommand
##     print "last:", command, "args:", args
##     return getNextArg(stanzas, command, args)
## 
## 
############################# MOVED FROM SEARCHHELPER
############################# MOVED FROM SEARCHHELPER
############################# MOVED FROM SEARCHHELPER

