import re, sys
import logging as logger
import json
import didYouMean,didYouKnow,utils,describer,next
import splunk.rest, splunk.entity

# Searchhelp
#
#
#  search error | ? report on top ips
#
# - suggest terms
#    - you might also be interested in ___?
#    - the ____ field can help narrow does these results
# - suggest operators
#   - matches search against savedsearch snippets (not valid save searches, but named/tagged/described snippets of search)
#     - [most common values of a field] = | top FIELD
#     - [make a graph to chart the max values against two other fields] = | stats max(FIELD) by _time | timechart FIELD2 by FIELD3
#   - match search operator against tags/description of operator: "|report " -- did you mean chart or timechart?
#   - order typeahead/search assistant by most common operators that follow an operator. e.g. search->stats,sort.  stats->chart,timechart
#     - get order grabbing all search commands used on tiny.
# - suggest searches
#   - keep history of user's searches that return results and suggest them when query is similar.
#     - ideally replace fields and values. e.g. stored: "search ip=10* | top ip" queried "search source=*error*"  .. suggest: "search source=*error* | top source"
# - generate a description from each search:
#   - "search ip=10* source=*mail*| top ip | chart ip" -- "search for events where ip starts with '10' and contains 'mail'.  From those results, calculate the most frequent ip addresses. And from those results,  graph a chart of the ip values".  simpler -- "search for ip=10* source=*mail* and calculate most frequent values, and then make a chart."
#
# - suggest optimizations. swapping of operators.  search | sort FIELD | head N  --> search | sort N FIELD
#                                                  search | sort | fields -> search | fields | sort  (test case 20% faster)


# help outputs dic with these keys:
#    notices : list of string notices
#    args : list of (arg,perc) pairs
#    nexts : list of (next,perc) pairs
#    command : dict of strings ('name', 'shortdesc', 'details', 'syntax') and 'examples' pair of (example, comment)

def help(sessionKey, namespace, user, search, insertpos=None, earliest_time=None, latest_time=None, count=10, max_time=None, servers=None, useTypeahead=False):
    """
    "did you mean ___?"
    "did you know ___?"
    "the 'sort' operator takes blah arguments and does blah"
    "you might also be interested in ___?"
    "the fields ___ can help narrow does these results"
    "these past searches are similar to your search"
    "these saved searches are similar to your search"
    "you are searching for ip and host and then deduplicating by host"
    "your search would be faster if you ..."
    """

    originalsearch = search
    if insertpos == None: # no insertion point, use end
        insertpos = len(search)
    else:
        try:
            insertpos = int(insertpos)
        except:
            insertpos = len(search)

    search = search[:insertpos].strip()

    if search == "":
        search = "| search"
    elif not search.startswith("|"):
        search = "| " + search

    usersquery = originalsearch
    if usersquery.startswith("search "):
        usersquery = usersquery[len("search "):]
    queryprefix = utils.allButLast(usersquery)

    # defaults
    output = { 'notices': [], 'args': [], 'nexts': [], 'autocomplete':[], 'autocomplete_match':'', 'command':{}, 'typeahead': [],
               'search': usersquery, 'searchprefix': queryprefix, 'allcommands': [], 'savedsearches': [] }

    try:
        bnf = utils.getStanzas("searchbnf", sessionKey, namespace)
        output['allcommands'] = utils.getAllCommands(bnf)
        aliasMap = utils.getAliasMap(bnf)
        
        if (splunk.util.normalizeBoolean(useTypeahead)):
            suggestSearchTypeahead(output, search, count, max_time, earliest_time, latest_time, servers, namespace)
        
        firstTermShouldBeCommand(output, search, aliasMap)
        didYouMean.help(output, bnf, sessionKey, namespace, user, search)
        didYouKnow.help(output, aliasMap, user, search)
        relatedPastSearches(output, user, search)
        relatedSearches(output, sessionKey, namespace, user, search)
        commandHelp(output, user, search, aliasMap, bnf)
        nextCommand(output, sessionKey, namespace, user, search, aliasMap, bnf)
        relatedTerms(output, user, search)
        usefulFields(output, user, search)
        describeSearch(output, user, search)
        suggestOptimizations(output, user, search)
    except Exception, e:
        msg = "! Error in search assistant: %s" % e
        output['notices'].insert(0,msg)
        import traceback
        msg += traceback.format_exc()
        logger.error(msg)
    return output


def suggestSearchTypeahead(output, search, count, max_time, earliest_time, latest_time, servers, namespace):

    commandAndArgs = utils.getLastCommand(search, None)
    if commandAndArgs != None:
        command, args = commandAndArgs
        if command == "search":
            output['typeahead'] = getTypeaheadTerms(args, count, max_time, earliest_time, latest_time, servers, namespace)


def firstTermShouldBeCommand(output, search, aliasMap):
    m = re.match("\s*\|\s*search\s+([^ ]+)", search)
    if m:
        groups = m.groups()
        if len(groups) > 0:
            firstArg = groups[0].lower()
            if firstArg in aliasMap:
                output['notices'].append('Your first search term is also a search command.  Did you mean " | %s"?' % firstArg)

def getTypeaheadTerms(q, count, max_time, earliest_time, latest_time, servers, namespace):
    
    terms = []

    # don't bother with empty requests
    if not q or q.strip() in ('', '*'):
        return terms
                 
    requestArgs = { 'output_mode': 'json', 'prefix': q, 'count': count, 'max_time': max_time}
    if earliest_time: requestArgs['earliest_time'] = earliest_time
    if latest_time: requestArgs['latest_time'] = latest_time
    if servers: logger.warn('typeahead server spec not implemented yet')

    uri = splunk.entity.buildEndpoint('search', 'typeahead', namespace=namespace)
    
    try:
        response, content = splunk.rest.simpleRequest(uri, getargs=requestArgs, raiseAllErrors=True)
    except Exception, e:
        logger.error('Searchhelper could not fetch typeahead terms')
        logger.exception(e)
        
    # server says we have data #
    if response.status == 200:
        output = json.loads(content)
        for item in output:
            try:
                count = int(item['count'])
                token = item['content']
                # unclear what "operator=<bool>" is
                terms.append((token,count))
            except Exception, e:
                logger.error('unable to parse typeahead values: %s' % e)
                
    return terms



def commandHelp(output, user, search, aliasMap, bnf):
    commandAndArgs = utils.getLastCommand(search, aliasMap)
    if commandAndArgs != None:
        command, args = commandAndArgs
        description = describer.describeCommand(bnf, command, True)
        if description != None:
            output['command'] = description


MAX_NEXT_COMMANDS = 10
MAX_NEXT_ARGS = 10

def nextCommand(output, sessionKey, namespace, user, search, aliasMap, bnf):
    showargs = True

    # if search ends in "|", don't give args for last command, but
    # give next information from previous commandd
    if search[-1] == "|":
        search = search[:-1]
        showargs = False

    nextcommands = []
    commandAndArgs = utils.getLastCommand(search, aliasMap)
    if commandAndArgs == None:
        # list all generating commands.
        # make a list() copy so we don't trash it by adding search
        nextcommands = list(utils.getAllCommands(bnf, True)) 
        nextcommands.insert(0,'search')
    else:        
        command, args = commandAndArgs
        data, pastsearches = next.getNextData(user, bnf, sessionKey, namespace)
        for datum in data:
            if datum['command'] == command:
                nextcommands = [x for x,y in datum['nextcommands'] if x != "<RUN>"]
                if showargs:
                    output['args'] =  datum['args']
                break
        # now add in all commands that were not already added
        if command in aliasMap:
            allcommands = utils.getAllCommands(bnf)
            # if we have some commands, set a 'break' marker before adding all the other commands
            if len(nextcommands) > 0:
                    nextcommands.append("<break>")
            # adding all the other commands not already added
            for thiscommand in allcommands:
                if thiscommand not in nextcommands:
                    nextcommands.append(thiscommand)

        # look for pastsearches that the current search is a subset of  (like firefox url autocomplete looking for any term)
        usersearch = normalizeSearch(search)
        if len(usersearch) > 0:
            # reverse order of searches, so most recent are on top
            pastsearches.reverse()
            # of the last 5000 past searches (don't need to work on anything more)
            pastsearches = pastsearches[:5000]
            pastMatches = [userifySearch(pastsearch) for pastsearch in pastsearches if normalizedSearchMatch(usersearch, pastsearch)]
            # dedup
            pastMatches = sorted(list(set(pastMatches)), key=pastMatches.index)         
            output['autocomplete'] = pastMatches[:10] # just the 10 most recent
            output['autocomplete_match'] = usersearch

    # keep only those that alias to themselves -- i.e., don't show aliases
    nextcommands = [x for x in nextcommands if x == '<break>' or aliasMap.get(x, '') == x]
    output['nexts'] = [(x, utils.getAttr(bnf,x,"shortdesc","")) for x in nextcommands]

# return true if usersearch is a substring of normalized pastsearch (but not equal)
def normalizedSearchMatch(usersearch, pastsearch):
    pastsearch = normalizeSearch(pastsearch)
    return usersearch in pastsearch and usersearch != pastsearch

# take past searches of one form and make them how the user would type them
# "search foo | sort bar" --> "foo | sort bar"
# "crawl | sort bar"      --> "| crawl | sort bar"
def userifySearch(search):
    if search.startswith("|"):
        return search
    if search.startswith("search "):
        return search[7:]
    return "| " + search

def normalizeSearch(search):
    normalizedsearch = search.lower().strip()
    prefixes = ["| search ", "|search ", "search "]
    for prefix in prefixes:
        if normalizedsearch.startswith(prefix):
            normalizedsearch = normalizedsearch[len(prefix):]
            break
    else:
        if search == "|search" or search == "| search":
            normalizedsearch = ""
    return normalizedsearch.strip()


def relatedTerms(output, user, search):
    """you might also be interested in ___?"""
    return 

def usefulFields(output, user, search):
    """the fields ___ can help narrow does these results"""
    return 

def relatedSearches(output, sessionKey, namespace, user, search):
    """these saved searches are similar to your search"""
    import difflib
    
    savedsearches = utils.getStanzas("savedsearches", sessionKey, namespace)
    searchmap = {}
    for name in savedsearches:
        ssearch = savedsearches[name].get('search', None)
        if ssearch != None:
            searchmap[ssearch] = name
    searches = searchmap.keys()
    bestmatches = difflib.get_close_matches(search, searches, cutoff=0.65)
    if len(bestmatches) == 0:
        return 
    
    output['savedsearches'] = [(searchmap[match],match) for match in bestmatches if match!=search]



# security issue or showing other user's searches?
def relatedPastSearches(output, user, search):
    """these past searches are similar to your search"""
    # ideally replace fields and values.
    #    stored: "search ip=10* | top ip"
    #    queried "search source=*error*" ..
    #    suggest: "search source=*error* | top source"
    
    return


# generate a description from each search
def describeSearch(output, user, search):
    # search ip=10* source=*mail*| top ip | chart ip
    #
    # "search for events where ip starts with '10' and contains 'mail'.
    # From those results, calculate the most frequent ip addresses. And
    # from those results, graph a chart of the ip values".  simpler --
    # "search for ip=10* source=*mail* and calculate most frequent
    # values, and then make a chart.
    return 
    

def suggestOptimizations(output, user, search):
    # swapping of operators.
    #      search | sort FIELD | head N  --> search | sort N FIELD
    #      search | sort | fields -> search | fields | sort  (test case 20%-400% faster)
    return 



# returns likely next arguments and command
def nextHelp(output, user, search):
    #   - order typeahead/search assistant by most common operators that
    #     follow an operator. e.g. search->stats,sort.
    #     stats->chart,timechart
    #   - get order grabbing all search commands used on tiny.
    return 




    
if __name__ == '__main__':
    argc = len(sys.argv)
    argv = sys.argv
    if argc >= 3:
        user = argv[1]
        search = argv[2]
        insertpos = None
        if argc > 3:
            insertpos = int(argv[3])
        help = help(utils.TEST_SESSION(), utils.TEST_NAMESPACE(), user, search, insertpos)
        #for attr,val in help.items():
        #    print "%s%s = %s\n" %(attr," "*abs(40-len(attr)),val)
        print help
    else:
        print 'Usage:'
        print argv[0], '<user> <search> [<insertpos>]'

