
Splunk.ModuleLoader= $.klass({

    initialize: function() {
        this.logger = Splunk.Logger.getLogger("module_loader.js");
        this.messenger = Splunk.Messenger.System.getInstance();
        this._modules = [];
        this._topLevelModules = [];
        this._modulesByID = {};

        // step 1 - decorate all module prototypes with module classNames.
        for (var name in Splunk.Module) {
            if (Splunk.Module[name] && this._hasSuperclass(Splunk.Module[name], Splunk.Module) && !Splunk.Module[name].isAbstract) {
                Splunk.Module[name].prototype.moduleType = 'Splunk.Module.' + name;
            }
        }

        // step 1a - listen to the moduleLoadStatusChange event. ModuleLoader fires this itself in most cases,
        // but some modules need to do their own asynchronous loading so some will fire it
        // themselves, and in general the events can come from anywhere.
        $(document).bind('moduleLoadStatusChange', this.onModuleLoadStatusChange.bind(this));
        this._moduleLoadStatesById = {};

        // step 2 -  identify and instantiate all the module classes in the view, as a flat list.
        this.buildModuleInstanceList();

        //this.insertFlashStub()
        // step 3 - using the flat list, and the parentModule id references,
        // walk through the list, building the hierarchy.
        // at the end return the modules at the top level of the hierarchy.
        this.buildHierarchyAndTopLevelModules(this._modules);

        // hierarchy is built. Switch the modules with simple synchronous loading, to INITIALIZED.
        for (var i=0, lim=this._modules.length; i<lim; i++) {
            var containerId = $(this._modules[i].container).attr("id");

            // if the module says he doesnt need async loading, then we mark him INITIALIZED here.
            if (!this._modules[i].requiresAsynchronousLoading()) {
                $(document).trigger('moduleLoadStatusChange', [containerId, Splunk.util.moduleLoadStates.INITIALIZED]);
            }
        }


        if (!this._settingsLoadHasStarted && this.getLowestLoadState() >= Splunk.util.moduleLoadStates.INITIALIZED) {
            this.loadSettingsAndContexts();
        }
    },

    /*
    insertFlashStub: function() {
        var targetId    = "SplunkFlashStub";
        var swfObjectId = targetId + "_swf";
        var swfUrl = Splunk.util.make_url("/static/flash/timeline.swf");
        var expressInstallUrl = Splunk.util.make_url("/static/flash/expressInstall.swf");
        var minVersion = Splunk.util.getConfigValue("FLASH_MAJOR_VERSION") + "." + Splunk.util.getConfigValue("FLASH_MINOR_VERSION") + "." + Splunk.util.getConfigValue("FLASH_REVISION_VERSION");

        var params = {
        //    wmode: "opaque"
        };
        var flashVars = {
            'basePath':   Splunk.util.make_url('/splunkd'),
            'sessionKey': Splunk.util.getConfigValue('SPLUNKD_SESSION_KEY')
        };
        var attributes = {
            id: swfObjectId,
            name: swfObjectId
        };
        //SWFObject does complete node replacement, not target child replacement.
        swfobject.embedSWF(swfUrl, targetId, "20", "20", minVersion, expressInstallUrl, flashVars, params, attributes);
    },
    */

    loadSettingsAndContexts: function() {
        if (this._settingsLoadHasStarted) {
            this.logger.warn("caught a second call to ModuleLoader.loadSettingsAndContext. Exiting.  This is either an extremely unlikely race condition, or a mistake introduced into the code subsequent to this writing.")
            return false;
        } this._settingsLoadHasStarted = true;

        // A bit of a misnomer, INITIALIZED really means 'ready to have receive settings and context information'.
        // it does not merely mean that the constructor has exited and the module exists.
        $(document).trigger('allModulesInitialized', [Splunk.util.moduleLoadStates.INITIALIZED]);
            
            
        
        // as of the SPL-20016, which is a better solution there is no need for this.
        // settings changes will get quietly pushed when the context change happens later.
        // eliminates the nasty chicken and egg problem here that many modules have run into.
        this._withEachModule(this._topLevelModules, function(module) {
            module.pushSettingsToChildren({}, true);
        });

        // check to see if the python gave us a parsed search to resurrect.
        var decomposedSearch = $("div#toBeResurrected");
        if (decomposedSearch && decomposedSearch.text()) {
            var jsonContext = JSON.parse(decomposedSearch.text());
            if(typeof(jsonContext)=="string") {//TODO: update view layout escaping to remove need for double json parsing
                jsonContext = JSON.parse(jsonContext);
            }
            var context = Splunk.SearchContext.resurrect(jsonContext);
            // if this job was resurrected from a search id,  then we cannot safely allow it to be cancelled when we leave the view. 
            // trivial example is the sid links used by report builder. 
            if (context.isJobDispatched()) {
                context.job.setAsAutoCancellable(false);
            }
            this.startResurrection(context, this._topLevelModules);
        } else {
            this._withEachModule(this._modules, function(module) {
                if (Splunk.util.normalizeBoolean(module.container.attr('s:autoRun'))) {
                    module.pushContextToChildren();
                }
            });
        }
        
        for (var i=0, lim=this._modules.length; i<lim; i++) {
            $(document).trigger('moduleLoadStatusChange', [this._modules[i].container.attr('id'), Splunk.util.moduleLoadStates.LOADED]);
        }
        // Fired after everything is loaded.
        // Used by Jobber and Printing to determine certain behaviors.
        if (this.getLowestLoadState() >= Splunk.util.moduleLoadStates.LOADED) {
            $(document).trigger('allModulesLoaded', [Splunk.util.moduleLoadStates.LOADED]);
        } else {
            this.logger.error('Assertion failed. All modules should be LOADED after ModuleLoader has pushed both settings and SearchContexts.');
        }
    },

    getLowestLoadState: function() {
            var minimumStatus = Splunk.util.moduleLoadStates.MAX;

        for (var moduleId in this._moduleLoadStatesById) {
            if (this._moduleLoadStatesById.hasOwnProperty(moduleId)) {
                minimumStatus = Math.min(minimumStatus, this.getModuleLoadState(moduleId));
            }
        }
        return minimumStatus;
    },

    getModuleLoadState: function(moduleId) {
        return this._moduleLoadStatesById[moduleId];
    },

    onModuleLoadStatusChange: function(event, moduleId, statusInt) {
        //this.logger.debug("onModuleLoadStatusChange event=", event, " moduleId=", moduleId, " statusInt=", statusInt);
        if (!statusInt) {
            this.logger.error("Assertion failed - received null or invalid statusInt for module=", moduleId, " statusInt=", statusInt);
        }
        if (this._moduleLoadStatesById.hasOwnProperty(moduleId)) {
            var oldStatus = this.getModuleLoadState(moduleId);
            if (oldStatus > statusInt) {
                this.logger.error("Assertion failed - module moved backwards in load status. id=", moduleId , " new statusInt=", statusInt, "old status", oldStatus);
            } else if (oldStatus == statusInt) {
                this.logger.warn("same status received for same module more than once. id=", moduleId , " new statusInt=", statusInt);
            } else if (statusInt < oldStatus) {
                this.logger.error("Assertion failed - received module status changes out of sequence. id=", moduleId , " new statusInt=", statusInt);
            }
        }
        this._moduleLoadStatesById[moduleId] = statusInt;

        for (var i=0,l=this._modules.length; i<l; i++) {
            if (this._modules[i].moduleId == moduleId) {
                this._modules[i].onLoadStatusChange(statusInt);
                break;
            }
        }
        if (!this._settingsLoadHasStarted && (this.getLowestLoadState() >= Splunk.util.moduleLoadStates.INITIALIZED)) {
            this.loadSettingsAndContexts();
        }
    },

    buildModuleInstanceList: function() {
        this._modules = [];

        // get the collection of all module container divs.
        var moduleContainers = $('.SplunkModule');

        for (var i=0, lim=moduleContainers.length; i<lim; i++) {
            var container = moduleContainers[i];
            var subclass = container.className.replace("SplunkModule ","");

            var moduleId = $(container).attr('id');
            // no need to trigger the event that says they're NONEXISTENT.
            this._moduleLoadStatesById[moduleId] = Splunk.util.moduleLoadStates.NONEXISTENT;

            if (subclass in Splunk.Module) {
                var module = new Splunk.Module[subclass](container)
                this._modules.push(module);

                if (this._modulesByID.hasOwnProperty(container.id)) {
                    this.logger.error(sprintf(_("Splunk.ModuleLoader - Assertion failed. Two modules were given the same id %s"),container.id));
                }
                this._modulesByID["#" + container.id] = module;
            } else {
                alert(sprintf(_('Splunk encountered the following unknown module: "%s" .  The view may not load properly.'), subclass));
                this.logger.error("Splunk.ModuleLoader - Assertion failed. View config specifies a Module whose implementation cannot be found. className=",
                subclass);
            }
        }

    },

    buildHierarchyAndTopLevelModules: function(allModuleInstances) {
        this._topLevelModules = [];

        // then, setup parent-child relationships
        this._withEachModule(allModuleInstances, function(childModule) {

            var parentSelector = childModule.container.attr('s:parentmodule');
            if (parentSelector) {

                var parentContainer = $(parentSelector)[0];
                // throw an exception if a parentmodule selector is specified but no matching elements are found
                if (!parentContainer) {
                    this.logger.error(
                            'Splunk.ModuleLoader - Module "'
                            + childModule.container.attr('id')
                            + '" could not find its declared parent module "'
                            + parentSelector + '"');
                }
                this._modulesByID[parentSelector].addChild(childModule);

            } else {
                this._topLevelModules.push(childModule);
            }
        }.bind(this));
    },

    _hasSuperclass: function(subclass, superclass) {
        // return true if the subclass has the superclass as a parent or ultimate grandparent
        var i = 0;
        while ((subclass.superclass) && i < 100) {
            //TODO - this had a single equals, but everything seemed to work. Changed it, and Im now deeply suspicious.
            if (subclass.superclass == superclass) return true;

            subclass = subclass.superclass;
            i++;
        }
        return false;
    },

    startResurrection: function(context, topLevelModules) {

        this._withEachModule(topLevelModules, function(module) {

            var seamModule = module;
            // we are walking down, looking for the first module whose IMMEDIATE children
            // require dispatch.
            // if none are found, we will go to the first branch in the tree.  ie the first module with more than one child.
            while (!seamModule.requiresDispatch() && !seamModule.someChildrenRequireDispatch(context)) {

                if (seamModule._children && seamModule._children.length == 1) {
                    seamModule = seamModule._children[0];
                // if there is more than one child, then we can go no deeper and we have to start resurrection from here. 
                // and if there are NO children, then it doesnt make much sense to resurrect at all, but lets do it anyway. 
                //    (maybe someone will make a view where there's no dispatching modules, but somehow the resurrected values 
                //     do end up doing *something* meaningful.)
                } else {
                    break;
                }
            }
            this.logger.warn("resurrection proceeding from ", seamModule.moduleType, " id=", seamModule.container.attr("id"));
            var upwardContext = context.clone();
            seamModule.applyContext(upwardContext);
            seamModule.passContextToParent(upwardContext);

            seamModule.pushContextToChildren(context);
        }.bind(this));
        
    },

    /*
     * Utility function to iterate over all the modules and run a callback for each.
     */
    _withEachModule: function(moduleInstances, moduleFunc) {
        var numberOfModules = moduleInstances.length;
        if (!numberOfModules) {
            this.logger.warn("No modules found in view.");
        }
        for (var i=0;i<numberOfModules;i++) {
            moduleFunc(moduleInstances[i]);
        }
    },

    /**
     * Returns the active Javascript module class instance that corresponds to
     * a specific container DOM id
     *
     * NOTE: This method is only intended for testing introspection.  If you
     *      are using this method in primary development, you're doing it wrong.
     *
     */
    getModuleInstanceById: function(containerId) {
        for (var i=0,L=this._modules.length; i<L; i++) {
            if (this._modules[i].container.attr('id') == containerId) {
                return this._modules[i];
            }
        }
        throw new Error('Could not find module class instance for DOM ID=' + containerId);
    },

	///////////////////////////////////////////////////////////////////////////
	// Parameter persistence methods
	///////////////////////////////////////////////////////////////////////////

	/**
	 * Persists all of the current modules' parameters.
	 *
	 * @param {String} The unique ID under which to save the viewstate.  If
	 *      null, then a new ID will be generated.
	 * @param {Boolean} Indicates if this viewstate is to be saved in a shared
	 *      context within the current app.  Defaults to false.
	 *
	 */
	commitViewParams: function(viewstate_id, is_shared) {

        // ensure that a valid viewstate is passed
        // if the current view also specifies a display view, then push this
        // viewstate into the global context so that both the current and 
        // display view can access the pending viewstate
		if (!viewstate_id) {
			viewstate_id = this._generateViewstateId();
			if (Splunk.util.getCurrentDisplayView() != Splunk.util.getCurrentView()) {
			    viewstate_id = '*:' + viewstate_id;
			}
		} else {
			viewstate_id = $.trim(viewstate_id);
		}

		// collect all parameters from all modules

		var payload = [];

		this._withEachModule(this._modules, function(module) {
			var tmpParams = module.snapshotParamset();

			for (var key in tmpParams) {
				if (tmpParams.hasOwnProperty(key)) {
					payload.push(
						module._buildParamName(key)
						+ '=' + encodeURIComponent(tmpParams[key])
					);
				}
			}
		});

		if (payload.length == 0) {
			this.logger.warn(sprintf(
				'commitViewParams - no parameters to persist; app=%s view=%s',
				Splunk.util.getCurrentApp(),
				Splunk.util.getCurrentView()
			));
			return null;
		}

        if (is_shared == true) {
            payload.push('_is_shared=true');
        }

		$.ajax({
			type: 'POST',
			url: this._modules[0]._buildParamUri(viewstate_id),
			data: payload.join('&'),
			dataType: 'json',
			success: this._commitViewParamsCallback.bind(this),
			error: this._commitViewParamsErrorback.bind(this)
		});

		return viewstate_id;
	},

	/**
	 * Private method.  Handles commitViewParams callback.
	 *
	 */
	_commitViewParamsCallback: function(data, textStatus) {
		this.logger.debug('_commitViewParamsCallback - status=' + textStatus);
	},

	/**
	 * Private method.  Handles commitViewParams error condition.
	 *
	 */
	_commitViewParamsErrorback: function(xhr, status, error) {
		this.logger.error('_commitViewParamsErrorback - ' + error);
	},

    /**
     * Private method.  Returns a relatively unique ID that can be used as a
     * viewstate ID.
     *
     */
    _generateViewstateId: function() {
        return (new Date()).getTime().toString(36);
    },

    /**
     * Retrieve an object literal representation of all templateArgs returned from the view.py render method and trigger viewDataChange
     * event having event and data arguments.
     */
    refreshViewData: function(){
        $.ajax({
            type: "GET",
            url: Splunk.util.make_url("api", "app", Splunk.util.getCurrentApp(), Splunk.util.getCurrentView()),
            dataType: "text",
            complete: function(data, textStatus){
                if(data.status==200){
                    try{
                        viewData = JSON.parse(data.responseText);
                    }catch(e){
                        this.logger.warn("Could not parse view data with error", e);
                        return;
                    }
                    $(document).trigger("viewDataChange" , [viewData]);
                }else{
                    this.logger.error("Could not retrieve view data change status was", data.status);
                }
            }.bind(this)
        });
    }

});

