
function getModuleTypes() {
    var moduleTypes = [];
    for (var name in Splunk.Module) {
        if (Splunk.Module.hasOwnProperty(name) 
            && Splunk.Module[name] && Splunk.Module[name].hasOwnProperty("superclass")
            && !Splunk.Module[name].isAbstract) 
        {
            moduleTypes.push(name);
        }
    }
    return moduleTypes;
}

/**
 * Useful method that will run the given function once for every module class.
 */
function callForEachModule(func, generateArgs) {
    for (var i=0, lim=moduleTypes.length; i<lim; i++) {
        var className = moduleTypes[i];
        var moduleUnderTest = generateInstance(className, generateArgs);
        func(moduleUnderTest);
    }
}
/**
 * Useful method that will run the given function once for every combination of parentClass and childClass.
 */
function callForEachCombination(func, safely) {
    
    for (var i=0, lim=moduleTypes.length; i<lim; i++) {
        var moduleOneClassName = moduleTypes[i];
    
        for (var j=0, lim2=moduleTypes.length; j<lim2; j++) {    
            var moduleTwoClassName = moduleTypes[j];
            var moduleOne  = generateInstance(moduleOneClassName);
            var moduleTwo  = generateInstance(moduleTwoClassName);
            func(moduleOne, moduleTwo);
        }
    }
}

function checkChildVisibilities(moduleUnderTest, caseTitle, visibilityArr) {
    for (var i=0; i<visibilityArr.length; i++) {
        // PITA - using jquery's is:visible method doesnt work. Possibly cause we're not adding the elements to the DOM here.
        var expected = (visibilityArr[i] == 1) ? "block" : "none";
        var received = moduleUnderTest._children[i].container[0].style.display;
        // a null display property is as good as display block
        if (received == "" && expected=="block") expected = "";
        assertEquals(caseTitle + ", child #" + i + " should have display " + expected, expected, received);
    }
}

function addMockParent(moduleUnderTest) {
    var mockParent = new Splunk.Module($("<div>"));
    mockParent.addChild(moduleUnderTest);
    mockParent.moduleType = "mockParentModule";
    return mockParent;    
}
function addMockChildren(moduleUnderTest, n) {
    var mockChildren = [];
    for (var i=0;i<n; i++) {
        child = new Splunk.Module($("<div>"));
        child.moduleType = "mockChildModule";
        child._params["group"] = "group " + i;
        child._onSettingsChangeCounter = child._onContextChangeCounter  = 0;
        child.onSettingsChange = function() {this._onSettingsChangeCounter++;}.bind(child);
        child.onContextChange = function() {this._onContextChangeCounter++;}.bind(child);
        mockChildren[i] = child;
        moduleUnderTest.addChild(child);
    }
    return mockChildren;    
}
function addMockContext(module, firstId, reDispatchedId) {
    var mockContext = new Splunk.SearchContext("foo bar");
    mockContext._dispatchJobCounter = 0;
    mockContext._absorbIntentionsCounter = 0;
    // a bit risky as this may change behaviour, but we cant have the clone() method unmocking our context.
    mockContext.clone = function(){return this}.bind(mockContext);
    // mock the dispatchJob method
    mockContext.dispatchJob = function(onSuccess) { 
        this._dispatchJobCounter++;
        if (!reDispatchedId) reDispatchedId = "reDispatchedId not specified in testUtils addMockContext()";
        this.job.setSearchId(reDispatchedId);
        onSuccess(this)
    }.bind(mockContext);
    // mock the absorbIntentions method
    mockContext.absorbIntentions = function(onSuccess, onFailure) { 
        this._absorbIntentionsCounter++;
        onSuccess(this.getSearch())
    }.bind(mockContext);
    if (firstId) {
        mockContext.job.setSearchId(firstId);
    }
    module.baseContext = mockContext;
    return mockContext;
}