// Splunk.Module is the abstract base class for Splunk UI modular components. 

Splunk.Module.DispatchingModule = $.klass(Splunk.Module, {
    /*
     * DispatchingModule's constructor handles the attachment of jobProgress and
     * jobDone events, such that onJobProgress and onJobDone will only be called
     * when the job held in this.baseContext is actually changed.  
     * Other unrelated job events will not result in the handlers being called.   
     * NOTE: If you want to make a module subscribe to ALL jobProgress or
     * jobDone events the simplest way would be to override initialize and not
     * call $super(), and then just bind directly.
     */
    initialize: function($super, container) {
       
        this.logger = Splunk.Logger.getLogger("search_modules.js");
        // This guarantees that onJobProgress and onJobDone will ONLY
        // be called when it's the baseContext's job that has the progress or
        // has become done. 

        $(document).bind('jobProgress', function(event, job) {
            // other modules' jobProgress events may get fired before any contexts are set.
            if (!this.baseContext || !this.baseContext.job) return false;

            //if (this.baseContext.job == job) {
            if (this.baseContext.job.getSearchId() == job.getSearchId()) {
                this.onJobProgress(event);
            }
        }.bind(this));

         //see simlar comment above for jobProgress
        $(document).bind('jobDone', function(event, job) {
            // other modules' jobProgress events may get fired before any contexts are set.
            if (!this.baseContext || !this.baseContext.job) return false;

            //if (this.baseContext.job == job) {
            if (this.baseContext.job.getSearchId() == job.getSearchId()) {
                this.onJobDone(event);   
            }
        }.bind(this));

        $(document).bind('jobStatusChanged', function(event, sid, status) {
            if (this.baseContext && (this.baseContext.job.getSearchId() == sid)) {
                this.onJobStatusChange(event, status);
            }
        }.bind(this));
       
        $super(container);
    },
   
    // the module's current Job instance, which contains all details of the search that this module has dispatched   
    baseContext: false,
   
    /*
     * Public function.  Returning true here tells the Module loading code that
     * this module's job needs to be dispatched if it hasnt been already.
     */
    requiresDispatch: function(context) {return true},

    pushContextToChildren: function($super, explicitContext) {
        var context = explicitContext || this.getModifiedContext();
        // check to see whether or not we have a searchId already. 
        if (!context.isJobDispatched() && this.requiresDispatch(context)) {
            this.logger.error(this.moduleType, ".onContextChange - hit the corner case where a DispatchingModule instance requires dispatch for q=", context.getSearch(), "timeRange=", context.getTimeRange());
            this._fireDispatch();
        } else {
            $super(context);
        }
    },
    /**
     * override me to handle job progress notifications.
     * (only for the job currently referenced by this.baseContext)
     */
    onJobProgress: function(event) {},
    /**
     * override me to handle the global notification that the job is done.
     * (only for the job currently referenced by this.baseContext)
     */
    onJobDone: function(event) {},
    
   
    onJobStatusChange: function(event, status) {
        if (status == 'cancel') {
            this.reset();
        }
    }
});
Splunk.Module.DispatchingModule.isAbstract = true;
