
//put Module in the namespace if it isnt already there.
Splunk.namespace("Module");

Splunk.Module.BaseReportBuilderField = $.klass(Splunk.Module, {
    initialize: function($super, container) {
        $super(container);
        this.messenger = Splunk.Messenger.System.getInstance();
        this.lockBaseContext = true;
        this._selectElement = $("select",this.container)
            .change(this.onUIChange.bind(this));
    },
    onUIChange: function(event) {
        this.pushContextToChildren();
    },
    onLoadStatusChange: function(statusInt) {
        if (statusInt == Splunk.util.moduleLoadStates.MAX) {
            this.lockBaseContext = false;
        }
    },
    /** if nobody has ever given us a
     * a context reference, we cannot let getBaseContext() pull it from above.
     * Thisis because in resurrection cases
     * a) this can trigger nasty jaBridge exceptions
     * b) this can trigger race conditions in resurrection 
     */
    getBaseContext: function($super) {
        // only call the $super if we're unlocked, or we have a local property.
        if (this.baseContext || !this.lockBaseContext) return $super();
        return false;
    },
    _showIfCompatible : function() {
        var context = this.getBaseContext();
        if (!context) {
            this.logger.debug(this.moduleType + ", No base context. Hiding  Foo");
            this.hide();
            return false;
        }

        if (this.isCompatibleWithSettings(this.getBaseSettings())) {
            
            var plotIntention = context.getIntentionReference("plot");
            if (this.isCompatibleWithIntention(plotIntention)) {
                this.show();
            } else {
                this.hide();
            }
        } else {
            this.hide();
        }
    },

    onSettingsChange: function(settingsMap) {
        this._showIfCompatible();
    },
    
    isCompatibleWithSettings: function() {
        return !this.getSetting("reportBuilderAdvancedMode");
    },
    isCompatibleWithIntention: function(plotIntention) {return true;},
    onContextChange: function($super) {
        this._showIfCompatible();
    },
       
    getPlotIntentionArg: function() {
        this.logger.error(this.moduleType, "getPlotIntentionArg not implemented");
    },
       
    absorbPlotIntentionArg: function(intentionArg) {
        this.logger.error(this.moduleType, "absorbPlotIntentionArg not implemented");
    },

    applyContext: function(context) {
        // TODO - getIntention should maybe be replaced or augmented with a getIntention method.
        //        constantly popping and pushing it on is odd and seems to be happening a lot.
        var plotIntention = context.getIntentionReference("plot");
        
        //this.logger.debug("before ", this.moduleType, " applyContext, the arg is " + $.param(plotIntention["arg"]));
        // order is important because absorbPlotIntentionArg needs to happen even if ultimately the intention is not compatible.
        if (plotIntention && this.absorbPlotIntentionArg(plotIntention["arg"]) && this.isCompatibleWithIntention(plotIntention)) {
            //this.logger.debug("after successful applyContext, " + $.param(plotIntention["arg"]));
            this.logger.debug(this.moduleType, " absorbPlotIntention returned true. Showing Foo.");
            this.show();
            return true;
        } else {
            //this.logger.debug("after UNsuccessful  ", this.moduleType, " applyContext, " + $.param(plotIntention["arg"]));
            this.logger.debug(this.moduleType, " absorbPlotIntention returned false. Hiding  Foo.");
            this.hide();
            return false;
        }
    },
    
    getModifiedContext: function() {
        var argumentDict = this.getPlotIntentionArg();
        // the fact that we preserve the jobInstance is a little unusual.
        // typically modules that know they are modifying searchContext, will
        // bleach away the Job information.   However for BaseReportBuilderField
        // instances, it is very helpful to pass along the preexisting job, as
        // so downstream modules can obtain the field list and other information
        // about the 'dataset'.
        var context = this.getBaseContext();
        
        var plotIntention = context.getIntentionReference("plot");
        if (!plotIntention) return context;

        
            
        if (argumentDict && this.isCompatibleWithIntention(plotIntention)) {
            
            for (var key in argumentDict) {
                if (plotIntention["arg"].hasOwnProperty(key)) {
                    this.logger.error(this.moduleType, "plot intention already has a value for " , key);
                    if (this.getFieldType) {
                        this.logger.error(this.moduleType, this.getFieldType(), " plot intention already has a value for " , key);
                    }
                }
                plotIntention["arg"][key] = argumentDict[key];
            }
            // add the plot intention back before returning
            
            return context;
        }
        else {
            return this.getBaseContext();
        }
    },
    lazyAdd: function(arg, asKey, map) {
        if (arg) {
            map[asKey] = arg;
        }
    },
    show: function($super, invisibilityMode) {
        invisibilityMode = invisibilityMode || "report_builder";
        return $super(invisibilityMode);
    },
    hide: function($super, invisibilityMode) {
        invisibilityMode = invisibilityMode || "report_builder";
        return $super(invisibilityMode);
    },
    puntToAdvancedMode: function() {
        
        // advancedModeSwitcher gave this to us in case we needed to push the big red button.
        var advancedModeCallback = this.getSetting("switchToAdvancedModeCallback");
        if (advancedModeCallback) {
            this.messenger.send('info', 'splunk.search', _("This search can only be loaded in the advanced mode."));
            advancedModeCallback(true);
        } else {
            this.messenger.send('info', 'splunk.search', _("This search can not be properly loaded in this view."));
        }
    }
});
Splunk.Module.BaseReportBuilderField.isAbstract = true;
