
Splunk.Module.ReportBuilderSearchField = $.klass(Splunk.Module.BaseReportBuilderField, {


    /*
     * overriding initialize to set up references and event handlers.
     */
    initialize: function($super, container) {
        $super(container);

        this.logger = Splunk.Logger.getLogger("report_builder_search_field.js");


        this.input = $('textarea', this.container)
            .keypress(function(uselessEvent) {this.pushContextToChildren()}.bind(this))
            .blur(function(uselessEvent) {this.pushContextToChildren()}.bind(this));
        this.setAdvancedMode(false);
    },
    setInputField: function(searchStr) {
        this.input.val(Splunk.util.stripLeadingSearchCommand(searchStr));
    },
    onParseFailure: function() {
        this.messenger.send('info', 'splunk.search', _("Splunk cannot construct a working search from your entry. Please try again."));
    },
    onContextChange: function() {
        this.setAdvancedMode(this.getSetting("reportBuilderAdvancedMode"))
        // if were in advanced mode, we actually completely ignore the base context, and instead hold onto the
        // secret property we stashed away when we snuck into the advanced mode.
        var context = this._advancedModeContext || this.getBaseContext();
        
        context.absorbIntentions(this.setInputField.bind(this), this.onParseFailure.bind(this));
    },
    getModifiedContext: function() {
        var baseContext = this.getBaseContext();
        if (!baseContext) {
            // there's a transient fail here during resurrection. It seems to be immediately cleared by subsequent win, 
            // TODO - needs further investigation.
            this.logger.error("Assertion failed ", this.moduleType, ".getModifiedContext() called but module has null baseContext. Exiting.");
            return false
        }
            
        

        var plotIntention = baseContext.getIntentionReference("plot");

        if (this.input.attr("disabled")) return baseContext;
        else {
            var context = new Splunk.SearchContext(this.input.val(), baseContext.getTimeRange());
            return context;
        }
    },
    setAdvancedMode: function(bool) {
        if (bool) {
            this.input.removeAttr("disabled");
            this._advancedModeContext = this.getModifiedContext();
        } else {
            this._advancedModeContext = null;
            this.input.attr("disabled", "true");
        }
    },
    onSettingsChange: function(settingsMap) {
        this.setAdvancedMode(this.getSetting("reportBuilderAdvancedMode"));
    },
    applyContext: function(context) {
        // we might conceivably have upstream click interaction in a reporting view someday
        // so we dont rewrite our field for non-resurrected contexts.
        if (context.isResurrected()) {
            this.setInputField(context.getSearch());
        }
    }
});
