import cherrypy
import logging
import controllers.module as module
import splunk
import datetime
from splunk.appserver.mrsparkle.lib import i18n, util, decorators
import splunk.entity

# TOTAL FAIL THAT THIS IS HERE. IT BASICALLY EMULATES controllers/view.py
# WHICH CANNOT BE IMPORTED.
DEFAULT_DISPLAYVIEW = 'flashtimeline'

logger = logging.getLogger('splunk.modules.job_manager')

class JobManager(module.ModuleHandler):
    
    SECONDARY_SORT_KEY = 'createTime'
    
    STATUS_SORT_ORDER = {
        'isPaused': 1,
        'isFinalized': 2,
        'isDone': 3
    }
    
    TYPE_SORT_ORDER = {
        'search': 0,
        'report': 1,
        'scheduledSearch': 2,
        'scheduledReport': 3
    }

    SCHEDULER_DELEGATE_KEY = 'scheduler'
    FILTER_NONE_CHAR = '*'
    APPS_KEY = 'app'
    VIEW_KEY = 'ui_dispatch_view'

    def rankJobStatus(self, job):
        '''Return a numerical value based on the STATUS_SORT_ORDER map defined in the class.'''
        if job['isFinalized']: return self.STATUS_SORT_ORDER['isFinalized']
        if job['isDone']: return self.STATUS_SORT_ORDER['isDone']
        if job['isPaused']: return self.STATUS_SORT_ORDER['isPaused']
        return 0 # running jobs
    
    
    def rankJobTTL(self, job):
        '''
        Return a numerical value based on the job properties that defines ttl.
        Running > 0
        Saved = 0
        Expired = -1
        '''
        if job['isSaved']: return 0
        return job['ttl']

    #def rankJobType(self, job):
    #    if job['delegate'] and job['delegate'].lower() == self.SCHEDULER_DELEGATE_KEY:
    #        if job['reportSearch']: return self.TYPE_SORT_ORDER['scheduledReport']
    #        else: return self.TYPE_SORT_ORDER['scheduledSearch']
    #    if job['reportSearch']: return self.TYPE_SORT_ORDER['report']
    #    return self.TYPE_SORT_ORDER['search']
        
    @decorators.set_cache_level('never')
    def generateResults(self, count=10, offset=0, sort_by='createTime', sort_dir='desc', sid=None, filters=None, search=None, **kw):
        '''
        Returns an html fragment or json string depending on the incoming request params.
        Adds some extra properties onto each job dictionary representation.
        '''
        
        # Calling of the _() function has to be done in the context
        # of an incoming request where the desired locale is known.
        # Without the local we cannot call _() accurately, thus these
        # functions are defined inside of generateResults(). Less than
        # ideal but it works for now.


        # TRANS: String representation of job states
        def getJobStatusString(job):
            if job['isFinalized']: return _('Finalized')
            if job['isDone']: return _('Done')
            if job['isPaused']: return _('Paused')
            return _("Running (%.0f%%)" % (float(job['doneProgress']) * 100.0) )

        # This may no longer be used
        # TRANS String representations of job types
        def getJobTypeString(job):
            if job['delegate'] and job['delegate'].lower() == self.SCHEDULER_DELEGATE_KEY:
                if job['reportSearch']: return _('Scheduled report')
                else: return _('Scheduled search')
            if job['reportSearch']: return _('Report')
            return _('Search')
        
        # TRANS String representation of the job's expires time
        def getJobExpiresString(job, tz):
            if job['isSaved']: return _('Saved')
            elif job['ttl'] < 0: return _('Expired')
            else:
                delta = datetime.timedelta(seconds=job['ttl'])
                now = datetime.datetime.now(tz)
                future = now + delta
                return i18n.format_datetime(future)

        def getJobApp(job):
            if 'eai:acl' in job and self.APPS_KEY in job['eai:acl']:
                return job['eai:acl'][self.APPS_KEY]
            return ''

        savedSearchViews = {}
        def getJobURI(job):
            sid = job.get('sid')
            app = getJobApp(job)
            view = None
            uri = "/app/%(app)s/%(view)s"

            if job.get('request'):
                view = job.get('request').get(self.VIEW_KEY)

            # First see if we have all the necessary view data
            if sid != None and app != None and view != None:
                return self.controller.make_url(uri % { 'app': app, 'view': view }, _qs=dict(sid=sid))

            # Next see if it's a saved search
            isSavedSearch = job.get('isSavedSearch')

            label = job.get('label')
            owner = None
            if 'eai:acl' in job and 'owner' in job['eai:acl']:
                owner = job['eai:acl']['owner']

            if isSavedSearch and label and owner:
                if label in savedSearchViews:
                    view = savedSearchViews[label]
                else:
                    try:
                        savedSearch = splunk.entity.getEntity('saved/searches', label, namespace=app, owner=owner)
                        view = savedSearch.get('displayview')
                        if view: savedSearchViews[label] = view
                    except splunk.ResourceNotFound, e:
                        logger.debug("The saved search '%s' could not be found and was likely deleted. The default view will be used for this job." % label)
                        pass 

            # If we got here, just use the default view
            if view == None:
                view = DEFAULT_DISPLAYVIEW

            # Now can we construct the uri?
            if sid != None and app != None and view != None:
                return self.controller.make_url(uri % { 'app': app, 'view': view }, _qs=dict(sid=sid))

            # We cannot end up here right now, but if we do, render no link
            return None

        try:
            count = int(count)
            offset = int(offset)
            # TODO
            # Rethink this, probably can't assume it's a str obj
            if sid and type(sid) == str: sid = [sid]
            if filters and isinstance(filters, basestring): filters = [filters]
        except ValueError, e:
            logger.warn("Cannot coerce count: %s or offset: %s into integers." % (count, offset))
            count = 10
            offset = 0
        
        try:
            if (search):
                jobs = splunk.search.listJobs(normalizeValues=True, search=search)
            else:
                jobs = splunk.search.listJobs(normalizeValues=True)
        except splunk.ResourceNotFound, e:
            raise cherrypy.HTTPError(status=404, message="Cannot find job listing." % sid)
        
        # Filter; only AND filtering for now
        # Can filter for the presence of a field by just supplying the field  =>  user
        # Can filter for a field with particular valued by supplying a field and value  =>  user:admin
        if filters:
            filter_values = [pairs.split(':') for pairs in filters if not pairs == '']
            for val in filter_values:
                if len(val) > 1 and val[1] == self.FILTER_NONE_CHAR: continue
                if val[0] == self.APPS_KEY:
                    tmp_jobs = []
                    for job in jobs:
                        if 'eai:acl' in job:
                            if len(val) == 1:
                                if job['eai:acl'].has_key(self.APPS_KEY):
                                    tmp_jobs.append(job)
                            elif len(val) == 2:
                                if job['eai:acl'].has_key(self.APPS_KEY) and job['eai:acl'][self.APPS_KEY] == val[1]:
                                    tmp_jobs.append(job)
                    jobs = tmp_jobs
                else:                    
                    if len(val) == 1:
                        jobs = [job for job in jobs if job.has_key(val[0]) and not job[val[0]] == None]
                    if len(val) == 2:
                        jobs = [job for job in jobs if job.has_key(val[0]) and job[val[0]] == val[1]]
 
        # Sorting
        reverse = False
        if sort_dir == 'desc':
            reverse = True
        try:
            if sort_by == 'status':
                jobs = sorted(jobs, key=lambda x: (self.rankJobStatus(x), x[self.SECONDARY_SORT_KEY]), reverse=reverse)
            elif sort_by == 'ttl':
                jobs = sorted(jobs, key=lambda x: (self.rankJobTTL(x), x[self.SECONDARY_SORT_KEY]), reverse=reverse)
            # Job TYPE may be removed
            #elif sort_by == 'type':
            #    jobs = sorted(jobs, key=lambda x: (self.rankJobType(x), x[self.SECONDARY_SORT_KEY]), reverse=reverse)
            elif sort_by == 'app':
                jobs = sorted(jobs, key=lambda x: (getJobApp(x), x[self.SECONDARY_SORT_KEY]), reverse=reverse)
            else:
                jobs = sorted(jobs, key=lambda x: (x[sort_by], x[self.SECONDARY_SORT_KEY]), reverse=reverse)
        except KeyError:
            logger.warn('Tried sorting jobs by key %s, but that key is invalid.' % sort_by)
        
        # Get the total length before paginating
        total_length = len(jobs)
        
        # Paginate
        if (sid):
            if type(sid) != list: sid = [sid]
            jobs = [job for job in jobs if job['sid'] in sid]
        else:
            jobs = jobs[offset : offset + count]

        # Internalize dates and numbers, stringify special properties.
        # Add some properties for pretty printing.
        # In general, do a lot of silly one off things...
        for job in jobs:
            runtime = util.get_time(seconds=job['runDuration'])
            job['RUNTIME'] = '%02i:%02i:%02i' % (runtime[2], runtime[3], runtime[4])
            job['EXPIRES'] = getJobExpiresString(job, job['createTime'].tzinfo)
            job['STATUS'] = getJobStatusString(job)
            # TYPE may be removed
            #job['TYPE'] = getJobTypeString(job)
            job['APP'] = getJobApp(job)
            job['eventCount'] = i18n.format_number(job['eventCount'])
            job['createTime'] = i18n.format_date(job['createTime'], 'short') + ' ' + i18n.format_time(job['createTime'])
            job['EARLIEST_TIME'] = ' '.join([i18n.format_date(job['earliestTime'], 'short'), i18n.format_time(job['earliestTime'])])
            job['LATEST_TIME'] = ' '.join([i18n.format_date(job['latestTime'], 'short'), i18n.format_time(job['latestTime'])])
            job['URI'] = getJobURI(job)

        cherrypy.response.headers['X-Splunk-List-Length'] = total_length
        return self.controller.render_template('jobs/JobManager_tr_template.html', {'data': jobs})
