Splunk.Module.JobStatus = $.klass(Splunk.Module.DispatchingModule, {

    BACKGROUND_CLASS_NAME: "background",
    CANCEL_CLASS_NAME: "cancel",
    CLOSE_CLASS_NAME: "close",
    FINALIZE_CLASS_NAME: "finalize",
    OUTPUT_CLASS_NAME: "output",
    PAUSE_CLASS_NAME: "pause",
    RESULTS_LINK_CLASS_NAME: "resultsLink",
    SAVE_CLASS_NAME: "save",
    UNPAUSE_CLASS_NAME: "unpause",
    PAUSED_CLASS_NAME: "paused",
    FINALIZING_CLASS_NAME: "finalizing",
    RUNNING_CLASS_NAME: "running",
    COMPLETE_CLASS_NAME: "complete",
    SAVED_SEARCH_POPUP_SELECTOR: ".savedsearchpopup",
    SAVED_SEARCH_POPUP_TITLE: "Save Search",
    EMPTY_CLASS_NAME: "empty",

    initialize: function($super, container){
        $super(container);
        this.logger = Splunk.Logger.getLogger("Splunk.Module.JobStatus");
        this.container.click(this.onUIEvent.bind(this));//Bootstrap top level event dispatcher.
        this.currentState = this.EMPTY_CLASS_NAME;
        this.headerClass = "";
        this.links = $("ul", this.container);
        this.header = $(".splHeader", this.container);
        // the resultsLink param offers a way for the user to view the search results
        // in a different view.
        // the primary use of this param is to launch report builder, in the report_builder_define_data view,
        this.setupResultsLink();
    },
    setState: function(className, optionalJob) {
        
        this.links.removeClass(this.currentState);
        this.currentState = className;
        this.links.addClass(this.currentState);

        this.updateHeader(className, optionalJob || null);
    },
    /**
     * given one of 'events', 'scanned', 'progress' or 'results', this method will 
     * return a human-readable header string. 
     * As a part of a strategy to preserve CSS backward compatibility, we have to return an array 
     * of dictionaries instead of a simple array of strings.  This is so that the 'name' 
     * you see here can end up rendered as a classname on the corresponding DOM element
     */
    getHeaderFragment: function(name, job) {
        // kind of strange to return a dict, but we need to preserve the classname, for backward compatibility with the skins. 
        //  ie.  all of the little em's and h2's have class="progress"  etc. and the skins unfortunately picked that up.
        var returnDict = {"name": name}
        switch (name) {
            case "events":
                var events = job.getEventCount();
                
                if (job.isDone()) {
                    returnDict["text"] = sprintf(ungettext('%s matching event', '%s matching events', events), format_number(events));
                } else {
                    returnDict["text"] = sprintf(/* TRANS: &#8805 is the greater than or equal to symbol */ungettext('&#8805; %s matching event', '&#8805; %s matching events', events), format_number(events));
                }
                break;
            case "scanned":
                var scanned = job.getScanCount();
                returnDict["text"] = sprintf(ungettext('%s scanned event', '%s scanned events', scanned), format_number(scanned));
                break;
            case "progress":
                var progress = Math.round(job.getDoneProgress() * 100);
                returnDict["text"] = sprintf(_('%s%% complete'), progress);
                break;
            case "results":
                var results = job.getResultCount();
                returnDict["text"] = sprintf(ungettext('%s result', '%s results', results), format_number(results));
                break;
            default:
                this.logger.error("getHeaderFragment - unknown name provided - ", name);
                returnDict["text"] = "";
        }
        return returnDict;
    },
    /**
     * Called when we need to update the headers and <em> elements. 
     * Generally called onJobProgress, onJobDone, and onContextChange.
     *
     */
    updateHeader: function(className, optionalJob) {
        
        var headers = [];
        
        // CSS compatibility issue 1 - in the old implementation the 'splHeader' class got a custom 'state' classname 
        // so we have to preserve that.  Here we remove the old one and later we'll add the new one.
        this.header.removeClass(this.headerClass);

        var job = optionalJob || this.getBaseContext().job;

        switch (className) {
            case "finalizing":
                // CSS compatibility issue 2 - all of the h2 and em elements had their own 
                // class="progress"   class="scanned"  etc..  
                // to allow getHeaderFragment to do this *somewhat* generically, 
                // we are forced here to use a more complex headers array. 
                headers.push({"name":"","text":_('Your search is finalizing...')});
                this.headerClass = "running";
                break;
            case "pausing":
                headers.push({"name":"","text":_('Your search is pausing...')});
                break;
            case "paused":
                headers.push({"name":"","text":_('Your search is paused.')});
                this.headerClass = "paused";
                break;
            case "running":
                this.headerClass = "running";
                //TODO - it would be cool to add a 
                // <param name="transformingDisplay">progress,results</param>
                // and 
                // <param name="nonTransformingDisplay">events,scanned</param>
                // so that you could really customize your own view to give you whatever
                // ie <param name="transformingDisplay">progress,scanned,results</param>
                // or going the other way, <param name="transformingDisplay">progress</param>
                if (job.areResultsTransformed()) {
                    headers.push(this.getHeaderFragment("progress", job));
                    headers.push(this.getHeaderFragment("results", job));
                } else {
                    headers.push(this.getHeaderFragment("events", job));
                    headers.push(this.getHeaderFragment("scanned", job));
                }
                break;
            case "complete":
                this.headerClass = "complete";
                headers.push(this.getHeaderFragment("events", job));
                break;
            case "empty":
                break;
            default : 
                this.logger.warn("there should be no unspecified cases in this switch. className=", className);
        }
        this.header.addClass(this.headerClass);
        this.header.html('');
        // The first element in the headers array is always rendered as an h2 element. 
        if (headers.length>0) {
            h2 = $("<h2>")
                .html(headers[0]["text"])
                .addClass(headers[0]["name"]);
            
            var onDeckCircle = $("<div>");

            onDeckCircle.append(h2);
        
            for (var i=1; i<headers.length; i++) {
                // follow the css convention of using .splPipe on our | chars.
                onDeckCircle.append($('<span class="splPipe">|</span>'));
                // subsequent items in the headers array are rendered as <em> elements, and are given a matching classname.
                emElement = $("<em>")
                    .addClass(headers[i]["name"])
                    .text(headers[i]["text"]);
                onDeckCircle.append(emElement);
            }
            this.header.html(onDeckCircle.html());
        }
        
    },
    
    /**
     * Called when a new search comes down from above.
     */
    onContextChange: function($super){
        this.header.html('');
        this.show();

        var context = this.getBaseContext();
        var job = context.job;
        var events = context.job.getEventCount();
        if (job.isDone()){
            this.setState(this.COMPLETE_CLASS_NAME)
        } else if (job.isCanceled()){
            this.setState(this.EMPTY_CLASS_NAME)
        } else if (job.isPaused()){
            this.setState(this.PAUSED_CLASS_NAME)
            
        } else if (job.isFinalized()){
            this.setState(this.COMPLETE_CLASS_NAME);
            
        } else if (job.isRunning()){
            this.setState(this.RUNNING_CLASS_NAME) 
            
        } else{
            this.setState(this.EMPTY_CLASS_NAME);
        }
        $super();
    },
    /**
     * unlike most implementations of onBeforeJobDispatched, we dont actually need to modify the ancestor's context
     * before dispatched, we just need to know so we can put up the UI for the 'running' state
     * This is so that we can give immediate user feedback rather than waiting for the POST to return.
     */
    onBeforeJobDispatched: function(context) {
        var jobOverride = new Splunk.Job();
        this.setState(this.RUNNING_CLASS_NAME, jobOverride);
    },
    /**
     * Handle job complete and update UI.
     */
    onJobDone: function(event){
        var context = this.getBaseContext();
        
        var events = context.job.getEventCount();
        if (this._params["hideOnJobDone"] == "True") {
            this.hide();
            this.setState(this.EMPTY_CLASS_NAME);
        } else {
            this.setState(this.COMPLETE_CLASS_NAME);
        }
    },
    /**
     * Handle job progress.
     */
    onJobProgress:function(){
        var context = this.getBaseContext();
        if(!context.job.isPaused() && !context.job.isFinalized()){
            this.setState(this.RUNNING_CLASS_NAME);
            this.updateResultsLink(context.job.getSearchId());
        }
    },
    /**
     * Handle the in-between states of the links, after a user clicks on
     * a control but before the control is acknowledged by the server.
     *
     * @param {Object} event The event that fired the onJobStatusChange (see jobber.js)
     * @param {String} status The status event identifier.
     */
    onJobStatusChange: function(event, status){
        switch(status){
            case "pause":
                this.setState(this.PAUSED_CLASS_NAME);
                break;
            case "unpause":
                this.setState(this.RUNNING_CLASS_NAME);
                break;
            case "finalize":
                this.setState(this.FINALIZING_CLASS_NAME);
                break;
            // NOTE - the autocancellation of previous jobs will not hit this, 
            // because the previous job is only cancelled after the next one is POSTed
            // which means that by that time the previous job wont match this module's sid.
            case "cancel":
                this.setState(this.EMPTY_CLASS_NAME);
                break;
            default:
                this.logger.warn("Job status change that is not being monitored.");
        }
    },

    /**
     * Handle a save action. Spawns a popup window from  EAI, w00t!.
     *
     * @param {Object} event A jQuery event.
     */
    onSave: function(event){
        var formContainer = $(this.SAVED_SEARCH_POPUP_SELECTOR)[0];
        var title = this.SAVED_SEARCH_POPUP_TITLE;
        Splunk.Popup.createSavedSearchForm(formContainer, title, this.getBaseContext());
        return false;
    },

    /**
     * Top level UI event listener and dispatcher.
     *
     * @param {Object} event A jQuery event.
     * @return Control the cancellation of events triggered.
     * @type Boolean
     */
    onUIEvent: function(event){
        var eventType = event.type;
        var eventTarget = $(event.target);
        if((eventType==="click") && eventTarget.is("a")){
            var job = this.getBaseContext().job;
            if(eventTarget.hasClass(this.PAUSE_CLASS_NAME)){
                job.pause();
                return false;
            }else if(eventTarget.hasClass(this.UNPAUSE_CLASS_NAME)){
                job.unpause();
                return false;
            }else if(eventTarget.hasClass(this.FINALIZE_CLASS_NAME)){
                job.finalize();
                return false;
            }else if(eventTarget.hasClass(this.CANCEL_CLASS_NAME)){
                if (this.baseContext && this.baseContext.isJobDispatched()) {
                    this.baseContext.job.cancel();
                }
                return false;
            }else if(eventTarget.hasClass(this.BACKGROUND_CLASS_NAME)){
                job.save();
                $.each(this.getModulesInTree(), function(idx, module){
                    module.reset();
                });
                var messenger = Splunk.Messenger.System.getInstance();
                messenger.send('info', "splunk.search.job", _('Your search job has been backgrounded. To retrieve it, visit the Jobs page.'));
                
                return false;
            }else if(eventTarget.hasClass(this.CLOSE_CLASS_NAME)){
                this.setState(this.EMPTY_CLASS_NAME);
                return false;
            }else if(eventTarget.hasClass(this.SAVE_CLASS_NAME)){
                return this.onSave(event);
            }else if(eventTarget.hasClass(this.RESULTS_LINK_CLASS_NAME)){
                return this.onBuildReport(event);
            }
        }
        return true;
    },
    /**
     * Reset the UI to its original state.
     */
    resetUI: function(){
        this.setState(this.EMPTY_CLASS_NAME);
    },
    
    /**
     * BEGIN: please audit link, label, popup system.
     */
    setupResultsLink: function() {
        var paramDict = this._params[this.RESULTS_LINK_CLASS_NAME];
        // if there's no params, then the mako template will not have rendered
        // the link anyway so were done.
        if (!paramDict) {
            return false;
        }
        // we update the actual href attr, so that even unusual click interactions
        // like right-clicking will still have correct behaviour.
        var link = $("." + this.RESULTS_LINK_CLASS_NAME, this.container);
        link.text(paramDict["label"]);
        if (paramDict.hasOwnProperty("popup")) {
            // rel="popup" looks weird but it's a best practice for accessibility.
            // we check for it later onclick.
            link.attr("rel", "popup");
        }
    },
    updateResultsLink: function(sid) {
        var context = this.getBaseContext();
        var link = $("." + this.RESULTS_LINK_CLASS_NAME, this.container);
        var paramDict = this._params[this.RESULTS_LINK_CLASS_NAME];
        if (link && paramDict) {
            var viewTarget = paramDict["viewTarget"];
            if (context.job.areResultsTransformed() && paramDict.hasOwnProperty("transformedResultsViewTarget")) {
                viewTarget = paramDict["transformedResultsViewTarget"];
                // we almost dont have to worry about rewriting this because there is now a 'running' status
                // vs a 'runningReport'  status.  However we also need to rewrite the link in the 'complete' status 
                // and 'paused'  status etc so here we just rewrite them all.
                link.text(_("Show report"));
            }
            else {
                link.text(_("Build report"));
            }
            var path = Splunk.util.make_url("/app/", Splunk.util.getCurrentApp(), viewTarget);
            var url = path + "?" + $.param({"sid": sid});
            link.attr("href", url);
            link.show();
        }
    },
    /**
     * Handle a build report action. Spawn a popup and cancel execution of the anchor element or allow the execution of
     * anchor element to change the address location.
     *
     * @param {Object} event The DOM event triggered.
     * @type Boolean
     */
    onBuildReport: function(event){
        var eventTarget = $(event.target);
        var viewTarget = this._params[this.RESULTS_LINK_CLASS_NAME]["viewTarget"];
        
        var context = this.getBaseContext();
        var args = {};
        // mark this job as off limits to the auto-cancellation. 
        // NOTE - we cant let the base_sid be cancelled in this window cause report builder might be using it
        // and we cant let it be cancelled in the report_builder view because THIS view might be using it. 
        // upshot is that these sids will hang around until their TTL kills them. 
        // to fix that we'll eventually have to have these windows talking to eachother and have the last one out 
        // turn out the lights or something....
        context.job.setAsAutoCancellable(false);
        args["base_sid"] = context.job.getSearchId();

        // if this search already has transforming commands,  and we're configured to send them to step 2, do so.
        if (context.job.areResultsTransformed() ) {
            if (this._params[this.RESULTS_LINK_CLASS_NAME].hasOwnProperty("transformedResultsViewTarget")) {
                viewTarget = this._params[this.RESULTS_LINK_CLASS_NAME]["transformedResultsViewTarget"];
            }
        } else {
            var placeholderIntention = {"name": "plot", "arg": {"mode": "timechart", "fields" : [["count", "__events"]]}};
            context.abandonJob();
            context.addIntention(placeholderIntention);
        }
        context.sendToView(viewTarget, args, false, $(eventTarget).attr("rel")=="popup", {autosize: true});
        return false;
    }
    /**
     * END: please audit link, label, popup system.
     */
});
