
Splunk.Module.AbstractInternalSearch = $.klass(Splunk.Module, {

    DEFAULT_STATUS_BUCKETS: 300,

    _outerIntentions: [],

    initialize: function($super, container) {
        $super(container);
        this.logger = Splunk.Logger.getLogger("Splunk.module.AbstractInternalSearch.js");
        this.messenger = Splunk.Messenger.System.getInstance();

        this._internalContext = null;

        if (this.getParam('search') || this.getParam('savedSearch')) {
            this.dispatchInternalJob();

            $(document).bind('jobProgress', function(event, job) {
                if (this._internalContext && 
                    this._internalContext.job &&
                    this._internalContext.job.getSearchId() == job.getSearchId()) {
                    this.onInternalJobProgress(event);
                }
            }.bind(this));

             //see simlar comment above for jobProgress
            $(document).bind('jobDone', function(event, job) {
                if (this._internalContext && 
                    this._internalContext.job &&
                    this._internalContext.job.getSearchId() == job.getSearchId()) {
                    this.onInternalJobDone(event);
                }
            }.bind(this));

            $(document).bind('jobStatusChanged', function(event, sid, status) {
                if (this._internalContext && 
                    this._internalContext.job &&
                    this._internalContext.job.getSearchId() == sid) {
                    this.onInternalJobStatusChanged(event);
                }
            }.bind(this));
        } else {
            this.messenger.send('error', 'Splunk.module.AbstractInternalSearch', sprintf('SearchLister module %(moduleId)s was not given a search or a saved search name to power itself and thus cannot render any data.', {moduleId: this.moduleId}));
        }
    },

    dispatchInternalJob: function(force) {
        // First cancel any running jobs
        if (this._internalContext && 
            this._internalContext.job &&
            this._internalContext.job.getSearchId()) {
            this._internalContext.job.cancel();
        }

        var search = this.getParam('search');
        var savedSearch = this.getParam('savedSearch');
        var entityName = this.getParam('entityName');
        var statusBuckets = this.getParam('statusBuckets');
        var useHistory = this.getParam('useHistory');

        if (search) {
            var earliest = this.getParam('earliest');
            var latest = this.getParam('latest');

            this._internalContext = new Splunk.SearchContext(search);

            // Properly set the timerange, if provided
            if (earliest || latest) {
                var range = new Splunk.TimeRange(earliest, latest);
                this._internalContext.setTimeRange(range);
            }
            
        } else if (savedSearch) {
            // Case 1: we get a bad SavedSearch name. 
            if (!this.getParam('jsonContext')) {
                this.displayInlineErrorMessage(sprintf(_('Could not find a saved search named "%(savedSearch)s".'), this._params));
                return;
            }
            
            // Valid saved search name so we should go try to retrieve it
            this._internalContext = Splunk.SearchContext.resurrect(this.getParam('jsonContext'));
            this._internalContext.job.setAsAutoCancellable(false);
            
            // Case 2: We get a valid saved search but no job is returned and we have useHistory == true
            if (this._internalContext.isResurrected() &&
                this._internalContext.isJobDispatched() == false &&
                useHistory.toLowerCase() == 'true') {
                
                var msg = '';
                if (window.$C && window.$C.hasOwnProperty('SPLUNKD_FREE_LICENSE') && window.$C['SPLUNKD_FREE_LICENSE']) {
                    msg = sprintf(_('The module "%(moduleType)s" has its useHistory flag set to True, which is not compatible with a free license. Try setting useHistory to Auto or False.'), {'moduleType': this.moduleType});
                } else {
                    msg = sprintf(_('Could not find a job in the saved search "%(savedSearch)s" history.'), this._params);
                }
                this.displayInlineErrorMessage(msg);
                return;
            }
        } else {
            this.logger.error('Assertion failed, no search or saved search provided for module=', this.moduleType);
            return
        }

        if (this._internalContext) {
            var failHandler = function(failedContext) {
                this.logger.error(this.moduleType, "Internal context failed to dispatch job for search=", failedContext.getSearch());
                this.displayInlineErrorMessage(sprintf(_('The internal search "%(searchStr)s" for the "%(moduleType)s" module failed.'), {searchStr: failedContext.getSearch(), moduleType: this.moduleType}));
            }.bind(this);

            var successHandler = function(contextDispatched) {
                this.logger.debug("Successfully dispatched the internal search=", contextDispatched.getSearch(), 'for module=', this.moduleType);
            }.bind(this);

            // If we're looking for events make sure there are some statusBuckets
            if (entityName == 'events') {
                if (!statusBuckets) statusBuckets = this.DEFAULT_STATUS_BUCKETS;
                this._internalContext.setMinimumStatusBuckets(statusBuckets);                
            }

            var useIntentions = this.getParam('applyOuterIntentionsToInternalSearch');
            var len = this._outerIntentions.length;
            if (useIntentions && len > 0) {
                for(var i=0; i<len; i++) {
                    this._internalContext.addIntention(this._outerIntentions[i]);
                }
                this._outerIntentions = []; //reset
            }

            if (force || !useIntentions) {
                // This protects against dispatching a stubbed out job created by the resurrection framework. 
                if (!this._internalContext.isResurrected() || 
                    (this._internalContext.isResurrected() && useHistory.toLowerCase() == 'false') ||
                    (this._internalContext.isResurrected() && useHistory.toLowerCase() == 'auto' && !this._internalContext.isJobDispatched())) {
                    this._internalContext.dispatchJob(successHandler, failHandler, this.getGroupName());
                    this.onInternalJobDispatched();
                }
            }
        } else {
            this.logger.error('Somehow AbstractInternalSearch never got an internal context with which to dispatch a job. Not much we can do here.');
        }
    },

    onContextChange: function() {
        var useIntentions = this.getParam('applyOuterIntentionsToInternalSearch');
        if (useIntentions) {
            var outerContext = this.getBaseContext();
            var len = outerContext._intentions.length;
            if (len > 0) {
                for(var i=0; i<len; i++) {
                    this._outerIntentions.push($.extend(true, {}, outerContext._intentions[i]));
                }
                this.dispatchInternalJob(true);
            }
        }
    },
    
    onInternalJobProgress: function() {},
    onInternalJobDone: function() {},
    onInternalJobStatusChanged: function() {},
    onInternalJobDispatched: function() {}
});
