//put Module in the namespace if it isnt already there.
Splunk.namespace("Module");

Splunk.Module.TitleBar = $.klass(Splunk.Module.DispatchingModule, {

    SAVED_SEARCH_FORM_CONTAINER_CLASS: '.savedSearchForm',
    SAVED_SEARCH_FORM_TITLE: _('Save Search'),

    SHARE_LINK_FORM_CONTAINER_CLASS: '.shareLinkForm',
    SHARE_LINK_FORM_TITLE: _('Get Link to Results'),

    EVENTTYPE_FORM_CONTAINER_CLASS: '.eventtypeForm',
    EVENTTYPE_FORM_TITLE: _('Save Eventtype'),

    REPORTBUILDER_LINK_CLASS: "resultsLink",
    DISABLED_CLASS: "disabled",

    // don't let this thing dispatch, we're only a DispatchingModule
    // to get the onJobDone method
    requiresDispatch: function(context) {return false},


    initialize: function($super, container) {
        $super(container);
        this.messenger = Splunk.Messenger.System.getInstance();
        // an extra property "alwaysEnabled" is used here for some of the menu items.
        // this tells the _disabledMenu function to always show the menu item,
        // regardless of whether the search has finished running or not
        this.actionsMenuItemsDict = [
            {
                "label" : _("Save search..."),
                "alwaysEnabled" : true,
                callback: this.onSavedSearchForm.bind(this),
                "showInFilter" : ['search']
            },
            {
                "label": _("Save results"),
                callback: function(event) {
                    if (this.baseContext && this.baseContext.isJobDispatched()) {
                        this.baseContext.job.save();
                        this.messenger.send("info", "splunk.search", _("These search results have been saved. You can retrieve them later via the jobs manager."))
                    }
                }.bind(this),
                "showInFilter" : ['search']
            },
            {
                // formerly Link to search
                "label" : _("Get link to results..."),
                callback: function(event) {
                    var formContainer = $(this.SHARE_LINK_FORM_CONTAINER_CLASS, this.container)[0]
                    var title = this.SHARE_LINK_FORM_TITLE;
                    Splunk.Popup.createShareLinkForm(formContainer, title, this.getBaseContext());
                }.bind(this),
                "showInFilter" : ['search']
            },
            /*
            {
                "label" : _("Link to this search"),
                "style" : this.PERMALINK_SEARCH_CLASS,
                "showInFilter" : ['search']
                
            },
            */
            {
                "label" : _("Export results..."),
                callback: function(event) {
                    var formContainer = $('.exportPopupContainer', this.container)[0];
                    Splunk.Popup.createExportResultsForm(formContainer, this.getBaseContext().job.getSearchId() );
                }.bind(this),
                "showInFilter" : ['search']
            },
            {
                "label" : _("Save as event type..."),
                "alwaysEnabled" : true,
                callback: this.onEventtypeForm.bind(this),
                "showInFilter" : ['search']
            },
            {
                "label" : _("Build report..."),
                callback: function(event) {
                    this.buildReportLink();
                }.bind(this),
                "showInFilter" : ['search']
            },
            {
                "divider": "actionsMenuDivider",
                "showInFilter" : ['search']
            },
            {
                "label" : _("Create new dashboard..."),
                "alwaysEnabled" : true,
                "callback": function() { Splunk.Globals.Viewmaster.openDashCreateForm(); },
                "showInFilter" : ['search', 'dashboard', 'dashboard-SimpleDashboard']
            },
            {
                "label" : _("Edit dashboard..."),
                "alwaysEnabled" : true,
                "callback": function() { Splunk.Globals.Viewmaster.openDashEditForm(Splunk.util.getCurrentView()); },
                "showInFilter" : ['dashboard-SimpleDashboard']
            },
            
            /** DISABLED: pending permission-aware context
            {
                "label" : _("Edit view..."),
                "alwaysEnabled" : true,
                "uri"   : Splunk.util.make_url(
                        'manager', 
                        Splunk.util.getCurrentApp(), 
                        'data', 'ui', 'views', 
                        Splunk.util.getCurrentView()) + '?action=edit',
                "showInFilter" : ['search', 'dashboard']
            },
            {
                "label" : _("Use as template..."),
                "alwaysEnabled" : true,
                "uri"   : Splunk.util.make_url(
                            'manager', 
                            Splunk.util.getCurrentApp(), 
                            'data', 'ui', 'views', '_new') 
                        + '?action=edit&template=' + encodeURIComponent(Splunk.util.getCurrentView()),
                "showInFilter" : ['search', 'dashboard', 'dashboard-SimpleDashboard']
            },
            **/
            
            {
                "label" : _("Print..."),
                "alwaysEnabled" : true,
                callback: function(event) {
                    $(document).trigger("PrintPage");
                    return false;
                }.bind(this),
                "showInFilter" : ['search', 'dashboard', 'dashboard-SimpleDashboard']
            }
        ];

        this.messenger = Splunk.Messenger.System.getInstance();
        this._showActionsMenu = Splunk.util.normalizeBoolean(this._params["showActionsMenu"]);
        
        // determine the menu filter set; dashboards need a runtime check on
        // the kind of view definition; defaults to 'search' filterset
        this._filter = Splunk.util.normalizeBoolean(this.getParam("actionsMenuFilter")) || 'search';
        if (this._filter == 'dashboard') {
            var viewObjectMode = Splunk.util.getCurrentViewConfig()['view']['objectMode'];
            if (viewObjectMode == 'SimpleDashboard') {
                this._filter = 'dashboard-SimpleDashboard';
            }
        }
        

        if ( this._showActionsMenu ) {
            this.PERMALINK_SEARCH_CLASS = "permalinkSearch";
            this.PERMALINK_SID_CLASS    = "permalinkSID";
            this.REPORTBUILDER_CLASS	= "reportBuilderLink";


            // Maintain a reference to the active search context.
            this.activeSearchContext = false;

            // build the actions menu
            this.actionsMenu = new Splunk.MenuBuilder({
                containerDiv: this.container,
                menuDict: this._disabledMenu(this.actionsMenuItemsDict),
                activator: $('.actionsMenu', this.container),
                menuClasses: 'splMenu-primary',
                filter: this._filter
            });

            this._initEventListeners();
        }
    },

    _initEventListeners: function() {
        // PERMALINK PART 1
        $(document).bind('jobDispatched', this.updatePermalinks.bind(this));
    },
    applyContext: function($super, context) {
        var retVal = $super(context);
        if (context.isResurrected()) {
            // PERMALINK PART 2
            this.updatePermalinks();
        }
        return retVal;
    },
    _disabledMenu: function(menu) {
        // generate a copy of a menuDict that has only disabled items
        disabledMenu = new Array();
        var l=menu.length;
        for (var i=0; i<l; i++) {
            if (menu[i].hasOwnProperty("divider") || menu[i].hasOwnProperty("alwaysEnabled")) {
                disabledMenu.push( menu[i] );
            } else if (menu[i].hasOwnProperty("label") ) {
                disabledMenu.push( {
                    "label": menu[i].label,
                    "style": this.DISABLED_CLASS,
                    "showInFilter": menu[i].showInFilter
                } );
            }
        }
        return disabledMenu;
    },

    buildReportLink: function() {
        /*
            This callback clones out the href in the Jobstatus module's link for the report builder
            If the search has not yet been completed/finalized, it will display an alert box to the user
            If there is no Report Builder link anywhere in the DOM (perhaps the JobStatus module does
            not exist in this view, for example), nothing happens at all.
        */
        var context = this.getBaseContext();
        var builderLink = $(".JobStatus a." + this.REPORTBUILDER_LINK_CLASS);
        if (builderLink.length < 1) {
            this.logger.error(this.moduleType, " user clicked 'build report', but we could not find the 'Build report' link in this view.");
            return false;
        }
        if (context.isJobDispatched()) {
            builderLink.click();
            return false;
        } else {
            alert(_("Please run a search first."));
        }
    },

    updatePermalinks: function(event) {
        var context = this.getBaseContext();
        var args = {};
        args["q"] = Splunk.util.addLeadingSearchCommand(context.getSearch());
        var range = context.getTimeRange();
        if (range) {
            if (range.getEarliestTimeTerms()) {
                args["earliest"] = range.getEarliestTimeTerms();
            }
            if (range.getLatestTimeTerms()) {
                args["latest"] = range.getLatestTimeTerms();
            }
        }
        $("li." + this.PERMALINK_SEARCH_CLASS + " a", this.container).attr("href", "?" + Splunk.util.propToQueryString(args));
        $("li." + this.PERMALINK_SID_CLASS + " a",    this.container).attr("href", "?sid=" + encodeURIComponent(context.job.getSearchId()));
    },

    // Export dialog functions
    exportPopupAccept: function() {
        this.exportForm = $(this.exportPopupHandle).find(".exForm");
        return this.exportForm.submit();
    },

    exportPopupCancel: function() {
            return true;
    },

    onJobStatusChange: function(event, status) {
        // reset the menu on a starting a new job (ie, cancelling the previous one)
        switch(status) {
            case 'cancel':
                this.actionsMenu.updateMenu( this._disabledMenu(this.actionsMenuItemsDict) );
            default: break;
        };
    },

    resetUI: function() {
        this.actionsMenu.updateMenu( this._disabledMenu(this.actionsMenuItemsDict) );
    },

    onJobDone: function() {
        // put in the usable menu on job completion
        this.actionsMenu.updateMenu(this.actionsMenuItemsDict);
    },

    //Set the saved search name if it exists
    onContextChange: function(){
        var savedSearchName = "";
        if ( this.getBaseContext().getSavedSearchName()){
            savedSearchName = ": " + this.getBaseContext().getSavedSearchName();
        }
        $("em", this.container).text(savedSearchName);
    },

    // Saved search form popup
    onSavedSearchForm: function(event) {
        var formContainer = $(this.SAVED_SEARCH_FORM_CONTAINER_CLASS, this.container)[0]
        var title = this.SAVED_SEARCH_FORM_TITLE;
        Splunk.Popup.createSavedSearchForm(formContainer, title, this.getBaseContext());
    },
    
    onEventtypeForm: function(event) {
        var formContainer = $(this.EVENTTYPE_FORM_CONTAINER_CLASS, this.container)[0];
        var search = '';
        if (this.getBaseContext()) search = Splunk.util.stripLeadingSearchCommand(this.getBaseContext().getSearch());
        Splunk.Popup.createEventtypeForm(formContainer, this.EVENTTYPE_FORM_TITLE, search);
    }

});
