import cherrypy, logging
import controllers.module as module
import splunk.appserver.mrsparkle.lib.paginator as paginator
import splunk

logger = logging.getLogger('splunk.modules.paginator.paginator')

class Paginator(module.ModuleHandler):

    def generateResults(self, length, offset=0, count=10, max_pages=10, **args):
        '''
        HTML formatted set of pages for a given length.

        Arg:
            length: Total result set length.
            offset: The current offset.
            count: The max results per offset.
            max_pages: See paginator module.
        '''
        
        length = int(length)
        offset = int(offset)
        count = int(count)
        max_pages=int(max_pages)
        pager = paginator.Google(length, count, max_pages=max_pages, item_offset=offset)
        
        response_data = {
            "length": length, 
            "count": count, 
            "max_pages": max_pages, 
            "offset": offset,
            "host_app": args['host_app'],
            "client_app": args['client_app']
        }
        
        if pager.previous_exists():
            response_data['page_previous'] = {'offset':pager.previous_offset()}
        response_data['page_range'] = []
        for page in pager.page_range:
            response_data['page_range'].append(
                {
                    'number':page,
                    'offset':pager.page_item_offset(page),
                    'active':pager.is_active_page(page)
                }
            )
        if pager.next_exists():
            response_data['page_next'] = {'offset':pager.next_offset()}
        return self.controller.render_template('paginator/paginator_generate_results.html', response_data)
