//put Module in the namespace if it isnt already there.
Splunk.namespace("Module");

Splunk.Module.AbstractPagedModule = $.klass(Splunk.Module.DispatchingModule, {

    initialize: function($super, container) {
        $super(container);
        // although these are guaranteed to be present by the conf, 
        // even the defaults there will only apply if
        // a) this instance never gets given a settingsMap with these keys defined.
        // b) no subclass sets another value in its initialize.
        this._PAGE_SETTING_NAMES = ["count", "maxLines", "fields", "offset"];

        this.useParamValuesAsSettings(this._PAGE_SETTING_NAMES);
    },
    /*
    onLoadStatusChange: function($super, status) {
        if (status == Splunk.util.moduleLoadStates.LOADED) {
            this.onSettingsChange();
        }
        return $super();
    },
    */
    // methods that are common to both eventsViewer and resultsViewer.
    getResultParams: function($super) {
        var params = $super();
        // we only add the PAGE_SETTINGS keys, not any of the other keys which will be present.
        for (var i=0; i<this._PAGE_SETTING_NAMES.length; i++) {
            // we only want it if we have a value defined for it. 
            var name = this._PAGE_SETTING_NAMES[i];
            if (this.hasSetting(name)) {
                params[name] = this.getSetting(name);
            }
        }
        var INCONSISTENT_NAMES = {"fields" : "field_list", "maxLines" : "max_lines"};
        for (var name in INCONSISTENT_NAMES) {
            if (params.hasOwnProperty(name)) {
                var patchedName = INCONSISTENT_NAMES[name];
                params[patchedName] = params[name];
                delete params[name];
            }
        }
        var range = this.getBaseContext().getTimeRange();
        if (range.isSubRangeOfJob()) {
            params.earliest_time = range.getEarliestTimeTerms();
            params.latest_time = range.getLatestTimeTerms();
        }
        return params;
    },
    
    onContextChange: function() {
        var context = this.getBaseContext();
        // The search could come to us totally done.  
        // Or be running and well underway.
        // in these cases we call getResults once right away.
        this.resetUI();
           
        if (context.isJobDispatched()&& this.baseContext && (context.job.isDone() || (context.job.getEventCount() > 0))) {
            this.getResults();
        }
    },

    onSettingsChange: function() {
        
        if (this.baseContext && this.getBaseContext().isJobDispatched() && this.haveResultParamsChanged()) {
            
            var context = this.getBaseContext();
            if (context && context.isJobDispatched()) {
                this._pageComplete = false;
                this.getResults();
                // experimenting with a convention in onSettingsChange, 
                // where returning true means an action was taken, 
                // the action typically being getResults();
                // The motivation is to help avoid situations where we call it twice.
                return true;
            }
        }
    }
});
Splunk.Module.AbstractPagedModule.isAbstract = true;
