Splunk.Module.EventsViewer = $.klass(Splunk.Module.AbstractPagedModule, {
    EVENT_SELECTOR: "li.item",
    INTENTION_NEGATE_TERM: "negateterm",
    INTENTION_ADD_TERM: "addterm",
    INTENTION_CLEAR_TERM: "clear",
    LOADING_MESSAGE: _("Getting search results..."),
    //negate and replace modifier keys (term clicking) can take one of shiftKey, altKey, ctrlKey or metaKey.
    MODIFIER_CUSTOM_MAP: [
        {"userAgentRex": /Macintosh/, "negate": "altKey", "replace": "metaKey"},//note: FF altKey+metaKey and click results in hand only possible negate/replace combo is shiftKey+metaKey or shiftKey+altKey.
        {"userAgentRex": /Linux.*Firefox/, "negate": "ctrlKey", "replace": "metaKey"},
        {"userAgentRex": /Windows/, "negate": "altKey", "replace": "ctrlKey"}
    ],
    MODIFER_DEFAULT: {"userAgentRex": /.*/, "negate": "altKey", "replace": "metaKey"},
    FIELD_MENU_CLASSES: "splMenu-primary",
    FIELD_MENU_CLASS_NAME: "fm",
    EVENT_ACTION_MENU_CLASSNAME: "actions",
    REPORT_LINK_INTENTION: '{"name": "plot", "arg": {"mode": "timechart", "fields": [["count", "__events"]], "splitby": "%(fieldName)s"}}',
    SCROLLER_ENABLED_CLASS_NAME: "EventsViewerScroller",
    SEGMENT_HOVER_CLASS_NAME: "h",
    SHOW_INLINE_CLASS_NAME: "showinline",
    SHOW_INLINE_LOADING_MESSAGE: _("Retrieving updated lines..."),
    SHOW_INLINE_ERROR_MESSAGE: _("Could not retrieve updated lines, try again?"),
    SHOW_SOURCE_CLASS_NAME: "showsrc",
    SOFT_WRAP_CLASS_NAME: "EventsViewerSoftWrap",
    TAG_CLASS_NAME: "tg",
    TAG_ERROR_MESSAGE: _("Tagging is currently unavailable."),
    TAG_FIELD_POPUP_ACTION_CLASS_NAME: "tf",
    TAG_FIELD_POPUP_TITLE: _("Tag This Field"),
    TAG_FIELD_POPUP_SELECTOR: ".tagfieldpopup",
    TAG_INTENTION_PREFIX: "tag::",
    TERM_CLASS_NAME: "t",
    TIMESTAMP_CLASS_NAME: "time",
    TIMESTAMP_EPOCH_TIME_ATTRIBUTE_NAME: "s:epoch",
    TIMESTAMP_TIMEZONE_OFFSET_ATTRIBUTE_NAME: "s:tzoffset",
    FIELD_VALUE_CLASS_NAME: "v",
    FIELD_NAME_CLASS_NAME: "k",
    UI_EVENT_TYPES: ["click", "keydown", "mouseover", "mouseout", "change"],
    UPDATE_SCROLLER_BUFFER_TIME: 250,
    UPDATE_SCROLLER_EVENT_TYPES: ["resize", "scroll"],
    UPDATE_SCROLLER_PIXEL_ADJUSTMENT: {other:-12, msie:-22},
    /**
     *  Override default initialize. See $super for implementation details.
     */
    initialize: function($super, container){
        $super(container);
        var modifier = this.getModifier();
        this.logger = Splunk.Logger.getLogger("events_viewer.js");
        this.useParamValuesAsSettings(["displayRowNumbers", "segmentation"]);
        this.eventMenu = null;
        this.enableBehavior = Splunk.util.normalizeBoolean(this._params.enableBehavior);
        this.enableEventActions = Splunk.util.normalizeBoolean(this._params.enableEventActions);
        this.enableFieldActions = Splunk.util.normalizeBoolean(this._params.enableFieldActions);
        this.enableTermSelection = Splunk.util.normalizeBoolean(this._params.enableTermSelection);
        this.fieldMenu = null;
        this.maxLinesConstraint = parseInt(this._params.maxLinesConstraint, 10);
        this.messenger = Splunk.Messenger.System.getInstance();
        this.negateModifier = modifier.negate;
        this.replaceModifier = modifier.replace;
        this.reportFieldLink = this._params.reportFieldLink || null;
        this.renderedCount = 0;
        this.scrollerEnable = Splunk.util.normalizeBoolean(this._params.scrollerEnable);
        this.scrollerMaxHeight = parseInt(this._params.scrollerMaxHeight, 10);
        this.scrollerMinHeight = parseInt(this._params.scrollerMinHeight, 10);
        this.updateScrollerBuffer = [];//Buffer resize and scroll events are tres rapide.
        if(this.scrollerEnable){
            this.scrollerCreate();
        }
        if(this.enableBehavior){
            this.bindEventListeners();
        }
    },
    /**
     * Bind top level event listener.
     */
    bindEventListeners: function(){
        this.resultsContainer.bind(this.UI_EVENT_TYPES.join(" "), this.onUIEvent.bind(this));
    },
    /**
     * Get the adjacent field name for an element.
     *
     * @param {Object} event A jQuery event.
     * @type String
     * @return The text value of an adjacent field node.
     */
    getAdjacentFieldName: function(element){
        var selector = "." + this.FIELD_NAME_CLASS_NAME + ":first";
        return element.siblings(selector).text();
    },
    /**
     * Get the adjacent field value for an element.
     *
     * @param {Object} event A jQuery event.
     * @type String
     * @return The text value of an adjacent field node.
     */
    getAdjacentFieldValue: function(element){
        var selector = "." + this.FIELD_VALUE_CLASS_NAME + ":first";
        return element.siblings(selector).text();
    },
    /**
     * Get an intention name based on a UI event and keyboard modifier state.
     *
     * @param {Object} event The DOM event triggered.
     * @type String
     * @return The intention name adjusted to negate modifier setting. 
     */
    getIntentionName: function(event){
        return (event[this.negateModifier])?this.INTENTION_NEGATE_TERM:this.INTENTION_ADD_TERM;
    },
    /**
     * Convenience method for retrieving a normalized intention based on internal key-binding settings.
     * 
     * @param {Object} intention Object literal in standard intention format.
     * @param {Object} event The DOM event triggered.
     * 
     * @type Object
     * @return An object literal in standard intention format.
     */
    getKeydownMutatedIntention: function(intention, event){
        return (event[this.replaceModifier])?{arg: intention, name: this.INTENTION_CLEAR_TERM}:intention;
    },
    /**
     * Retrieve the best match of modifier key bindings based on navigator.userAgent.
     * Merges MODIFIER_CUSTOM_MAP and MODIFER_DEFAULT constants and peforms reverse iteration 
     * where the lowest index MODIFIER_CUSTOM_MAP entry takes highest precedent and 
     * MODIFER_DEFAULT takes lowest.
     * 
     * @type Object
     * @return The best match modifier object literal. Note: See a MODIFIER_CUSTOM_MAP entry or MODIFER_DEFAULT for attribute specifications.
     */
    getModifier: function(){
        var userAgent = navigator.userAgent || "";
        var modifierMatch = null;
        for(var i=this.MODIFIER_CUSTOM_MAP.length-1; i>-1; i--){
            var modifier = this.MODIFIER_CUSTOM_MAP[i];
            if(userAgent.search(modifier.userAgentRex)!=-1){
                modifierMatch = modifier;
            }
        }
        if(!modifierMatch){
            this.logger.warn("Could not find matching MODIFIER_CUSTOM_MAP, reverting to MODIFER_DEFAULT.");
            modifierMatch = this.MODIFER_DEFAULT;
        }
        //this.logger.info("Modifier keys bound with userAgentRex", modifierMatch.userAgentRex.toString(), "negate modifier", modifierMatch.negate, "replace modifier", modifierMatch.replace, "for navigator.userAgent", userAgent);
        return modifierMatch;
    },
    /**
     * Override to control firing. See $super for implementation details.
     */
    getResults: function($super){
        if(this.getBaseContext().isJobDispatched()){
            //this.logger.warn("Get results")
            $super();
        }
    },
    /**
     * Override to augment param(s). See $super for implementation details.
     */
    getResultParams: function($super){
        var params = $super();
        var baseContext = this.getBaseContext();
        params.display_row_numbers = (Splunk.util.normalizeBoolean(this.getSetting("displayRowNumbers")) ? 1 : 0);
        params.entity_name  = this.getParam('entityName');
        params.enable_event_actions = (this.enableEventActions)?1:0;
        params.enable_field_actions = (this.enableFieldActions)?1:0;
        params.segmentation = this.getSetting("segmentation");
        params.sid = baseContext.job.getSearchId();
        params.min_lines = params.max_lines;
        params.max_lines_constraint = this.maxLinesConstraint;
        if($.browser.msie){
            params.replace_newlines = 1;
        }
        return params;
    },
    /**
     * Get a segments parent.
     *
     * @param {Object} element A DOM element reference.
     * @type Object
     * @return A matching term elements parent casted as a jQuery element (excludes all non-term elements).
     */
    getSegmentParent: function(element){
        var parent = element.parentNode;
        if(parent.childNodes[parent.childNodes.length-1]==element && $(parent).hasClass(this.TERM_CLASS_NAME)){
            element = parent;
        }
        return $(element);
    },
    /**
     * Perform garbage collection on an array of menu pointers provided it is no longer required.
     * 
     * @param {Array} menus An array of menu pointers. Note, a pointer value can be null.
     * @param {Object} activator A DOM element to the menu activator (The element that triggered the event to create a menu).
     */
    menuGC:function(menus, activator){
        for(var i=0; i<menus.length; i++){
            var menu = menus[i];
            if(menu && menu.getActivator()[0]!=activator){
                menu.removeMenu();
                menu = null;
            }
        }
    },
    onBeforeJobDispatched: function(context) {
        context.setMinimumStatusBuckets(1);
    },
    /**
     * Handle a new search. See $super for implementation details.
     */
    onContextChange: function($super){
        this.renderedCount = 0;
        $super();
    },
    /**
     * Handle a UI event to click on a field and propagate intention to search.
     *
     * @param {Object} element The DOM element that triggered the event.
     */
    onFieldSelect: function(event){
        var element = $(event.target);
        var name = this.getIntentionName(event);
        var fieldName = this.getAdjacentFieldName(element);
        var intention = {
            arg: {},
            name: name
        };
        intention.arg[fieldName] = Splunk.util.escapeSearchTerm(element.text());
        intention = this.getKeydownMutatedIntention(intention, event);
        this.passIntention(intention);
    },
    /**
     * Handle job progress notification and get results if required. See $super for implementation details.
     *
     * @param {Object} event A jQuery event.
     */
    onJobProgress: function(event){
        var context = this.getBaseContext();
        var needMoreEvents = this.renderedCount < this.getSetting("count");
        var moreEventsAreAvailable = context.job.getEventAvailableCount()>this.renderedCount;
        if (needMoreEvents && moreEventsAreAvailable){
            // Need to remember that we've already asked for these events
            // for the case when the next progress event occurs before renderResults.
            // without this assignment we will get multiple calls to getResults 
            // competing with eachother or perhaps aborting eachother.
            this.renderedCount = Math.min(context.job.getEventAvailableCount(), this.getSetting("count"));
            //this.logger.info("onJobProgress triggering getResults");
            this.getResults();
        }
    },
    /**
     * Handle a report on field action. Spawns a report popup window with the appropriate field.
     *
     * @param {Object} element The DOM element that triggered the event.
     * @type Boolean
     * @return Cancel (false) or enable (true) the propagation of this event.
     */
    onReportField: function(event){
        var element = $(event.target);
        var href = (element.attr("href"))?element.attr("href"):$("a[href]", element).attr("href");
        try{
            var intention = JSON.parse(decodeURIComponent(Splunk.util.getParameter("intention", href)));
        }catch(e){
            this.logger.error("Could not serialize report link intention, please check the format of this.REPORT_LINK_INTENTION", e);
            return false;
        }
        var context = this.getBaseContext();
        var baseSID = context.job.getSearchId();
        context.abandonJob();
        context.addIntention(intention);
        context.sendToView(this.reportFieldLink, {base_sid: baseSID}, true, true, {autosize: true});
        return false;
    },
    /** AbstractPagedModule no longer fires onSettingsChange in onLoadStatusChange
     * as this has turned out to be unsafe in general.  For EventsViewer however it's OK for the time being.
     */
    onLoadStatusChange: function($super, status) {
        if (status == Splunk.util.moduleLoadStates.LOADED) {
            this.onSettingsChange(this.getBaseSettings());
        }
    },
    /**
     * Handle setting changes. See $super for implementation details.
     *
     * @param {Object} settings An object literal containing various types for values.
     */
    onSettingsChange: function($super, settings){
        if("softWrap" in settings){
            this.setSoftWrapClassName(settings.softWrap);
        }
        if("scrollerEnable" in settings){
            if(settings.scrollerEnable){
                this.scrollerCreate();
            }else{
                this.scrollerDestroy();
            }
        }
        var requestTriggered = $super(settings);
        if(!requestTriggered && (settings.hasOwnProperty("segmentation") || settings.hasOwnProperty("displayRowNumbers"))){//throttle getResults call
            this.getResults()
        }
    },
    /**
     * A factory method for creating a new field menu dropdown. Menus are created on the fly vs. multiple instances.
     *
     * @param {Object} event A DOM event.
     * @type Boolean
     * @return Cancel (false) or enable (true) the propagation of this event.
     */
    onShowFieldMenu: function(event) {
        var menuData = [];
        var element = $(event.target);
        var fieldName = this.getAdjacentFieldName(element);
        var fieldValue = this.getAdjacentFieldValue(element);
        this.menuGC([this.eventMenu, this.fieldMenu], element[0]);
        menuData.push(
            {
                uri: Splunk.util.make_url("/tags/"+Splunk.util.getCurrentApp()+"/fields/"+encodeURIComponent(fieldName)+"/"+encodeURIComponent(fieldValue)),
                label: sprintf(_("Tag %(name)s=%(value)s"), {name:fieldName, value:fieldValue}),
                callback: function(event){
                    if(this.fieldMenu){
                        this.fieldMenu.removeMenu();
                    }
                    return this.onTagField(event);
                }.bind(this)
            }
        );
        if(this.reportFieldLink){
            var intention = sprintf(this.REPORT_LINK_INTENTION, {fieldName: fieldName});
            menuData.push(
                {
                    uri: Splunk.util.make_url("app", Splunk.util.getCurrentApp(), this.reportFieldLink) + "#intention=" + encodeURIComponent(intention),
                    label: _("Report on field"),
                    callback: function(event){
                        if(this.fieldMenu){
                            this.fieldMenu.removeMenu();
                        }
                        return this.onReportField(event);
                    }.bind(this)
                }
            );
        }
        this.fieldMenu = new Splunk.MenuBuilder({
            containerDiv: this.container,
            menuDict: menuData,
            activator: element,
            menuClasses: this.FIELD_MENU_CLASSES,
            showOnInit: true
        });
        return false;
    },
    /**
     * Like onShowFieldMenu above, but generates menus builds event action menus instead
     *
     * @param {Object} event A DOM event.
     * @type Boolean
     * @return Cancel (false) or enable (true) the propagation of this event.
     */
    onShowEventMenu: function(event) {
        var element = $(event.target);
        var menuData = [];
        var menuDataRaw = $(event.target).parent().attr("s:menu");
        try{
            var menuData = JSON.parse(menuDataRaw);
        }catch(e){
            this.logger.error("Could not serialize event menu actions", e);
        }
        this.menuGC([this.fieldMenu, this.eventMenu], element[0]);
        if(menuData.length>0){
            this.eventMenu = new Splunk.MenuBuilder({
                containerDiv: this.container,
                menuDict: menuData,
                activator: element,
                menuClasses: this.FIELD_MENU_CLASSES,
                showOnInit: true
            });
        }
        return false;
    },
    /**
     * Handle a UI event to retrieve an individual events html via XHR.
     *
     * @param {Object} element The DOM element that triggered the event.
     * @type Boolean
     * @return Cancel (false) or enable (true) the propagation of this event.
     */
     onTimestampClick: function(event) {
        var em = $(event.target);
        var epoch          = em.attr(this.TIMESTAMP_EPOCH_TIME_ATTRIBUTE_NAME);
        var timezoneOffset = em.attr(this.TIMESTAMP_TIMEZONE_OFFSET_ATTRIBUTE_NAME);
        // beware of GMT itself, where offset is "0" (at least half the year).  This is why we dont parseInt right away.
        if (epoch && timezoneOffset) {
            epoch = parseInt(epoch, 10);
            timezoneOffset = parseInt(timezoneOffset, 10);
            var range = new Splunk.TimeRange(epoch,  epoch+1, timezoneOffset, timezoneOffset);
            var context = new Splunk.SearchContext();
            context.setTimeRange(range);
            this.passContextToParent(context);
        } else {
            this.logger.error("received a click on a timestamp however there was no ", this.TIMESTAMP_EPOCH_TIME_ATTRIBUTE_NAME, " attribute on the element so we cannot proceed.");
        }
     },
     onTimestampHighlight: function(event) {
         var element = $(event.target);
         element.addClass(this.SEGMENT_HOVER_CLASS_NAME);
     },
     onTimestampBlur: function(event) {
         var element = $(event.target);
        element.removeClass(this.SEGMENT_HOVER_CLASS_NAME);
     },
    /**
     * Handle a UI event to retrieve an individual events html via XHR.
     *
     * @param {Object} element The DOM element that triggered the event.
     * @type Boolean
     * @return Cancel (false) or enable (true) the propagation of this event.
     */
    onShowInline: function(event){
        var element = $(event.target);
        var resource = element.attr("href");
        var target = $(element.parent(this.EVENT_SELECTOR)[0]);//The element we want to replace.
        element.html(this.SHOW_INLINE_LOADING_MESSAGE);
        $.ajax({
            type: "GET",
            url: resource,
            dataType: "html",
            error: function(){
                this.logger.error(sprintf(_("Could not load %(resource)s"), {resource: resource}));
            }.bind(this),
            complete: function(data, textStatus){
                if(data.status==200){
                    target.html(data.responseText);
                }else{
                    element.html(this.SHOW_INLINE_ERROR_MESSAGE);
                }
            }.bind(this)
        });
        return false;
    },
    /**
     * Handle a UI event to click on the show source link for the event.
     *
     * @param {Object} element The DOM element that triggered the event.
     * @type Boolean
     * @return Cancel (false) or enable (true) the propagation of this event.
     */    
    onShowSource: function(event){
        var element = $(event.target);
        Splunk.window.open(element.attr("href"), "showsource");
        return false;
    },
    /**
     * Handle a tag field action. Spawns a popup window, pre-populates the field input.
     * NOTE: This method is large to encapsulate all popup dependencies.
     *
     * @param {Object} element The DOM element that triggered the event.
     * @type Boolean
     * @return Cancel (false) or enable (true) the propagation of this event. 
     */
    onTagField: function(event){
        var element = $(event.target);
        var resource = (element.attr("href"))?element.attr("href"):$("a[href]", element).attr("href");
        $.ajax({
            type: "GET",
            url: resource,
            dataType: "html",
            async: false,
            error: function(){
                this.logger.error(sprintf(_("Could not load %(resource)s"), {resource: resource}));
            }.bind(this),
            complete: function(data, textStatus){
                var content = (data.status==200)?data.responseText:this.TAG_ERROR_MESSAGE;
                $(this.TAG_FIELD_POPUP_SELECTOR).html(content);
            }.bind(this)
        });
        var htmlTemplate = $(this.TAG_FIELD_POPUP_SELECTOR);
        var popup = new Splunk.Popup(htmlTemplate, {
            title: this.TAG_FIELD_POPUP_TITLE,
            buttons: [
                {
                    label: _("Cancel"),
                    type: 'secondary',
                    callback: function(){ return true; }
                },
                {
                    label: _("Ok"),
                    type: 'primary',
                    callback: function(){ return false; }
                }
            ]
        });
        $("input[name='add']", popup.getPopup()).focus();
        var ajaxFormOptions = {
            success: function(){
                this.getResults();
                popup.destroyPopup();
            }.bind(this),
            target: this.TAG_FIELD_POPUP_SELECTOR
        };
        var popupForm = $("form", popup.getPopup());
        popupForm.ajaxForm(ajaxFormOptions);
        $("input[name='add']", popup.getPopup()).keydown(function(event){
            if(event.keyCode==13){
                popupForm.submit();
            }
        });
        $("button.splButton-primary", popup.getPopup()).click(function(){popupForm.submit()});
        return false;
    },
    /**
     * Handle a UI event to click on a tag and propagate intention to search.
     *
     * @param {Object} event The DOM event triggered.
     */
    onTagSelect: function(event){
        var element = $(event.target);
        var name = this.getIntentionName(event);
        var fieldName = this.TAG_INTENTION_PREFIX + this.getAdjacentFieldName(element);
        var intention = {
            arg: {},
            name: name
        };
        intention.arg[fieldName] = Splunk.util.escapeSearchTerm(element.text());
        intention = this.getKeydownMutatedIntention(intention, event);
        this.passIntention(intention);
    },
    /**
     * Handle a UI event to hover over a segment.
     *
     * @param {Object} event The DOM event triggered.
     */
    onTermHighlight: function(event){
        var element = this.getSegmentParent(event.target);
        element.addClass(this.SEGMENT_HOVER_CLASS_NAME);
    },
    /**
     * Handle a UI event to hover off a segment.
     *
     * @param {Object} event The DOM event triggered.
     */
    onTermBlur: function(event){
        var element = this.getSegmentParent(event.target);
        element.removeClass(this.SEGMENT_HOVER_CLASS_NAME);
    },
    /**
     * Handle a UI event to click on a segment and propagate intention to search.
     *
     * @param {Object} event The DOM event triggered.
     */
    onTermSelect: function(event){
        var element = this.getSegmentParent(event.target);
        var name = this.getIntentionName(event);
        var intention = {
            arg: Splunk.util.escapeSearchTerm(element.text()),
            name: name
        };
        intention = this.getKeydownMutatedIntention(intention, event);
        this.passIntention(intention);
    },
    /**
     * Top level UI event listener and dispatcher.
     *
     * @param {Object} event A DOM event.
     * @type Boolean
     * @return Cancel (false) or enable (true) the propagation of this event.
     */
    onUIEvent: function(event){
       var eventType = event.type;
       var eventTarget = $(event.target);
       if(eventType==="click" || eventType==="keydown"){
           if(eventTarget.hasClass(this.SHOW_INLINE_CLASS_NAME)){
               return this.onShowInline(event);
           }else if(eventTarget.hasClass(this.TERM_CLASS_NAME) && this.enableTermSelection){
               return this.onTermSelect(event);
           }else if(eventTarget.hasClass(this.SHOW_SOURCE_CLASS_NAME)){
               return this.onShowSource(event);
           }else if(eventTarget.hasClass(this.FIELD_VALUE_CLASS_NAME) && this.enableTermSelection){
               return this.onFieldSelect(event);
           }else if(eventTarget.hasClass(this.TAG_CLASS_NAME) && this.enableTermSelection){
               return this.onTagSelect(event);
           }else if(eventTarget.hasClass(this.FIELD_MENU_CLASS_NAME)){
               return this.onShowFieldMenu(event);
           }else if(eventTarget.hasClass(this.EVENT_ACTION_MENU_CLASSNAME)){
               return this.onShowEventMenu(event);
           }else if(eventTarget.hasClass(this.TIMESTAMP_CLASS_NAME) && this.enableTermSelection){
               return this.onTimestampClick(event);
           }
        }else if(eventType==="mouseover" && this.enableTermSelection) {
            if (eventTarget.hasClass(this.TERM_CLASS_NAME)){
                return this.onTermHighlight(event);
            }else if(eventTarget.hasClass(this.TIMESTAMP_CLASS_NAME)){
                return this.onTimestampHighlight(event);
            }
           
        }else if(eventType==="mouseout") {
            if (eventTarget.hasClass(this.TERM_CLASS_NAME)){
                return this.onTermBlur(event);
            }else if(eventTarget.hasClass(this.TIMESTAMP_CLASS_NAME)){
                return this.onTimestampBlur(event);
            }
        }
    },
    /**
     * Resize the content area (container) to pageYOffset, window height and cumulativeOffsetTop while respecting
     * scrollerMaxHeight and scrollerMinHeight constraints.
     *
     * @param {Object} event A DOM event.
     */
    onUpdateScroller: function(event){
        this.updateScrollerBuffer.push("");
        setTimeout(
            function(){
                if(this.updateScrollerBuffer.length>0){
                    var pixelAdjustment = (jQuery.browser.msie)?this.UPDATE_SCROLLER_PIXEL_ADJUSTMENT.msie:this.UPDATE_SCROLLER_PIXEL_ADJUSTMENT.other;
                    var height = ($(window).height() + Splunk.util.getPageYOffset() + pixelAdjustment) - Splunk.util.getCumlativeOffsetTop(this.container[0]);
                    height = Math.min(height, this.scrollerMaxHeight);
                    height = Math.max(height, this.scrollerMinHeight);
                    this.updateScrollerBuffer = [];
                    this.container.height(height);
                }
            }.bind(this),
            this.UPDATE_SCROLLER_BUFFER_TIME
        );
    },
    /**
     * Convenience method for passing an intention to (a) parent(s).
     *
     * @param {Object} intention See Splunk.SearchContext specification.
     */
    passIntention: function(intention){
        var context = new Splunk.SearchContext("*");
        context.addIntention(intention);
        this.passContextToParent(context);
    },
    /**
     * Reset the UI to its original state.
     */
    resetUI: function(){
        this.offset = 0;
        this.resultsContainer.html("");
    },
    /**
     * Override and set renderedCount. See $super for implementation details.
     */
    renderResults: function($super, htmlFragment){
        $super(htmlFragment, true);//TODO: turboInnerHTML strips event listeners for most browser, benchmark performance vs jQuery.html method (see SPL-22873)
        if(this.enableBehavior && !$.browser.msie){
            this.bindEventListeners();//Event listeners are lost from clone operation (except on IE)
        }
        this.renderedCount = $(this.EVENT_SELECTOR, this.container).length;
        if(this.scrollerEnable){
            this.onUpdateScroller();
            this.container[0].scrollTop = 0;
        }
    },
    /**
     * Create/Enable scroller behavior, add event listeners and trigger an onUpdateScroller event.
     */
    scrollerCreate: function(){
        $(window).bind(this.UPDATE_SCROLLER_EVENT_TYPES.join(" "), this.onUpdateScroller.bind(this));
        this.container.addClass(this.SCROLLER_ENABLED_CLASS_NAME);
        this.onUpdateScroller();
    },
    /**
     * Destroy/Disable scroller behavior, remove event listeners and reset the height of the container.
     */
    scrollerDestroy: function(){
        $(window).unbind(this.UPDATE_SCROLLER_EVENT_TYPES.join(" "), this.onUpdateScroller.bind(this));
        this.container.removeClass(this.SCROLLER_ENABLED_CLASS_NAME);
        this.container.height("");
    },
    /**
     * Set/Reset the class name for soft wrap control.
     *
     * @param {Boolean} state Add or remove the softwrap class.
     */
    setSoftWrapClassName: function(state){
        if(state){
            this.resultsContainer.addClass(this.SOFT_WRAP_CLASS_NAME);
        }else{
            this.resultsContainer.removeClass(this.SOFT_WRAP_CLASS_NAME);
        }
    }
});
