
//put Module in the namespace if it isnt already there.
Splunk.namespace("Module");

Splunk.Module.SimpleResultsHeader = $.klass(Splunk.Module.DispatchingModule, {
    initialize: function($super, container) {
        $super(container);

        // Note: these params are marked as required in the conf.  
        // so the view itself will have thrown an exception if they were absent. 
        // however the unit testing doesnt currently know how to set required params
        this._entityName   = this._params["entityName"]   || "(required entityName unset)";
        this._headerFormat = this._params["headerFormat"] || "(required headerFormat unset) %s";
    },
    onBeforeJobDispatched: function(context) {
        if (this._entityName=="events") context.setMinimumStatusBuckets(1);
    },
    onJobProgress: function() {
        var context = this.getBaseContext();
        var count = this.getEntityCount(context);
        this.setCountHeader(count, context.job.isDone());
    },
    onContextChange: function() {
        var context = this.getBaseContext();
        var newSID = context.job.getSearchId();
        // if either weve never had a job, or this job is different from the last
        // one we had.
        if (!this._previousSID || this._previousSID != newSID) {
            this._previousSID = newSID;
        }
        this.onJobProgress();
    },
    ////////////////////////////////
    // Methods that are not overriding anything.
    ////////////////////////////////
    getEntityCount: function(context) {
        switch  (this._entityName) {
            case "events" :
                return context.getEventCount();
            case "results" :
                return context.job.getResultCount();
            case "scanned" :
                return context.job.getScanCount();
            //TODO - it would be cool if this could be wired up to a fieldValue...
            // in which case it would have to talk to the summary endpoint, and
            // defer the actual writing of the count into the DOM to a passed callback.
            default:
                this.logger.error("fell into default case of switch entityName=", this._entityName);
        }
    },
    setCountHeader: function(count, isDone) {
        var modifier = (isDone) ? "" : "&#8805; ";
        $(".headerText", this.container).html(sprintf(this._headerFormat, {count: modifier + Splunk.util.getCommaFormattedNumber(count)}));
    },
    resetUI: function() {
        this.setCountHeader(0, true);
        this._timeRangeHeader.text('');
    }
});
