//put Module in the namespace if it isnt already there.
Splunk.namespace("Module");

// A simple, but extensible, results table
Splunk.Module.SimpleResultsTable = $.klass(Splunk.Module.AbstractPagedModule, {

    /*
     * overriding initialize to set up references and event handlers.
     */
    initialize: function($super, container) {
        $super(container);
        this.popupHandler = null;
        this.menu = null;
        this.entityName = this._params.entityName;
        // basically just copies over the given keys from the load params
        // into the runtime settings map.
        this.useParamValuesAsSettings(["displayRowNumbers", "dataOverlayMode"]);

        this.displayMenu = Splunk.util.normalizeBoolean(this._params.displayMenu);
        this.renderedCount = 0;
        this.resultsContainer = $(".SimpleResultsTableResults", this.container);
        this.container.bind("mouseover", this.onRowMouseover.bind(this));
        this.container.bind("mouseout", this.onRowMouseout.bind(this));
        this.container.bind("click", this.onRowClick.bind(this));
    },
    onLoadStatusChange: function($super, status) {
        if (status == Splunk.util.moduleLoadStates.LOADED) {

            if (this.getSetting("dataOverlayMode")) {
                this.onResultsRendered();
            }
        }
    },

    /**
     * A factory method for creating a new menu dropdown for a cell.
     *
     * @param {Object} event A DOM event.
     */
    menuFactory: function(event) {
        var element = $(event.target);
        if (this.menu && this.menu.getActivator()[0]!=element[0]) {
            this.menu.removeMenu();
            this.menu = null;
        }
        var coordinates = this.getXYCoordinates(element);
        this.menuData = [];
        this.menuData.push(this.getMenuData(1, coordinates.x, coordinates.y, element));
        this.menuData.push(this.getMenuData(2, coordinates.x, coordinates.y, element));
        if (coordinates.x>0) {
            this.menuData.push(this.getMenuData(3, coordinates.x, coordinates.y, element));
            this.menuData.push(this.getMenuData(4, coordinates.x, coordinates.y, element));
            this.menuData.push(this.getMenuData(5, coordinates.x, coordinates.y, element));
        }
        this.menu = new Splunk.MenuBuilder({
            containerDiv: this.container,
            menuDict: this.menuData,
            activator: element,
            menuClasses: 'splMenu-primary',
            showOnInit: true
        });
    },

    /**
     * A results table cell menu data item.
     *
     * @param {Number} type Menu item intention to create.
     * @param {Number} x Table cell x-coordinate.
     * @param {Number} y Table cell y-coordinate.
     * @param {Object} element jQuery table cell element reference.
     * @type Object
     * @return An object literal having a uri, label and callback properties.
     */
    getMenuData: function(type, x, y, element) {
        var intentions = [];
        var menuData = {
            uri:"javascript:void(0);",
            label:null,
            callback:this.onMenuItemSelect.bind(this)
        };
        switch(type) {
            //<value>
            case 1:
                var value = element.text();
                var intention = this.getIntentionBoilerPlate();
                menuData.label = value;
                intention.arg = value;
                intentions.push(intention);
                break;
            //<column_name>=<value>
            case 2:
                var columnName = this.getColumnName(x, element);
                var value = element.text();
                var intention = this.getIntentionBoilerPlate();
                menuData.label =  columnName + "=" + value;
                intention.arg[columnName] = value;
                intentions.push(intention);
                break;
            //<row_name>=<value>
            case 3:
                var rowName = this.getRowName(element);
                var value = element.text();
                var intention = this.getIntentionBoilerPlate();
                menuData.label = rowName + "=" + value;
                intention.arg[rowName] = value;
                intentions.push(intention);
                break;
            //<row_name> <column_name=<value>
            case 4:
                var rowName = this.getRowName(element);
                var columnName = this.getColumnName(x, element);
                var value = element.text();
                var intention1 = this.getIntentionBoilerPlate();
                var intention2 = this.getIntentionBoilerPlate();
                menuData.label = rowName + " " + columnName + "=" + value;
                intention1.arg = rowName;
                intentions.push(intention1);
                intention2.arg[columnName] = value;
                intentions.push(intention2);
                break;
            //<column_name:first>=<row_name>
            case 5:
                var columnNameFirst = element.parents("table.simpleResultsTable").find("th:nth-child(1)").text();
                var rowName = this.getRowName(element);
                var intention = this.getIntentionBoilerPlate();
                menuData.label = columnNameFirst + "=" + rowName;
                intention.arg[columnNameFirst] = rowName;
                intentions.push(intention);
                break;
            default:
                var intention = this.getIntentionBoilerPlate();
                menuData.label = _("Invalid cell menu data type " + type);
                intentions.push(intention);
        }
        menuData.uri = escape(JSON.stringify(intentions));
        return menuData;
    },

    /**
     * Get a simple for boiler plate for a search intention.
     * @type Object
     * @return An object having arg and name properties.
     */
    getIntentionBoilerPlate: function() {
        return {
            arg:{},
            name:"addterm"
        };
    },

    /**
     * Get a table cells column name.
     *
     * @param {Number} x The x-coordinate for a cell in a table.
     * @param {Object} element A jQuery cell element reference.
     * @type String
     */
    getColumnName: function(x, element) {
        return element.parents("table.simpleResultsTable").find("th:nth-child("+ (x + 1) +")").text();
    },

    /**
     * Get a table cells row name.
     *
     * @param {Object} element A jQuery cell element reference.
     * @type String
     */
    getRowName: function(element) {
        return element.siblings(":first").text();
    },

    /**
     * Get a table cells x and y coordinates.
     *
     * @param {Object} element A jQuery cell element reference.
     * @type Object
     * @return An object having x and y properties.
     */
    getXYCoordinates: function(element) {
        var previousColumns = element.prevAll();
        var previousRows = element.parent().prevAll();
        return {x:previousColumns.length, y:previousRows.length-1};
    },

    /**
     * Handle a menu item selection and pass an intention(s) to (a) parent(s).
     *
     * @param {Object} event.
     * @type Boolean
     * @return A false value.
     */
    onMenuItemSelect: function(event) {
        var intentions = [];
        var element = $(event.target); //Cast to a jquery element.
        var resource = element.attr("href");
        try {
            intentions = JSON.parse(unescape(resource));
        }
        catch(err) {
            this.logger.error("Could not get intention from menu item selection.", err);
        }
        var context = new Splunk.SearchContext("*");
        for (var i=0; i<intentions.length; i++) {
            context.addIntention(intentions[i]);
        }
        this.passContextToParent(context);
        return false;//Cancel the click/keyboard event from making a request.
    },

    /**
     * Retrieve the inferred data results or events. If entityName is "auto" defers to job areResultsTransformed method.
     */
    getInferredEntityName: function() {
        if (this.entityName=="auto") {
            return (this.getBaseContext().job.areResultsTransformed())?"results":"events";
        } else {
            return this.entityName;
        }
    },

    getResultParams: function($super) {
        var params = $super();
        var baseContext = this.getBaseContext();
        var sid         = baseContext.job.getSearchId();
        var timeRange   = baseContext.getTimeRange();

        if (!sid) this.logger.error(this.moduleType, "Assertion Failed. getResultParams was called, but searchId is missing from my job.");

        params.sid = sid;
        params.entity_name = this.getInferredEntityName();
        params.display_row_numbers = this.getSetting("displayRowNumbers");

        if (baseContext.job.isPreviewable()) params.show_preview = "1";

        var postprocess = baseContext.getPostProcess();
        if (postprocess) params.postprocess = postprocess;

        // Check that this search has any transforming commands.
        // If it does, then our field_list argument will have unintended consequences
        // and will generally be meaningless.
        // TODO: this currently has a quirk that makes tabular event data show
        //      all fields when entityName=events, isTransforming=true
        var dataContext = this.getBaseContext();
        if (!Splunk.util.normalizeBoolean(this.getParam('allowTransformedFieldSelect'))
            && dataContext.job.areResultsTransformed()) {
            delete params["field_list"];
        }
        return params;
    },

    onRowClick: function(evt) {
        var el = evt.target;
        if (el.tagName != 'TD') return;
        // skip the number column
        if ($(el).hasClass('pos')) return;
        if(this.displayMenu){
            this.menuFactory(evt);
        }
    },

    /**
     * Handle setting changes. See $super for implementation details.
     *
     * @param {Object} settings An object literal containing various types for values.
     */
    onSettingsChange: function($super, settings){
        // 
        // if AbstractPagedModule.onSettingsChange fired a request already it will return true 
        // in which case we dont need to worry about also re-rendering here.
        // Also the settingsMap has already been updated,  so as long as we always use this.getSetting() 
        // to retreive our settings, there's no need to update any properties
        if ($super(settings)) return;
        // another point where getBaseContext refactoring is needed. 
        // this can be called during resurrection, and calling getBaseContext() when there's no 
        // local property will backfill with "*" all time searches.  Which get dispatched
        // and we kill kittens.
        // as written below, we'll only get results when we already have results to get. 
        if (this.baseContext && this.getBaseContext().isJobDispatched() && settings.hasOwnProperty("displayRowNumbers")){
            this.getResults();
        }
        else if (settings.hasOwnProperty("dataOverlayMode")){
            this.onResultsRendered();
        }
    },
    
    popupCancel: function() {
        return true; //closes popup
    },

    popupAccept: function() {
        return true; //closes popup
    },

    onRowMouseover: function(evt) {
        var el = evt.target;
        if (el.tagName != 'TD') return false;
        $(el).parent().addClass('mouseoverHighlight');
    },

    onRowMouseout: function(evt) {
        var el = evt.target;
        if (el.tagName != 'TD') return false;
        $(el).parent().removeClass('mouseoverHighlight');
    },

    /**
     * Handle a new search. See $super for implementation details.
     */
    onContextChange: function($super) {
        this.renderedCount = 0;
        $super();
    },

    onJobProgress: function(event) {
        if (this.getInferredEntityName()=="events") {
            var context = this.getBaseContext();
            var needMoreEvents = this.renderedCount < this.getSetting("count");
            var moreEventsAreAvailable = context.job.getEventAvailableCount()>this.renderedCount;
            if (needMoreEvents && moreEventsAreAvailable){
                // Need to remember that we've already asked for these events
                // for the case when the next progress event occurs before renderResults.
                // without this assignment we will get multiple calls to getResults 
                // competing with eachother or perhaps aborting eachother.
                this.renderedCount = Math.min(context.job.getEventAvailableCount(), this.getSetting("count"));
            
                //this.logger.info("onJobProgress triggering getResults");
                this.getResults();
            }
        } else {
            var context = this.getBaseContext();
            if (context.job.isPreviewable()) {
                this.getResults();
            } else {
                this.resultsContainer.html(_('Waiting for search to complete...'));
            }
        }
    },

    onJobDone: function(event) {
        if (this.getInferredEntityName()=="results") {
            this.getResults();
        }
    },
    /**
     * Reset the UI to its original state.
     */
    resetUI: function(){
        this.resultsContainer.html("");
    },
    /**
     * Override and set renderedCount. See $super for implementation details.
     */
    renderResults: function($super, htmlFragment) {
        $super(htmlFragment);
        if (this.getInferredEntityName()=="events") {
            this.renderedCount = $("tr", this.container).length - 1;
        }
    },

    onResultsRendered: function() {
        this.logger.debug('SimpleResultsTable.onResultsRendered - START');
        switch(this.getSetting("dataOverlayMode")) {
            case "none":
                this.resetHeatMap(this.container);
                this.decorateBounds(this.container, false, false);
                break;
            case "heatmap":
                this.decorateBounds(this.container, false, false);
                this.decorateHeatMap(this.container);
                break;
            case "highlow":
                this.resetHeatMap(this.container);
                this.decorateBounds(this.container, true, true);
                break;
            default:
                this.logger.warn("Unsupported dataOverlayMode settings value of", this.getSetting("dataOverlayMode"));
        }
    },

    decorateHeatMap: function(container) {

        // define heat map targets
        var high = [230, 49, 53];
        var low = [62, 135, 191];
        var neutral = [245 ,245 ,245];

        $('td[heat]', container).each(function() {
            var heat = parseFloat(this.getAttribute('heat'));
            var r2, b2, g2;
            if (heat != 0) {
                target = (heat > 0 ? high : low);

                r2 = parseInt(target[0] + (neutral[0] - target[0]) * (1-Math.abs(heat)), 10);
                b2 = parseInt(target[1] + (neutral[1] - target[1]) * (1-Math.abs(heat)), 10);
                g2 = parseInt(target[2] + (neutral[2] - target[2]) * (1-Math.abs(heat)), 10);

                this.style.backgroundColor = 'rgb(' + r2 + ',' + b2 + ',' + g2 + ')';
            }
        });

    },

    resetHeatMap: function() {
        $('td[heat]', this.container).each(function() {
            this.style.backgroundColor = '';
        });

    },

    decorateBounds: function(container, showLow, showHigh) {
        if (showHigh) {
            $('td[isMax]', container).addClass('highValue');
        } else {
            $('td[isMax]', container).removeClass('highValue');
        }

        if (showLow) {
            $('td[isMin]', container).addClass('lowValue');
        } else {
            $('td[isMin]', container).removeClass('lowValue');
        }
    }
});
