#
# Splunk UI module python renderer
# This module is imported by the module loader (lib.module.ModuleMapper) into
# the splunk.appserver.mrsparkle.controllers.module.* namespace.
#


# required imports
import cherrypy
import controllers.module as module

# common imports
import splunk, splunk.search, splunk.util, splunk.entity
import lib.util as util
import lib.i18n as i18n

# logging setup
import logging
logger = logging.getLogger('splunk.appserver.controllers.module.SingleValue')

# define standard time field name        
TIME_FIELD = '_time'


class SingleValue(module.ModuleHandler):
    
    def generateResults(self, host_app, client_app, sid, entity_name='results', label=None, field=None, classField=None, format=None, postprocess=None):
    
        # assert input
        if not sid:
            raise Exception('SingleValue.generateResults - sid not passed!')
            
        # get job
        try:
            job = splunk.search.getJob(sid, sessionKey=cherrypy.session['sessionKey'])
        except splunk.ResourceNotFound:
            return _('Invalid Job')
        
        # set formatting
        job.setFetchOption(output_time_format=i18n.ISO8609_MICROTIME)
        # set the postprocess arg.
        if postprocess:
            job.setFetchOption(search=postprocess)

        for result in job.results:
            # displayable fields
            fieldNames = [x for x in getattr(job, entity_name).fieldOrder if (not x.startswith('_') or x == TIME_FIELD)]
            if len(field):
                if field not in fieldNames:
                    return _('Invalid Field')
            else:
                field = fieldNames[0]
            value = result[field][0].value
            
            try:
                if format == 'number':
                    value = i18n.format_number(float(value))
                elif format == 'decimal':
                    value = i18n.format_decimal(float(value))
                elif format == 'percent':
                    value = i18n.format_percent(float(value))
            except Exception, e:
                logger.error('Unable to cast value to %s; got %s' % (format, value))
                
            if len(classField):
                className = result[classField][0]
                return "%s//!-!//%s" % (value, className)
            return value
            
        return _('N/A')

