
Splunk.Module.SuggestedFieldViewer = $.klass(Splunk.Module.MultiFieldViewer, {
    
    initialize: function($super, container) {
        $super(container);
        this.logger = Splunk.Logger.getLogger("SuggestedFieldViewer.js");
        this._fieldHeaderText = _("Other interesting fields (%s)");
        
        this.exclusionRegexes = [];
        for (var i=0, len=this._params["exclude"].length; i<len; i++) {
            var regEx = new RegExp(this._params["exclude"][i]);
            this.exclusionRegexes.push(regEx);
        }
    },
    getResultParams: function($super) {
        var params = $super();
        // (currently defaults in conf file to 0.2)
        params["min_freq"] = this._params["minFrequency"];
        
        if (this.hasSetting("fields")) {
            delete params["field_list"];
        }
        return params;
    },
    /**
     * MultiFieldViewer breaks this out as a method, basically so subclasses
     * can override it and provide some different view of the fields. 
     * In this case, we have overridden getResultParams to NOT pass 
     * "fields".  And then here we filter out based on our criteria.
     */
    getMatchingFields: function(fieldNodes) {
        var selectedFields = this.getSetting("fields");
        var moduleInstance = this;
        var matchingFields = [];

        var minDistinctCount= this._params["minDistinctCount"];

        $.each(fieldNodes, function(index) {
            var fieldElt  = $(this);
            var fieldName = fieldElt.attr("k");
            // we *DONT* want selected fields.  The point of this module is to suggest 
            // unselected fields.
            if (selectedFields && selectedFields.indexOf(fieldName) !=-1) return true;
            
            // go through our exclusion regexes.
            for (var i=0, len=moduleInstance.exclusionRegexes.length; i<len; i++) {
                if (fieldName.match(moduleInstance.exclusionRegexes[i])) return true;
            }

            //var isExact         = Splunk.util.normalizeBoolean(fieldElt.attr("exact"));
            //var isNumeric       = fieldElt.find("mean").length > 0;

            // check against the minimum distinctCount as specified in the view config.
            var distinctCount   = fieldElt.attr("dc");
            if (distinctCount >= minDistinctCount) {
                matchingFields.push(fieldElt);
            } 
        });
        // truncate down to maxFields;
        if (matchingFields.length > this._params["maxFields"]) {
            matchingFields = matchingFields.splice(0, this._params["maxFields"]);
        }
        return matchingFields;
    },
    getCustomLinks: function($super, fieldName, count, distinctCount, isExact, isNumeric) {
        var customLinks = $super(fieldName, count, distinctCount, isExact, isNumeric);
        customLinks.push($('<a href="#"></a>')
                .text(_("Select/show in results"))
                .click(this.addToSelected.bind(this)));
        return customLinks;
    },
    addToSelected: function() {
        var fieldList = this.getSetting("fields");
        // suppress duplicates
        if ($.inArray(this.fieldSummaryLayer.fieldName, fieldList) > -1) {
            return false;
        }
        fieldList.push(this.fieldSummaryLayer.fieldName);
        var intention = {"name": "setfields", "arg": fieldList};
        var fieldsContext = new Splunk.SearchContext();
        fieldsContext.addIntention(intention);
        this.passContextToParent(fieldsContext);
        return false;
    },
    updateHeader: function($super, fieldCount, isNumeric) {
        $super(fieldCount, isNumeric);
        if (fieldCount == 0) this.container.hide();
        else this.container.show();
    },
    renderResults: function($super, xmlDoc) {
        $super(xmlDoc);
        // out with the old
        $(".allFieldsLink", this.container).remove();
        var fieldPickerHeader = $(".FieldPicker .inlineHeader");
        var fieldPickerCount = this.getBaseContext().job.getEventFieldCount();
        if (fieldPickerHeader && fieldPickerCount) {
            // in with the new.
            var allFieldsLink = $("<a>")
                .addClass("allFieldsLink")
                .addClass("splBorder-n")
                .attr("href", "#")
                .click(function(evt) {
                    $("a.fpActivate", fieldPickerHeader).click();
                    return false;
                })
                .text(sprintf(ungettext("%(totalFieldCount)s Field", "All %(totalFieldCount)s Fields", fieldPickerCount), {totalFieldCount: fieldPickerCount}));
            this.container.append(allFieldsLink);
        } else {
            this.logger.error("no FieldPicker found in this view");
        }
    },
    onSettingsChange: function(settings) {
        if ("fields" in settings) {
            this.getResults();
        }
    },
    resetUI: function($super) {
        $super();
        $(".allFieldsLink", this.container).remove();
    }
});
