Splunk.Module.BaseChartFormatter = $.klass(Splunk.Module, {

    initialize: function($super, container){
        $super(container);

        this.logger = Splunk.Logger.getLogger("BaseChartFormatter.js");

        this._formElement = $();


        // try to init the default <select> element
        var elements = $("select", this.container);
        if (elements.length > 0) {
            this._formElement = $(elements[0]);
            if (elements.length > 1) {
                logger.warn('initialize - ' + this.moduleType + ' has extraneous form elements');
            }

            this._formElement.bind('change', this.handleInputChange.bind(this));
        }

        this._settingName = "SETTING_NAME_NOT_SET";

        // this is in the process of being pulled up into the Module class itself.
        this._baseSettingsMap = {};

        // init the internal param map with the default value
        //this.setParam('default', this._formElement.val(), true);
        this.assertParamsMatchUI('initialize');
    },

    /**
     * Returns the single setting associated with this chart formatter module.
     * The actual setting name is defined in this._settingName, but the persistence
     * param name for all ChartFormatters is 'default'.  This is an optional stanza
     * that can be added to the conf file and persisted.
     *
     */
    getSettings: function() {
        var args = {};
        if (this.isCompatibleWithSettings(this.getBaseSettings())) {

            var context = this.getBaseContext();
            var plotIntention = context.popIntention("plot");
            // if theres no plot intention at all, we assume the app developer
            // knows what they're doing and we let the control be visible.
            if (!plotIntention || this.isCompatibleWithIntention(plotIntention)) {
                args[this._settingName] = this.getParam('default');
                this.assertParamsMatchUI("getSettings")
                return args;
            }
        }
        if (this._safeDefault != null) {
            args[this._settingName] = this._safeDefault;
            return args;
        }
    },
    getSetting: function(name) {
        if (name==this._settingName) return this.getParam('default');
        this.logger.error(this.moduleType + " has no value for setting name=", name);
    },

    assertParamsMatchUI: function(callingMethodName) {
        if (this.getParam("default") != this._formElement.val()) {
            this.logger.error("Assertion failed in ", this.moduleType, ".", callingMethodName, " . Params have drifted out of sync with the UI\ngetParam('default')=", this.getParam("default"), " formElement.val()=", this._formElement.val());
        }
    },  
    handleInputChange: function(evt) {
        this.setParam('default', this._formElement.val());
        this.logger.debug('handleInputChange - got default=' + this.getParam('default'));
        this.pushSettingsToChildren();
    },
    _showIfCompatible : function() {
        // there might be a better solution, but if nobody has ever given us a
        // a context reference, we cannot let getBaseContext() load it below
        // because in resurrection cases
        // a) this can trigger nasty jaBridge exceptions
        // b) a race condition in resurrection that may or may not have been resolved by
        //    reversing the order of pushSettings and pushContext.
        if (!this.baseContext) {
            this.hide();
        }

        if (this.isCompatibleWithSettings(this.getBaseSettings())) {

            var context = this.getBaseContext();
            var plotIntention = context.popIntention("plot");
            // if theres no plot intention at all, we assume the app developer
            // knows what they're doing and we let the control be visible.
            if (!plotIntention || this.isCompatibleWithIntention(plotIntention)) {
                this.show();
                this.enableCompatibleOptions();
            }
        } else {
            this.hide();
        }
    },
    /*
     * Subclasses will implement this method, which is how they turn themselves
     * visible or invisible depending on other settings being set above them.
     */
    isCompatibleWithSettings: function(settingsMap) {return true;},
    /*
     * Subclasses will implement this method, which is how they turn themselves
     * visible or invisible depending on other settings being set above them.
     */
    enableCompatibleOptions: function() {},

    /*
     * Not to be overridden, this base class implementation wires up the generic
     * logic of checking the isCompatibleWithSettings method.
     */
    onSettingsChange: function(settingsMap) {
        if (settingsMap.hasOwnProperty(this._settingName)){
            this._formElement.val(settingsMap[this._settingName]);
            this.setParam('default', settingsMap[this._settingName], true);
        }
        $.extend(this._baseSettingsMap, settingsMap);

        this._showIfCompatible();

    },
    /*
     *  Although these module work with Settings and not Context, their visibility
     * can depend on certain values in the SearchContext.
     * Subclasses will implement this function, which is how they turn themselves
     * visible or invisible depending on plotIntention properties being set above them.
     */
    isCompatibleWithIntention: function(plotIntention) {return true;},
    /*
     * Not to be overridden, this base class implementation wires up the generic
     * logic of checking the isCompatibleWithIntention method.
     */
    onContextChange: function() {
        this._showIfCompatible();
    }
});
Splunk.Module.BaseChartFormatter.isAbstract = true;
