Splunk.namespace("Module");

Splunk.Module.ChartTypeFormatter = $.klass(Splunk.Module.BaseChartFormatter, {

    initialize: function($super, container){
        $super(container);
        this.logger = Splunk.Logger.getLogger("chart_type_formatter.js");
        this._settingName = "chart";
        this.optionCompatibilityKey = "mode";
        this.optionCompatibilityTable = {
            "timechart" : {
                "values"  : ["line", "area", "column"],
                "default" : "line"
            },
            // we dont fully support chart, but if we can extract it we might as well disable 'pie'.
            "chart" : {
                "values"  : ["column", "line", "area", "bar", "scatter", "bubble"],
                "default" : "line"
            },
            "top"  : {
                "values"  : ["column", "bar", "pie"],
                "default" : "column"
            },
            "rare" : {
                "values"  : ["column", "bar"],
                "default" : "column"
            }
        };
    },
    
    /**
     * Adjust the chart type so that it makes sense when given a plot intention
     * mode.  Recognized modes are listed above in optionCompatibilityTable
     *
     */
    setCompatibleDefaults: function() {
        var context = this.getBaseContext();
        if (context.getViewStateId()) return false;
        var plotIntention = context.getIntentionReference("plot");
        
        if (plotIntention && plotIntention["arg"]) {

            var mode = plotIntention["arg"]["mode"];

            switch (mode) {
                
                case 'timechart':
                    var statOpsForColumn = {"sum":1};
                    var fields = plotIntention["arg"]["fields"];
                    var statop = fields[0][0];

                    // force specific stats commands into fixed chart types
                    if (statOpsForColumn.hasOwnProperty(statop)) {
                        this._formElement.val("column");

                    // switch the current widget option to a default only if the
                    // currently selection option is invalid
                    } else if (this.optionCompatibilityTable[mode]['values'].indexOf(this._formElement.val()) == -1) {
                        this._formElement.val("line");
                    }
                    break;
                    
                case 'top':
                case 'rare':
                    this._formElement.val("bar");
                    break;
                    
                default:
                    // nothing; currently catches the 'chart' mode
            }
            
            // TODO: temporary solution, to override the stickiness entirely
            this.setParam('default', this._formElement.val(), true);
        }
    },
    applyContext: function(context) {
        this.setCompatibleDefaults();
    },
    enableCompatibleOptions: function() {
        
        var context = this.getBaseContext();
        if (context.isResurrected()) {
            this.setCompatibleDefaults()
        }
        var plotIntention = context.getIntentionReference("plot");

        if (plotIntention && plotIntention["arg"]) {
            var defaultValue = false;
            var key = this.optionCompatibilityKey;
            var table = this.optionCompatibilityTable;
            var keyValue = plotIntention["arg"][key];

            if (table.hasOwnProperty(keyValue)) {
                defaultValue = table[keyValue]["default"];
                var legalValues = table[keyValue]["values"];

                this.withEachOption(function(i, element) {
                    if (legalValues.indexOf($(element).val())!=-1) {
                        this.enableOption($(element));
                    } else {
                        this.disableOption($(element));
                    }
                }.bind(this));
            }
        } else {
            this.enableAllOptions();
        }
        this.disableOption($("option[value='bubble']", this.container));
        //this.disableOption($("option[value='scatter']", this.container));
    },
    withEachOption: function(callback) {
        this._formElement.find("option").each(function(i, element){
            callback(i, element);
        });
    },
    enableOption: function(element) {
        element.removeAttr("disabled");
    },
    disableOption: function(element) {
        element.attr("disabled", "true");
    },
    enableAllOptions: function() {
        this.withEachOption(function(i, element) {
            this.enableOption($(element));
        }.bind(this))
    }
});
