Splunk.namespace("Module");

Splunk.Module.StackModeFormatter = $.klass(Splunk.Module.BaseChartFormatter, {
   
    initialize: function($super, container){
        $super(container);
   
        this.logger = Splunk.Logger.getLogger("StackModeFormatter.js");
        this._settingName = "chart.stackMode";
        // if the module is incompatible with the context or with the settings, it will 
        // return this value in addition to hiding itself.
        this._safeDefault = "none";
    },
    /*
     *  Although this module sets a Setting, it has some dependencies on logic
     *  in SearchContext.
     */
    isCompatibleWithIntention: function(plotIntention) {
        // stackMode is only relevant in chart or timechart
        if (plotIntention["arg"]["mode"] != "chart" && plotIntention["arg"]["mode"] != "timechart") return false;
        // if a module has dictated that there should be no splitType, we obey.
        if (plotIntention["arg"]["splitType"] == "none") return false;
        return true;
    },
    /*
     * only area, bar and column can be stacked. 
     * For line charts it's very misleading
     * for scatter, pie charts it's nonsense.
     */
    isCompatibleWithSettings: function(settingsMap) {
        if (!settingsMap.hasOwnProperty("chart")) return true;
        
        var relevantChartTypes = {"area":1, "bar":1, "column":1};

        return relevantChartTypes.hasOwnProperty(settingsMap["chart"]);
    },
    onSettingsChange: function($super, settingsMap) {
        this.logger.debug(this.moduleType, this.getBaseSettings());
        return $super(settingsMap);
    }
   
   
});