Splunk.namespace("Module");

Splunk.Module.FlashWrapper = $.klass(Splunk.Module.DispatchingModule, {
    
    _isBridgeConnected: false,

    initialize: function($super, container){
        $super(container);
        this.logger = Splunk.Logger.getLogger("flash_wrapper.js");

        this.swfObjectId = "swfObject_" + this.container.attr('id');
        this.bridge = new Splunk.JABridge(this.swfObjectId);
        this.minVersion = Splunk.util.getConfigValue("FLASH_MAJOR_VERSION") + "." + Splunk.util.getConfigValue("FLASH_MINOR_VERSION") + "." + Splunk.util.getConfigValue("FLASH_REVISION_VERSION");
        
        // used to communicate with PageStatus singleton about rendering in progress. 
        this.renderMonitor = false;
        // used to communicate with PageStatus singleton about the async swf loading. 
        this.swfLoadMonitor = Splunk.Globals['PageStatus'].register(this.moduleType + ' - loading swf file - ' + this.container.attr('id'));
        
        this.initializeBridge();
        if (swfobject.hasFlashPlayerVersion(this.minVersion)){
            this.addObjectStructure();
        } else {
            this.showFlashError();
        }
        $(document).bind("PrintStart", this.onPrintStart.bind(this));
        $(document).bind("PrintEnd", this.onPrintEnd.bind(this));
    },
    
    /**
     * Retrieve base64 encoded image snapshot of Flash movie for overlay and shutdown existing movie.  
     */
    onPrintStart: function(){
        if($.browser.msie){
            return;
        }    
        var snapshot = null;
        try {
            snapshot = this.bridge.callMethod("getSnapshot");
        } catch(e) {
            this.logger.warn("JABridge instance is either unavailable, not-connected or does not support callMethod getSnapshot");
        }
        if (snapshot){
            var img = document.createElement("img");
            img.setAttribute("width", snapshot.width);
            img.setAttribute("height", snapshot.height);
            this.logger.warn("closing down JABridge connection");
            this.bridge.close();
            $(this.bridge.getFlashElement(this.swfObjectId)).css("display", "none");
            $('.FlashWrapperContainer', this.container)[0].appendChild(img).src = snapshot.data;//bypass jquery for performance (base64 encoded images are large)
        }
    },
    
    /**
     * Destroy base64 encoded image snapshot of Flash movie and bring back Flash movie to previous state.
     */
    onPrintEnd: function(){
        if ($.browser.msie) {
            return;
        }
        $('.FlashWrapperContainer img', this.container).remove();
        $(this.bridge.getFlashElement(this.swfObjectId)).css("display", "");
        this.logger.warn("bringing back JABridge connection");
        this.connectBridge(true);
    },
    
    /**
     * Make SWF movies start. Workaround for offscreen flash initialization.
     * If the flash movie is offscreen, scroll briefly to it.
     */
    initSWFMovies: function(){
        // Workaround for SPL-17758. PART 1.  
        //if ($(window).scrollTop() + $(window).height()  < this.container.offset().top) {
        //    $(window).scrollTop(this.container.offset().top);
        //}
        // Workaround for SPL-17758. PART 2.  if we did the scrolling trick, then scroll back to 0 here.
        // i dont think onConnect fires multiple times, but just in case.
        //setTimeout(function(){$(window).scrollTop(0)}, 500);
    },
   
    /**
     * Current version of jQuery ui is buggy. Additional logic to make things work consistently.
     */
    enableResizable: function(){
        if (!($.browser.safari && $.browser.version < "526")) { //disable resizing for safari 3 and below only
            $("div.FlashWrapperContainer", this.container).resizable({autoHide: true, handles: "s", stop: function(){$(this).width('100%')}});
            $("div.FlashWrapperContainer").mouseup( //workaround until jquery ui is updated
                function(event){
                    $(this).width('100%'); 
                }
            );
        }
    },
    
    connectBridge: function(isReconnect) {
        if (!isReconnect)
            isReconnect = false;
        this.bridge.connect(function(){this.onConnect(isReconnect);}.bind(this), this.onClose.bind(this));
    },

    /**
     * Template method that subclasses can implement if they need to call 
     * bridge.addMethod or bridge.addEvent,  which can only be called after the 
     * bridge object has been constructed, but before the connect() method 
     * has been called.
     */
    initializeBridge: function() {},

    /**
     * Retrieve the normalized computed style for a specified element.
     * 
     * @param {Array} specificity An array of elements to try and find a related css property from. The first element to return a property exits the routine.
     * @param {String} cssProperty The css property following standard css property convention not camel case.
     * 
     * @type String || null
     * @return Returns a hexadecimal value of a matching element css selector property or null.
     */    
    getCSSColor: function(specificity, cssProperty){
        var color;
        for (var i=0; i<specificity.length; i++){
            var computedColor = specificity[i].css(cssProperty);
            color = Splunk.util.normalizeColor(computedColor);
            if (color){
                return color;
            }
        }
        return null;
    },
    
    /**
     * Handler for when SWFObject has embedded Flash content.
     * SWFObject adds movies asynchronously (absolutely unnecessary) so this is a workaround for all its stupidity.
     * 
     * @param {Object) event SWFObject event object having success, id and ref attributes.
     */
    onSWFReady: function(event){
        if (event.success) {
            this.connectBridge(false);
            this.enableResizable();
            //this.initSWFMovies();
        } else {
            this.logger.error("The embedding of the SWF was unsuccessful.");
        }
    },

    addObjectStructure: function(){
        var targetId = "FlashWrapperContainer" + this.container.attr('id');//SWFObject requires an explicit id.
        var swfUrl = Splunk.util.make_url("/static/flash/" + this._params["swfFile"]);
        var expressInstallUrl = false;
        var flashVars = {
            'basePath': Splunk.util.make_url('/splunkd')
        };
        var params = {
            wmode: "opaque"
        };
        var bgcolor = this.getCSSColor([this.container], "background-color");
        if(bgcolor){
            params["bgcolor"] = bgcolor;
        }
        var attributes = {
            id: this.swfObjectId,
            name: this.swfObjectId
        };
        $(".FlashWrapperContainer", this.container).append("<div id="+targetId+"></div>");//SWFObject does complete node replacement, not target child replacement.
        swfobject.embedSWF(swfUrl, targetId, "100%", "100%", this.minVersion, expressInstallUrl, flashVars, params, attributes, this.onSWFReady.bind(this));
    },

    showFlashError: function() {
        var msg = _("Splunk requires a newer version of Flash.")
        this.logger.warn(msg);
        var target = $("div.FlashWrapperContainer", this.container)[0];
        target.innerHTML = sprintf(
            '<p class="error">%s. (Minimum version: %s.%s.%s)</p>',
            msg,
            Splunk.util.getConfigValue('FLASH_MAJOR_VERSION'),
            Splunk.util.getConfigValue('FLASH_MINOR_VERSION'),
            Splunk.util.getConfigValue('FLASH_REVISION_VERSION')
        );
    },
    
    /**
     * All Flash modules that use the JaBridge, or any asynchronous loading
     * technology, will need to return true from requiresAsynchronousLoading,
     * and when they *are* loaded, they will need to trigger a 'moduleLoaded'
     * event and pass their container id.
     */
    //requiresAsynchronousLoading: function() {return true;},
    
    /**
     * Handle JABridge close event.
     */
    onClose: function(){
        this.logger.warn("The JABridge connection was closed with an id of", this.bridge.id());
        this._isBridgeConnected = false;
    },
    
    /**
     * Handle JABridge connect event.
     * 
     * @param {Boolean} isReconnect Controls if the flash movie should be brought back to life.
     */
    onConnect: function(isReconnect) {
        this._isBridgeConnected = true;
        this.setPresentation();
        // formerly we only called the rest if this was a reconnect attempt. 
        this.onSettingsChange(this.getBaseSettings());
        this.onContextChange();
        this.bridge.addEventListener('updated', this.onDataUpdated.bind(this));

        this.callBridgeMethod("update");
        this.swfLoadMonitor.loadComplete();
    },
    onJobProgress: function() {
        if (this.getBaseContext().job.isDone()) {
            // Notifying PageStatus that a render is beginning. 
            this.renderMonitor = Splunk.Globals['PageStatus'].register(this.moduleType + ' - rendering final data - ' + this.container.attr('id'));
        }
    },
    onDataUpdated: function() {
        if (this.getBaseContext().isJobDispatched()) {
        
            // renderMonitor is not assigned until the very last progress event. 
            if (this.renderMonitor) {
                this.renderMonitor.loadComplete();
                this.renderMonitor = false;
            }
        }
    },
    
    _changeVisibility: function(invisibilityMode) {
        var visible = true;
        for (var mode in this._invisibilityModes) {
            if (this._invisibilityModes.hasOwnProperty(mode)) {
                visible = false;
            }
        }
        this.logger.info("Change visibility of jabridge to", visible);
        if(visible){
            this.container.show();
            this.connectBridge(true);
        }else{
            this.bridge.close();
            this.container.hide();
        }
    },
    
    /**
     * Set presentation control settings on Flash movies. Used for skinning.
     * 
     * Example CSS:
     * .YOURCONTAINER {
     *    background-color:#CCC; -> backgroundColor
     *    border-left-color:#000; -> foregroundColor
     *    color:#FFF; -> fontColor
     *    border-right-color:#FFF; -> seriesColor
     * }
     */
    setPresentation: function(){
        var styleMap = [
            {css: "border-left-color", flash: "foregroundColor"},
            {css: "color", flash: "fontColor"},
            {css: "background-color", flash: "backgroundColor"}
        ];
        for(var i=0; i<styleMap.length; i++){
            var styleMapAttributes = styleMap[i];
            var value = this.getCSSColor([this.container], styleMapAttributes.css);
            if(value){
                try{
                    this.bridge.callMethod("setValue", styleMapAttributes.flash, value.replace("#", "0x"));
                    this.logger.info("Set", styleMapAttributes.flash, value);
                }catch(e){
                    this.logger.error("Could not setValue", styleMapAttributes.flash);
                }
            }
        }
    },
    
    getBridgeProperty: function(name) {
        try {
            return this.bridge.getProperty(name);
        } catch(e) {
            this.logger.error("externalInterface/jabridge exception on getProperty('", name, "')", e);
            return false;
        }
    },
    
    setBridgeProperty: function(name, value) {
        if (!this._isBridgeConnected) return;
        try {
            this.bridge.setProperty(name, value);
        } catch(e) {
            this.logger.error("externalInterface/jabridge exception on setProperty('", name, "', '", value, "')", e);
        }
    },
    
    callBridgeMethod: function() {
        if (!this._isBridgeConnected) return;

        try {
            this.bridge.callMethod.apply(this, arguments);
        } catch(e) {
            this.logger.error("externalInterface/jabridge exception on callMethod()", e);
        }
    }
});
