
Splunk.Module.MaxLines = $.klass(Splunk.Module.DispatchingModule, {

    // defines the parameter key both for
    // a) persisting the maxLines setting to the persistence layer, and
    // b) in the settingsMap to communicate with other modules downstream.
    MAXLINES_KEY: 'maxLines',


    initialize: function($super, container) {
        $super(container);
        this.logger = Splunk.Logger.getLogger("MaxLines.js");
        this.select = $("select[name='maxLines']", this.container);

        // provide a fallback for unit tests.
        if (this.select.length==0) {
            this.select = $("<select/>").appendTo(this.container);
        }

        // attach handlers
        this.select.bind("change", this.handleSelectChange.bind(this));
    },

    ///////////////////////////////////////////////////////////////////////////
    // subclassed methods
    ///////////////////////////////////////////////////////////////////////////

    getSettings: function() {
        var output = {}
        output[this.MAXLINES_KEY] = this.getSelectedMaxLines();
        return output;
    },


    ///////////////////////////////////////////////////////////////////////////
    //  Internal methods
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Returns selected option value casted as a number.
     */
    getSelectedMaxLines: function() {
        var val = this.getParam(this.MAXLINES_KEY);
        if (!val) {
            this.setParam(this.MAXLINES_KEY, this.select.val(), true);
        }
        val = parseInt(this.getParam(this.MAXLINES_KEY), 10);
        if (isNaN(val) || val < 0) {
            this.logger.error('getSelectedMaxLines - Invalid max lines value; defaulting to 10');
            return 10;
        }
        return val;
    },


    /**
     * Handles user selection change on the max lines selector
     */
    handleSelectChange: function(evt) {

        this.setParam(this.MAXLINES_KEY, this.select.val());
        this.logger.debug('handleSelectChange - changing maxLines=' + this.getParam(this.MAXLINES_KEY));
        this.pushSettingsToChildren();
    }

});