Splunk.Module.FieldPicker = $.klass(Splunk.Module.HiddenFieldPicker, {
   
    _timeoutId: null,
    _renderTimeoutInterval: 250,

    initialize: function($super, container) {
        $super(container);
        this._requestInProgress = false;
        
        // FieldSummaryLayer 
        this._fieldLayerContainer = $(".fieldLayers",this.container);
        this._fieldTabsContainer =  $(".fieldTabs", this.container);
        this._fieldHeaderText = _("Selected fields (%s)");
        // 1 Initialize FieldSummaryLayer 
        var popupLayerTemplate = $('.popupContainer', this.container);
        var reportLinkTarget = (this._params["link"]) ? this._params["link"]["view"] : false;        
        this.fieldSummaryLayer = new Splunk.Popup.FieldSummaryLayer(
            popupLayerTemplate, 
            this._fieldLayerContainer,
            reportLinkTarget
        );
        this.displaySideBar(Splunk.util.normalizeBoolean(this.getParam("sidebarDisplay")));
        // call event handler initialize
        this.setupEventHandlers();
    },
    /**
     * Control the display of the side bar. If the side bar is not in the DOM pass through.
     * 
     * @parm {Boolean} display The desired display state, true to show false to hide.
     */
    displaySideBar: function(display) {
        var control = $("a", this.container.siblings(".sidebarControl"));
        var currentDisplay = (control.hasClass("splIcon-sidebar-closed"))?false:true;
        if(currentDisplay!=display){
            control.click();
        }
    },
    /**
     * Observe changes in the side bar, update param value for persistence .
     * 
     * @param {Object} evt A jQuery event.
     */
    onSideBarChange: function(evt){
        display = ($(evt.target).hasClass("splIcon-sidebar-closed"))?true:false;
        this.setParam("sidebarDisplay", display);
    },
    onBeforeJobDispatched: function(context) {
        // needs at least one status bucket in it's dispatched searches.
        context.setMinimumStatusBuckets(1);
        // needs *all* the fields in order to do it's job.
        context.setRequiredFields(["*"]);
    },
    /**
     * overriding the HiddenFieldPickers  method, so as to implement the persistence functionality
     * as declared in this module's conf file.
     */
    setSelectedFieldList: function($super, list, writeSessionOnly) {
        $super(list, writeSessionOnly);
        this.updateSelFieldsNumber(list.length);
        // send out to persistence layer
        this.setParam('fields', Splunk.util.fieldListToString(this._selectedFields), writeSessionOnly);
    },
    applyContext: function(context) {
        if (!context.isResurrected()) {
            var fieldsIntention = context.getIntentionReference("setfields");
            if (fieldsIntention) {
                var fieldList = fieldsIntention["arg"];
                // commit to persistence layer
                this.setSelectedFieldList(fieldList);
       
                // notify everyone else
                this.pushSettingsToChildren();
                return true;
            }
        }
    },
    /* set up event handlers on load.  NOTE: these could all be assigned to one top-level listener */
    setupEventHandlers: function() {
        // bind click action to handler
        this.container.find('.fpActivate').click( function(){
            this.onPopupClick();
        }.bind(this));

        // bind clicks in the fields list to an event handler
        this.container.find('.fpFieldList .scrollContent').click( function(evt){
            this.onFieldListClick(evt);
        }.bind(this));

        // bind clicks in the selected fields list to an event handler
        this.container.find('.fpSelectedFields')
            .click( function(evt){
                this.onSelFieldsClick(evt);
            }.bind(this));

        //bind clicks on add all link 
        $('.fpAddAllLink', this.container).click(function(){
            this.addAllTerms();
        }.bind(this));

        //bind clicks on remove all link
        $('.fpClearSelectedFields', this.container).click(function(){
            this.removeAllTerms();
        }.bind(this));
        
        //observer clicks on the sidebar display
        $("a", this.container.siblings(".sidebarControl")).click(function(evt){
            this.onSideBarChange(evt);
        }.bind(this));
        
        // document clicks should hide field value popups
        $(document).click(function(evt) {this.hideFieldValuePopup()}.bind(this));
        // esc key should also hide them...
        $(document).bind('keydown.FieldPicker', function(event) {
            if (event.keyCode == 27) {
                this.hideFieldValuePopup();
            }
        }.bind(this));
    },
    /**
     * added just for SPL-27486.  There's a more general problem in this module
     * where values with unsafe chars can get used in jquery selectors. 
     * however it only seems to be manifesting in removeFieldFromSelected so im 
     * just using this in that one place for now.
     */
    withTermLi: function(containerSelector, fieldName, callback) {
        // escapeHTML is a poor man's isSafeForJquerySelector()
        if (fieldName == Splunk.util.escapeHtml(fieldName)) {
            var elt = $(containerSelector + ' > [term="' + fieldName + '"]');
            callback(elt);
        } else {
            $(containerSelector + ' li').each(function(i, elt) {
                if ($(elt).attr("term") == fieldName) {
                    callback($(elt));
                    return false;
                }
            });
        }
    },
    
    
    onPopupClick: function() {
         // open field picker.  
         // we retain an instance because in at least one case we need to destroy it not from within either of 
         // it's callbacks.
         this.popupInstance = new Splunk.Popup($('.fpPopupContainer'), {
            title : _("Fields"),
            pclass : "FieldPickerPopup",
            buttons : [
                {
                    label: 'Cancel',
                    type : 'secondary',
                    callback: function(){
                        return this.popupCancel();
                    }.bind(this)
                },
                {    
                    label: 'Save',
                    type : 'primary',
                    callback: function(){
                        return this.popupAccept();
                    }.bind(this)
                }
            ]
        });

        this.popupHandle = $(this.popupInstance.getPopup());

        // make call to populate
        this.getResults();

        // make selected field list sortable
        this.popupHandle.find('.fpSelectedFields').sortable({
            axis: 'y',
            cursor: 'move'
        });
        
        // attach event handlers to the popup form.
        this.initFiltering();

    },
    
    
    popupCancel: function(){
        this.popupHandle.find('.fpSelectedFields').sortable("destroy");
        this.hideFieldValuePopup();
        return true; //closes popup
    },
    
    
    popupAccept: function(){
        this.commitChanges();
        this.hideFieldValuePopup();
        return true; //closes popup
    },
    
    
    onFieldListClick: function(evt) {
        var originator = evt.target;

        var fieldTerm = null;

        //nasty big conditional block to figure out what was clicked on, other solutions here welcome
        if ( $(originator).hasClass('fpFieldTerm')) { //click event happened on the field term td
            this.onFieldTermClick($(originator));
        } else if ( $(originator).parents('td.fpFieldTerm').length ) {  //click event happened on the div or span inside the field term td
            this.onFieldTermClick($(originator).parents('td.fpFieldTerm'));
        } else if ( $(originator).hasClass('fpAddTermCell') ) {  //click happened on arrow cell
            this.onFieldTermClick($(originator).next('td.fpFieldTerm'));
        } else if ( $(originator).parents('td.fpAddTermCell').length ) {  //click happened on the span inside arrow cell
            this.onFieldTermClick($(originator).parents('td.fpAddTermCell').next('td.fpFieldTerm'));
        } else if ( $(originator).is('a') && $(originator).parents('.fpFieldListSecond').length ) { //click event happened on the field value popup trigger (count field)
            evt.stopPropagation();
            this.doFieldValuePopup(evt);
        }

    },
    
    
    onFieldTermClick: function(fieldTerm) {
        if ( fieldTerm.parent('tr').hasClass('fieldSelected') ) {
            //remove selected class
            fieldTerm.parent('tr').removeClass('fieldSelected');
            this.removeFieldFromSelected ( fieldTerm.attr('term') );                           
        } else {
            //add selected class
            fieldTerm.parent('tr').addClass('fieldSelected');
            this.addFieldToSelected ( fieldTerm.attr('term') );
        }
    },
    
    onSelFieldsClick : function(evt) {
        var originator = evt.target;

        if ( $(originator).hasClass('fpSelFieldsNotPresent')  || $(originator).parents('li.fpSelFieldsNotPresent').length ) {
            return false;
        }

        var fieldTerm = null;
       
        if ( $(originator).is('span') ) {
            fieldTerm = $(originator).parent('li').attr('term');
            this.popupHandle.find('.fpFieldList [term=' + fieldTerm + ']').parent('tr').removeClass('fieldSelected');
            this.removeFieldFromSelected(fieldTerm);
        }
       
    },
    
    
    addAllTerms: function(){
        var fieldPickerInstance = this;
        $('.fpFieldList tbody tr', this.popupHandle).not('.fieldSelected').each(function(){
            term = $(this).children('.fpFieldTerm').attr('term');
            fieldPickerInstance.addFieldToSelected(term);
            $(this).addClass('fieldSelected');
        });
    },
    
    
    removeAllTerms: function(){
        $('.fpSelectedFields li', this.popupHandle).remove();
        $('.fpFieldList tbody tr.fieldSelected', this.popupHandle).removeClass('fieldSelected');
    },
    
    
    removeFieldFromSelected: function (fieldTerm) {
        var callback = function(elt) {
            elt.remove();
        }
        this.withTermLi('.fpSelectedFields', fieldTerm, callback);
        
    },


    addFieldToSelected: function ( fieldTerm ) {
        if (!fieldTerm) {
            return;
        }
        //add the selected item to the selected list
        var f = $('<li />').attr('term', fieldTerm).text(fieldTerm);
        $('<span class="splIcon splIcon-arrow-w" />').prependTo(f);
        f.appendTo($('.fpSelectedFields', this.popupHandle));
    },
    
    
    updateSelFieldsNumber: function (n) {
        $('.selectedFieldsCount', this.container).text(sprintf(ungettext('%d field', '%d fields',n), n));
    },
    
    
    updateTotalFieldsNumber: function (n) {
        if (!n) n=0;
        $('.fpPopupContainer .totalFieldsCount', this.container).text(n);
        $('.inlineHeader .totalFieldsCount', this.container).text(sprintf(ungettext('%d field', '%d fields',n), n));
        $('.inlineHeader', this.container).show();
    },
    
    
    getResultParams: function() {
        var params = {
            "top_count" : 0
        };
        var filterVal = $(".fpKeywordFilterField", this.popupHandle).val();
        if ((filterVal != undefined) && (filterVal.length > 0)) {
            params["search"] = filterVal;
        }

        var frequencyVal = $(".fpFrequencyFilterField", this.popupHandle).val();
        if ((frequencyVal != undefined) && (frequencyVal.length > 0)) {
            params["min_freq"] = frequencyVal / 100;
        }
        return params;
    },
    
    
    /**
     * Instead of getting some preformatted results, we'll have FieldPicker
     * hit the job/summary endpoint and use renderResults to handle some XML.
     */
    getResultURL: function(params) {
        params = params || {};
        var context = this.getBaseContext();
        return context.getUrl('summary', params);
    },
   
   
    /**
     * Render results takes the xml response and turns it into table rows
     */
    renderResults: function(xml) {

        // loop over XML data, generate <tr> nodes and insert
        var fieldTerms = [];

        var module = this;
        var fields = $(xml).find("summary").find("field");

        var numberOfEvents = parseInt($(xml).find("summary").attr('c'), 10);

        // get reference to tbody element
        var fieldListTable = this.popupHandle.find('table.fpFieldList tbody.scrollContent');

        fields.each(function() {
            var elem = $(this);

            // get count
            var count = format_number(elem.attr('c'));
            var distinctCount = format_number(elem.attr('dc'));
            var isExact = Splunk.util.normalizeBoolean(elem.attr('exact'));
            var isNumeric = (elem.attr("nc") > count/2);

            // get frequency
            var freq = parseFloat(elem.attr('c')) / numberOfEvents;
            var display_freq = format_percent(freq);
            
            // generate table row
            var row = $('<tr/>');
            var cell1 = $('<td class="fpAddTermCell"><span class="splIcon splIcon-arrow-e"></span></td>').appendTo(row);
            var cell2 = $('<td class="fpFieldTerm fpFieldListFirst splBorder-e"/>')
                .attr('term', elem.attr('k'))
                .append($('<div/>'))
                .append($('<span/>')).text(elem.attr('k'))
                .appendTo(row);
            var cell3Link = $('<a href="javascript:void(0)"/>')
            .attr('s:fieldname', elem.attr('k'))
            .attr('s:count', count)
            .attr('s:distinctcount', distinctCount)
            .attr('s:eventcount', numberOfEvents)
            .attr('s:isnumeric', isNumeric)
            .attr('s:isexact', isExact)
            .text((isExact ? '' : '>') + distinctCount);
            var cell3 = $('<td class="fpFieldListSecond splBorder-e"/>')
            .append(cell3Link)
            .appendTo(row);            
            var cell4 = $('<td class="fpFieldListThird"/>').text(display_freq).appendTo(row);
            
            fieldListTable.append(row);
            
        });

        // render the output
        this.addSelectedFieldsToDOM();
       
        // table sorting
        $('.fpFieldList', this.popupHandle).trigger("update");
        
        // custom sorter for percentage with operator support
        $.tablesorter.addParser({ 
            // set a unique id 
            id: 'percentoperators', 
            is: function(s) { 
                // return false so this parser is not auto detected
                return false; 
            }, 
            format: function(s) {
                var s = parseInt(s.replace("≥", ""), 10);
                return s;
            }, 
            // set type, either numeric or text 
            type: 'numeric' 
        });
        
        $('.fpFieldList', this.popupHandle).tablesorter( {
            headers: {
                2: {
                    sorter: 'percentoperators'
                }
            },
            sortList: [[1,0]] 
        });
       
        // Handle hover effects for IE6
        if ( $.browser.msie && $.browser.version == '6.0' ) {
            $('.fpFieldList tbody tr').hover(function(){
                    $(this).addClass('mouseoverHighlight');
                },
                function(){
                    $(this).removeClass('mouseoverHighlight');
            });
        }
    },
    
    
    addSelectedFieldsToDOM: function() {
        var selectedListContainer = $('.fpSelectedFields', this.popupHandle);
        selectedListContainer.html('');
        
        var tmpFieldList = this.getSelectedFieldList();
        
        for (var i=0,l=tmpFieldList.length; i<l; i++) {
            var fieldTerm = tmpFieldList[i];
            if (fieldTerm.charAt('0') == '_') continue;
            
            var f = $('<li/>').attr('term', fieldTerm).text(fieldTerm);
            $('<span class="splIcon splIcon-arrow-w" />').prependTo(f);
            f.appendTo(selectedListContainer);

            //add selected class
            $("td[term='" + fieldTerm + "']").parent('tr').addClass('fieldSelected');
        }

        this.popupHandle.find('.fpSelectedFields').sortable("refresh");
    },
    
    
    initFiltering: function() {
        var table = $('.fpFieldList', this.popupHandle);
        var fieldPickerInstance = this;        
        
        $('.fpFilterFields', this.popupHandle).click(function(evt){
            orig = evt.target;

            if ( $(orig).hasClass('fpKeywordFilter') || $(orig).parents('.fpKeywordFilter').length ) { //keyword filter click
                $('.fpKeywordFilterField', this.popupHandle).val('');
                fieldPickerInstance.getResults();
                //$.uiTableFilter( table, '', 'fpFieldListFirstHeader');
            } 
            if ( $(orig).hasClass('fpFrequencyFilter') || $(orig).parents('.fpFrequencyFilter').length ) { //keyword filter click
                $('.fpFrequencyFilterField', this.popupHandle).val('1');
                fieldPickerInstance.getResults();
                //$.uiTableFilter( table, '', 'fpFieldListFirstHeader');
            } 
        });
      
        // filter Name column table results
        $('.fpKeywordFilterField', this.popupHandle).keyup(function(){
            //$.uiTableFilter( table, this.value, 'Name');
            this._restartTimeout();
        }.bind(this) );

        //frequency filtering
        $('.fpFrequencyFilterField', this.popupHandle).keyup(function(){
             this._restartTimeout();
        }.bind(this) );

    },


    // Stuff to deal with deferring the re-population of the field list
    // until the user has stopped typing.
    _deferredRender: function() {
        this._stopTimeout();
        this.getResults();
    },
    
    
    _startTimeout: function() {
        this._timeoutId = window.setTimeout( this._deferredRender.bind(this), this._renderTimeoutInterval);
    },
    
    
    _stopTimeout: function() {
        if (this._timeoutId) {
            window.clearTimeout(this._timeoutId);
            this._timeoutId = null;
        }
    },
    
    
    _restartTimeout: function() {
        this._stopTimeout();
        this._startTimeout();
    },
    
    narrowToSelected: function() {
        var context = new Splunk.SearchContext();
        var fieldName = this.fieldSummaryLayer.fieldName;

        var intentionArg = {};
        intentionArg[fieldName] = "*"
        var intention = {name: "addterm", arg: intentionArg};
        context.addIntention(intention);
        this.passContextToParent(context);
        this.hideFieldValuePopup();
        return false;
    },
    
    getCustomLinks: function(fieldName, count, distinctCount, eventCount, isExact, isNumeric) {
        var customLinks = [];
        var context = this.getBaseContext();
        var frequencyText = _("%(fieldName)s is in %(percent)s%% of results");
        var frequencyPercentage = Math.round(count / eventCount * 100);
        var frequencyArgs = {fieldName: fieldName, percent: frequencyPercentage};
            
        customLinks.push($("<span>" +  sprintf(frequencyText,frequencyArgs) + "</span>"));
        
        customLinks.push($('<a href="#"></a>')
            .text(_("Show only events with this field"))
            .click(this.narrowToSelected.bind(this)));
        return customLinks;
    },    

    onFieldSummaryLayerRender: function() {
        this._requestInProgress = false;
    },    
    
    onFieldValueClick: function(evt, fieldName, fieldValue) {
        var intention = {};
        intention["name"] = (evt.altKey)? "negateterm" : "addterm";
        intention["arg"] = {}
        intention["arg"][fieldName]  = Splunk.util.escapeSearchTerm(fieldValue);

        var context = new Splunk.SearchContext("*");
        context.addIntention(intention);
        this.passContextToParent(context);
    },
    
    // Function to create and pop sub-popup.  Not using splunk.popup because we don't want additional baggage (overlays),
    //  and this is relatively straightforward to do with just jquery.
    doFieldValuePopup: function(event) {
        this._requestInProgress = true;
        var target = $(event.target);
        var fieldName = target.attr("s:fieldname");
        var count = parseInt(target.attr("s:count"), 10);
        var distinctCount = parseInt(target.attr("s:distinctcount"), 10);
        var eventCount = parseInt(target.attr("s:eventcount"), 10);
        var isNumeric = Splunk.util.normalizeBoolean(target.attr("s:isnumeric"));
        var isExact = Splunk.util.normalizeBoolean(target.attr("s:isexact"));
        
        //this._requestInProgress = true;
        //this._setSelectedTab(tab, fieldName);

        var onCancelled = function() {
            //this._setSelectedTab();
            //this._openTabFieldName = false;
        }.bind(this);
        
        var customLinks = this.getCustomLinks(fieldName, count, distinctCount, eventCount, isExact, isNumeric);

        //var isSelected = (this.getSetting("fields").indexOf(fieldName) !=-1);
        //if (this.fieldSummaryLayer) this.fieldSummaryLayer.close();
        
        // (step 1 was initializing it)   
        // 2. setField, and build the links in preparation for opening.
        this.fieldSummaryLayer.setField(
            this.getBaseContext(), 
            fieldName,
            count,
            distinctCount,
            eventCount,
            isNumeric,
            customLinks
        );

        // 2 Open it. 
        this.fieldSummaryLayer.open(target,onCancelled);
        
        // 3 Refresh the data, which we will call onJobProgress while it's open
        this.fieldSummaryLayer.refresh(
            this.onFieldSummaryLayerRender.bind(this),
            this.onFieldValueClick.bind(this),
            10
        );       
        /*
         Add back when we use a true mvc approach.
        var fieldValuePopup = $('.fpFieldValuePopupWrapper', this.container)
            .clone() //make a copy of the dom element 
            .appendTo($(this.popupHandle)); //attach it to the current field picker

        // set up popup contents
        $(fieldValuePopup)
            .show()
            .find('.splHeader h2').text(fieldTerm); // set popup header text to be field term
                
         $('body').bind('click.fpFieldValuePopup', function(evt) {
            var t = evt.target;

            if ( $(t).is('.splIcon-close') || ( !$(t).is('.fpFieldValuePopupWrapper') && !$(t).parents('.fpFieldValuePopupWrapper').length ) ) {
                $(fieldValuePopup).remove();
                $('body').unbind('click.fpFieldValuePopup');
            }
        }); 

        // load field summary info
        $('.popupContent', fieldValuePopup).html(_("Loading field summary information..."));
        var data = {
            activity: "summary",
            field_name: fieldTerm,
            sid: this.getBaseContext().job.getSearchId()
        };
        $.ajax({
            type: "GET",
            url: Splunk.util.make_url('module', Splunk.util.getConfigValue('SYSTEM_NAMESPACE'), this.moduleType, 'render'),
            data: data,
            dataType: "html",
            error: function(){
                this.logger.error("Could not load " + resource);
                $('.popupContent', fieldValuePopup).html(_("Could not load field summary information."));
            }.bind(this),
            complete: function(data, textStatus){
                $('.popupContent', fieldValuePopup).html(data.responseText);
            }.bind(this)
        });
        */
    },
    /**
     * close field value popup layers
     */
    hideFieldValuePopup: function() {
        if (this.fieldSummaryLayer) this.fieldSummaryLayer.close();
    },
    
    commitChanges: function() {

        //NOTE: not sure if it gains us anything to destroy and create the sorting every time.  trying for now
        this.popupHandle.find('.fpSelectedFields').sortable("destroy");

        // iterate over DOM list elements to get list of desired fields
        var tmpFieldList = [];
    
        $(this.popupHandle).find('.fpSelectedFields').children('li').each(function(){
            tmpFieldList.push($(this).attr('term'));
        });

        // commit to persistence layer
        this.setSelectedFieldList(tmpFieldList);
       
        // notify everyone else
        this.pushSettingsToChildren();
    },
    onContextChange: function() {
        $('.inlineHeader', this.container).hide();
        var context = this.getBaseContext();
        if (context.isResurrected()) {
            this.refreshTotalFieldCount();
        }
        // the field summary layers have a 'show only events with this field' link. 
        // clicking that link changes the underlying search, and will close the popup. 
        // shortly thereafter there will be an onContextChange. 
        // this code here is to detect when the rug is pulled out from under us and 
        // a) commit any changes the user has made so far
        // b) close the popup.
        if (this.popupInstance && this.popupHandle.is(":visible")) {
            this.commitChanges();
            this.popupInstance.destroyPopup();
        }
    },
    
    onJobProgress: function() {
        this.refreshTotalFieldCount();
    },
    refreshTotalFieldCount: function() {
        var context = this.getBaseContext();
        var totalFieldCount = context.job.getEventFieldCount();
        this.updateTotalFieldsNumber(totalFieldCount);
    },
    resetUI: function() {
        $('.inlineHeader', this.container).hide();   
    }
});
