import cherrypy, logging, splunk.search 
import controllers.module as module

logger = logging.getLogger('splunk.modules.fieldpicker')

SUMMARY_ACTIVITY = 'summary'
SUMMARY_HTML_TEMPLATE = 'search/FieldPickerSummary_generate_results.html'
SUMMARY_COUNT_CONSTRAINT = 10

class FieldPicker(module.ModuleHandler):
    
    def generateResults(self, activity=None, **args):
        '''
        FieldPicker Utility Controller
        
        Arg:
        activity: The activity to perform, summary or defaults to None. Note: The action keyword is soaked up higher level by the routing system.
        '''
        if activity==SUMMARY_ACTIVITY:
            return self._renderSingleFieldSummary(**args)
        else:
            return _('Splunk could not retrieve field summary data because the following activity is undefined or invalid: %s') % activity
        
        
    def _renderSingleFieldSummary(self, field_name=None, sid=None, count=None, **args):
        '''
        FieldPicker Summary
        
        Arg:
        field: The field to retrieve summary data from
        sid: The job search id
        count: The summary count constraint, 0 defaults to SUMMARY_COUNT_CONSTRAINT
        '''
        if field_name is None:
            return _('The field name is missing. Splunk could not retrieve field summary data.')
        elif sid is None:
            return _('The job id is missing. Splunk could not retrieve field summary data.')
        else:
            try:
                job = splunk.search.getJob(sid)
            except splunk.ResourceNotFound, e:
                logger.error(e)
                return _('The job appears to have expired or is invalid. Splunk could not retrieve field summary data.<!-- Exception: %s -->') % e
            except Exception, e:
                logger.error(e)
                return _('An error occurred. Splunk could not retrieve field summary data.<!-- Exception: %s -->') % e
            field_summary = job.summary.fields.get(field_name, None)
            if field_summary is None:
                return _('No summary information exists for field_name %s') % field_name
            if count is None:
                count = SUMMARY_COUNT_CONSTRAINT
            else:
                count = min(int(count), SUMMARY_COUNT_CONSTRAINT)
                #0 value delimits everything up to constraint
                if count is 0:
                    count = SUMMARY_COUNT_CONSTRAINT
            template_args = {
                'field_summary':field_summary,
                'count':count
            }
            return self.controller.render_template(SUMMARY_HTML_TEMPLATE, template_args)
