/** THE RESURRECTION SHIP!!!!!!!!!!!!!!!! **/
Splunk.Module.HiddenSavedSearch = $.klass(Splunk.Module, {
    savedSearchContext: null,
    NO_JOB_FOUND_MESSAGE : _('No scheduled job was found for saved search "%(name)s".'),
    NO_JOB_FOUND_WITH_NEXT_RUN_MESSAGE : _('No scheduled job was found for saved search "%(name)s". The next scheduled run for this search is at %(time)s'),
    USE_HISTORY_TRUE_FREE_MODE_MSG: _('This HiddenSavedSearch module is configured with the parameter useHistory set to True. This is not compatible with a free license, try setting useHistory to Auto or False in this view\'s configuration.'),

    initialize: function($super, container) {
        $super(container);
        this.messenger = Splunk.Messenger.System.getInstance();
        if (this._params['jsonContext']) {
            var boundingGroup = this.getGroupName();
            if (boundingGroup) {
                this._params['jsonContext']["group"] = boundingGroup;
            }
            this.savedSearchContext = Splunk.SearchContext.resurrect(this._params['jsonContext']);
            this.savedSearchContext.job.setAsAutoCancellable(false);
        }
        else {
            this.savedSearchContext = null;
        }
    },

    getModifiedContext: function() {
        
        // fail if somehow we got a null context. 
        if (!this.savedSearchContext) {
            // TODO - move these strings to the top of the file.
            this.displayInlineErrorMessage(sprintf(_("Configuration error - we were not able to find a saved search called '%s'."),  this._params["savedSearch"]));
            this.logger.error(this.moduleType + " null context resurrected");
            return null;
        }
        
        // fail if somehow we got a context but its not a saved search. 
        if (!this.savedSearchContext.getSavedSearchName()) {
            this.logger.error(this.moduleType + " context resurrected somehow without saved search name");
        }
        // so far so good but there's a couple error cases to check.
        else {
            
            var baseContext = this.getBaseContext();
            var savedSearchContext = this.savedSearchContext.clone();
            // 1) we were configured with parents that want to alter the search but we have inherited a specific timerange from parent modules.
            if (!baseContext.getTimeRange().isAllTime()) {  
                if (savedSearchContext.isJobDispatched()) {
                    if (savedSearchContext.job.delegate) {
                        // TODO - move these strings to the top of the file.
                        this.displayInlineErrorMessage(_("Configuration error - HiddenSavedSearch has both parent modules like TimeRangePicker, and also a dispatched job from the scheduler."));
                    } else {
                        this.logger.warn(this.moduleType, " is using a saved search that was dispatched by an upstream module but NOT by the scheduler. Abandoning job.")
                    }
                    savedSearchContext.abandonJob();
                } else {
                    this.logger.info(this.moduleType, " is using a saved search and a module above has altered the timerange to ", baseContext.getTimeRange())
                }
                savedSearchContext.setTimeRange(baseContext.getTimeRange());
            }

            // 2) we were configured with parents that want to add intentions. Not supported at all so log an error.
            if (baseContext.hasIntentions() && savedSearchContext.isJobDispatched()) {
            
                this.displayInlineErrorMessage("Configuration not supported - HiddenSavedSearch does not yet support layering additional intentions on top of a saved search.");
            }

            // 3) among the permutations of useHistory and isJobDispatched there are 2 more error conditions we check for.
            if (this._params['useHistory'].toLowerCase() == 'true'  && !savedSearchContext.isJobDispatched()) {
                var args = {name: savedSearchContext.getSavedSearchName()}

                if (window.$C && window.$C.hasOwnProperty('SPLUNKD_FREE_LICENSE') && window.$C['SPLUNKD_FREE_LICENSE']) {
                    var jobNotFoundMessage = this.USE_HISTORY_TRUE_FREE_MODE_MSG;
                } else {
                    var jobNotFoundMessage = sprintf(this.NO_JOB_FOUND_MESSAGE,args);

                    // if we know the next scheduled run, we rebake the message with that info.
                    if (savedSearchContext.next_scheduled_time) {
                        args["time"] = savedSearchContext.next_scheduled_time;
                        jobNotFoundMessage = sprintf(this.NO_JOB_FOUND_WITH_NEXT_RUN_MESSAGE,args);
                    }
                }

                // TODO - move these strings to the top of the file.
                this.displayInlineErrorMessage(jobNotFoundMessage);
                return null;
            }
            return savedSearchContext
        }
        return null;
    }
});
