/* The SearchBar provides a query input area along with a submit button. 
 * In this version, the initial DOM elements are added to the container.
 */
Splunk.Module.SearchBar = $.klass(Splunk.Module.FieldSearch, {

    // define module parameter name
    TYPEAHEAD_PARAM_KEY: 'useTypeahead',
    ASSISTANT_PARAM_KEY: 'useAssistant',
    AUTO_ASSIST_PARAM_KEY: 'autoOpenAssistant',
    SUBMIT_BUTTON_PARAM_KEY: 'useOwnSubmitButton',
    MAX_SEARCH_BAR_HEIGHT_MULTIPLE: 5,   
    
    /*
     * overriding initialize to set up references and event handlers.
     */
    initialize: function($super, container) {
        $super(container);
       
        this.logger = Splunk.Logger.getLogger("SearchBar.js");
        this.fieldName = false;
        this.timer = false;
    
        this.initAssistant();

        this.realBindEventListeners();
        
        //set up resizing
        this.resize = new Splunk.TextareaResize(this.input, { max_lines: this.MAX_SEARCH_BAR_HEIGHT_MULTIPLE });
        
        //   this may be removed, but SearchBar takes a param whereby it implements its own submit button. 
        //   if this is the case, it needs an extra 45px margin-right, which it gets from this classname
        if (Splunk.util.normalizeBoolean(this._params[this.SUBMIT_BUTTON_PARAM_KEY])) {
            this.container.addClass(this.SUBMIT_BUTTON_PARAM_KEY);
        }
    },

    
    ///////////////////////////////////////////////////////////////////////////
    // Search Assistant
    ///////////////////////////////////////////////////////////////////////////
    
    initAssistant: function() {
        this.assistantContainer = $('.assistantContainer', this.container);
        this.assistantEnabled = false;
        this.assistantFillPending = false;
        this.assistantNeedsUpdate = false;
        this.assistantCursor = 0;
        
        if (this.getParam(this.AUTO_ASSIST_PARAM_KEY) == null) {
            this.setParam(this.AUTO_ASSIST_PARAM_KEY, true);
        }
        
        // add toggle as click handler
        $('.assistantAutoOpener', this.container).bind('click',
            function(evt) {
                this.setAssistantAutoOpen(
                    !Splunk.util.normalizeBoolean(this.getParam(this.AUTO_ASSIST_PARAM_KEY))
                );
                return false;
            }.bind(this)
        );
        this.setAssistantAutoOpen();

    },

    setAssistantAutoOpen: function(isEnabled) {
        if (typeof(isEnabled) == 'undefined') {
            isEnabled = Splunk.util.normalizeBoolean(this.getParam(this.AUTO_ASSIST_PARAM_KEY));
        } else {
            this.setParam(this.AUTO_ASSIST_PARAM_KEY, isEnabled);
        }
        this.updateAssistantAutoOpen();
    },
    
    updateAssistantAutoOpen: function() {
        var controlText =  Splunk.util.normalizeBoolean(this.getParam(this.AUTO_ASSIST_PARAM_KEY)) ? _('turn off auto-open') : _('turn on auto-open');
        $('.assistantAutoOpener', this.assistantContainer).text(controlText);
    },
    
    openAssistant: function() {
        if (!Splunk.util.normalizeBoolean(this.getParam(this.ASSISTANT_PARAM_KEY))) {
            return false;
	    }      
        if (this.assistantFillPending) return false;
        this.assistantEnabled = true;
        $('.assTab', this.container).addClass('assistantEnabled');
        this.fillAssistant();
    },
    
    closeAssistant: function() {
        this.assistantContainer.hide();
        this.assistantEnabled = false;
        this.assistantFillPending = false;
        $('.assTab', this.container).removeClass('assistantEnabled');
        
    },
    
    toggleAssistant: function() {
        if (this.assistantEnabled) this.closeAssistant();
        else this.openAssistant();
        return false;
    },
     
    fillAssistant: function() {
        if (!this.assistantEnabled) return false;
        if (this.assistantFillPending) {
            this.assistantNeedsUpdate = true;
            return false;
        }

        var searchString = this._getUserEnteredSearch();
        var namespace    = Splunk.util.getCurrentApp();
        this.assistantContainer.load(
            Splunk.util.make_url('/api/shelper') + '?' +
            $.param({
                'snippet': 'true',
                'namespace': namespace,
                'search': searchString,
                'useTypeahead': this.getParam(this.TYPEAHEAD_PARAM_KEY),
                'useAssistant': this.getParam(this.ASSISTANT_PARAM_KEY)
            }),
            this.fillAssistantCallback.bind(this)
        );
        this.assistantFillPending = true;
        this.assistantCursor = -1;
        
        return true;
    },
    fillAssistantCallback: function() {
        this.assistantFillPending = false;
        if (!this.assistantEnabled) {
            this.assistantNeedsUpdate = false;
            return false;
        }
        this.assistantContainer.width(this.input.width()).show().bgiframe();
        if (this.assistantNeedsUpdate) {
            this.assistantNeedsUpdate = false;
            this.fillAssistant();
        }
        this.assistantKeywordCount = $('.saKeywords li.sakeyword', this.assistantContainer).length;
        this.updateAssistantAutoOpen();
    },
    
    selectNextKeyword: function() {
        if (this.assistantCursor >= (this.assistantKeywordCount-1)) {
            this.assistantCursor = this.assistantKeywordCount - 1;
        } else {
            this.assistantCursor += 1;
            this._highlightKeyword(this.assistantCursor);
        }
    },
    
    selectPreviousKeyword: function() {
        if (this.assistantCursor < 0) {
            this.assistantCursor = -1;
        } else {
            this.assistantCursor -= 1;
            this._highlightKeyword(this.assistantCursor);
        }
    },
    
    useSelectedKeyword: function(updateContents) {
        if (this.assistantCursor < 0) return false;
        var value = $('.saKeywords li.sakeyword a', this.assistantContainer
            ).slice(this.assistantCursor, this.assistantCursor + 1
            ).attr('replacement');
        this.setSearchTerm($.trim(value) + ' ');
        if (updateContents)
            this.fillAssistant();
    },
    
    _highlightKeyword: function(keywordPosition) {
        
        // set the CSS style for selected
        var el = $('.saKeywords li.sakeyword', this.assistantContainer
            ).removeClass('keywordSelected'
            ).slice(keywordPosition, keywordPosition + 1
            ).addClass('keywordSelected');
        
        // keep selected item in view
        var win = $('.saTypeaheadWrapper ul', this.assistantContainer);
        var visibleWindowTop = win.scrollTop();
        var visibleWindowBottom = win.scrollTop() + win.height();
        var elementTop = el.height() * keywordPosition;
        if (elementTop < visibleWindowTop) {
            win.scrollTop(elementTop);
        } else if (elementTop + el.height() > visibleWindowBottom) {
            win.scrollTop(elementTop + el.height() - win.height());
        }
        
    },

	
    ///////////////////////////////////////////////////////////////////////////
    // Search context handling
    ///////////////////////////////////////////////////////////////////////////
	

    applyContext: function($super,context) {
        
        var stopPropagation = false;
        // When intentions are passed up on contexts,  from clicks in SimpleEventsViewer
        // or clicks in FieldViewer,   they will always have 'addterm'.
        // Depending on the click preference we pick up (TBD),
        // this module will either  - replace addterm with toggleterm
        // or blow away the searchStr and leave the addterm unchanged.
        if (!context.isResurrected()) {
            if ((context._intentions.length == 1)  && (context._intentions[0]["name"] == "addterm")) {
                context._intentions[0]["name"] = "toggleterm";
            }
            var thisSearchStr = Splunk.util.trim(this._getUserEnteredSearch()) || "*";
            context.setBaseSearch(thisSearchStr);
            stopPropagation = true;
        }
        
        var contextWasResurrected = context.isResurrected();
        context.absorbIntentions(function(newQ) {
            this.baseContext = null;
            this.setInputField(Splunk.util.stripLeadingSearchCommand(newQ));
            // if this wasnt a resurrected context, then we'll need to do a push
            if (!contextWasResurrected) {
                this.pushContextToChildren();
            }
        }.bind(this));
        // potentially return true so we can stop the context from propagating upward any further.
        return stopPropagation;
    },
    isEmpty: function() {
        return ($.trim(this.input.attr('value')).length == 0);
    },

    /**
     * Returns a normalized search string by removing leading whitespace and
     * trimming all trailing whitespace to a single space; also send through
     * the 'search' command cleaner
     */
    _getUserEnteredSearch: function() {
        var q = this.input.attr('value') || '*';
        q = Splunk.util.addLeadingSearchCommand(q, true);
        return q;
    },
    
    /**
     * Necessitated by the need to resurrect jobs, sometimes search bars
     * may need to have their terms explicitly set.
     */
    setSearchTerm: function(terms) {
        this.input.attr('value', terms);
    },
    /**
     * When SearchBar receives a new context from above, 
     * it flattens the intentions and writes the result to 
     * it's input.  
     * NOTE: if there are no intentions the callback will just 
     * execute instantly.  As a practical matter that will 
     * probably be the case, but this way we also cover cases 
     * where SearchBar might have some modules like FieldSearch 
     * above it, and for some reason the developer wants SearchBar
     * to display the full search string for the user. 
     */
    onContextChange: function() {
        var context = this.getBaseContext().clone();
        context.absorbIntentions(function(newQ) {
            this.setInputField(Splunk.util.stripLeadingSearchCommand(newQ));
        }.bind(this));
    },
    /*
     *  Note:  Although getModifiedContext often just adds an intention,
     * sometimes it may also replace the Job instance.
     */
    getModifiedContext: function() {
        // pass through if the job has a search id
        var context = this.getBaseContext();
       
        if (!this.isEmpty()) {   
            var searchTermStr = this._getUserEnteredSearch();
            this.logger.info('getModifiedContext - got search=', searchTermStr);
            if (this.baseContext) this.logger.warn(this.moduleType, "WARNING: clobbering base search.");
            context.abandonJob();
            context.setBaseSearch(searchTermStr);
        }
        return context;
    },
    
    
    
    ///////////////////////////////////////////////////////////////////////////
    // User event handling
    ///////////////////////////////////////////////////////////////////////////
    
    /**
     * null out the superclass invocation; its load order messes up this class
     */
    bindEventListeners: function() {
        this.searchForm.submit(this._onFormSubmit.bind(this));
    },
    
    realBindEventListeners: function() {
        this.input = $('textarea', this.container);
        // provide a fallback for unit tests.
        if (this.input.length==0) {
            this.input = $('<textarea>').appendTo(this.container);
        }
        
        // bind key events.; 
        this.input.bind("keyup", this.onCanonicalKeyUp.bind(this));
        if ($.browser.msie) {
            this.input.bind("keydown", this.onCanonicalKeyPress.bind(this));
            this.input.bind("keypress", this.onErrantIEKeyPress.bind(this));
        } else if ($.browser.safari) {
            this.input.bind("keydown", this.onCanonicalKeyPress.bind(this));
        } else {
            this.input.bind("keypress", this.onCanonicalKeyPress.bind(this));
        }

        // setup the assistant layer
        if (Splunk.util.normalizeBoolean(this.getParam(this.ASSISTANT_PARAM_KEY))) {
            
            $('div.assistantWrapper', this.container).show();
            $('a.assistantActivator', this.container).css('display','block');
            $('div.searchFieldWrapper', this.container).addClass('hasAssistant');
            $('.assistantActivator', this.container).bind('click', this.toggleAssistant.bind(this));

            

            // attach event handler to catch search box modifying requests that
            // come from the search assistant
            this.assistantContainer.bind('click', 
                function(evt) {
                    var target = evt.target;
                    
                    if ($(target).hasClass('assistantAutoOpener')) {
                        this.setAssistantAutoOpen(
                            !Splunk.util.normalizeBoolean(this.getParam(this.AUTO_ASSIST_PARAM_KEY))
                        );
                    }
		    if ( $(target).hasClass('salink') ) {
                        return true;   
                    }                    
                    // walk up tree for a bit
                    for (var i=0; i<4; i++) {
                        if (target.tagName != 'A'){ 
                            target = target.parentNode;
                        } else {
                            break;
                        }
                    }
                    
                    if (target.tagName != 'A') {
                        evt.stopImmediatePropagation();
                        return false;
                    }

                    var searchString = $(target).attr('replacement');
                    if (searchString) {
                        this.setSearchTerm(searchString);
                        this.fillAssistant();
                    }
                    return false;
                }.bind(this)
            );
            
            // catch all 
            $(document).bind('keyup', 
                function(evt){ 
                    if (evt.keyCode == this.KEY_ESCAPE) this.closeAssistant(); 
                }.bind(this)
            );
            $(this.input).bind('click', function(evt) { evt.stopPropagation(); return true;});
            $(document).bind('click', this.closeAssistant.bind(this));
        }

    },


    _onFormSubmit: function($super, event) {
        // don't do anything if there's nothing in the search box
        if (this.isEmpty()) {
            return false;
        }
        
        this.baseContext = null;
        return $super(event);
    },
    
    
    onCanonicalKeyPress: function(evt) {
        switch (evt.keyCode) {
            case this.KEY_DOWN_ARROW:
                return this.onDownArrow(evt);
            case this.KEY_UP_ARROW:
                return this.onUpArrow(evt);
            case this.KEY_TAB:
                return this.onTab(evt);
            case this.KEY_ESCAPE:
                return this.onEscape(evt);
            case this.KEY_ENTER:
                return this.onEnter(evt);
        }

    },
    
    /**
     * This is more IE loveliness.  Most of the time, IE will fire key events
     * in the correct order: keyDown -> keyPress -> keyUp.  However, it 
     * frequently forgets to jiggle the handle, and neglects to fire keyDown,
     * instead preferring to fire keyPress first (sometimes without keyDown
     * or keyUp).  This listener anticipates our retarded cousin and ensures
     * that hitting enter will do the right thing and not insert random
     * carriage returns before a search submit.
     */
    onErrantIEKeyPress: function(evt) {
        if (evt.keyCode == this.KEY_ENTER) {
            //ctrl-enter|shift-enter adds line to the textarea
            if (evt.ctrlKey || evt.shiftKey) {
                return true;
            }
            return false;
        }
    },

    onCanonicalKeyUp: function(evt) {
        switch (evt.keyCode) {
            case this.KEY_ESCAPE:
                return this.onEscape(evt);

            // is handled by onCanonicalKeyPress
            case this.KEY_DOWN_ARROW:
            case this.KEY_UP_ARROW:
            case this.KEY_TAB:
                return false;
            case this.KEY_ENTER:
                return false;
            case this.KEY_LEFT_ARROW:
                return false;
            case this.KEY_RIGHT_ARROW:
                return false;

        }
        
        if (!this.assistantEnabled) {
            if (Splunk.util.normalizeBoolean(this.getParam(this.AUTO_ASSIST_PARAM_KEY))) {
                this.openAssistant();
            }
        } else {
            this.fillAssistant();
        }
    },    
    
    onEnter: function(evt) {
        
        //ctrl-enter|shift-enter adds line to the textarea
        if (evt.ctrlKey || evt.shiftKey) {
            return true;
        }

        if (this.assistantEnabled) {
            this.useSelectedKeyword();
            this.closeAssistant();
        }

        this._onFormSubmit();
        
        return false;
    },
    
    onUpArrow: function(evt) {
        if (this.assistantEnabled) {
            this.selectPreviousKeyword();
            return false;
        }
    },
    
    onDownArrow: function(evt) {
        if (this.assistantEnabled) {
            this.selectNextKeyword();
            return false;
        }
    },
    
    onTab: function(evt) {
        if (this.assistantEnabled) {
            this.useSelectedKeyword(true);
            return false;
        }
    },
    
    onEscape: function(evt) {
        this.closeAssistant();
        return false;
    }
});
