Splunk.Module.SubmitButton = $.klass(Splunk.Module, {
    /*
     * overriding initialize to set up references and event handlers.
     */
    initialize: function($super, container) {
        $super(container);
        this.logger = Splunk.Logger.getLogger("SubmitButton.js")
        this._previouslyPushedSettings = {};
        this._previouslyPushedContext  = false;
        
        if (this._params.hasOwnProperty("label")) {
            this._submitButton = $("button.splButton-primary", this.container);
        } else {
            this._submitButton = $("input.searchButton", this.container);
        }
        this._submitButton.click(this.onClick.bind(this));
    },

    pushContextToChildren: function($super, explicitContext) {
        //SubmitButton's magic, incantation #1 - unless the context is resurrected, ignore all requests.
        // instead the button click will pass SearchContext changes down.

        var context = explicitContext || this.getModifiedContext();
        
        if (context.isResurrected() || Splunk.util.normalizeBoolean(this._params["allowSoftSubmit"])) {
            $super(explicitContext);
        }
    },

    pushSettingsToChildren: function($super, settingsMap, pushSilently, simonSays) {
        //this.logger.debug(this.moduleType , " pushing settings " , pushSilently, simonSays);
        if (simonSays || pushSilently) $super(settingsMap, pushSilently);
    },
    onSettingsChange: function() {
        var newBaseSettings = this.getBaseSettings()
        if (!Splunk.util.objectSimilarity(this._previouslyPushedSettings, newBaseSettings)) {
            this._submitButton.removeClass("greyedOut");
        }
        this._previouslyPushedSettings = newBaseSettings;
    },
    onContextChange: function() {
        var newContext = this.getBaseContext();
        if (newContext.isResurrected() || !this._previouslyPushedContext ||  !newContext.equalToContext(this._previouslyPushedContext)) {
            this._submitButton.removeClass("greyedOut");
        }
        this.onSettingsChange(this.getBaseSettings());
    },
    applyContext: function($super,context) {
        this._previouslyPushedContext = context;
        this._submitButton.removeClass("greyedOut");
        return $super(context);
    },

    onClick : function(event) {
        if (Splunk.util.normalizeBoolean(this._params["allowSoftSubmit"])) {
            this.baseContext = null;
            var ancestors = this.getAncestors();
            for (var i=0, len=ancestors.length; i<len; i++) {
                ancestors[i].baseContext = null;
            }
        // the only time we can safely add and remove the greyed out class, is if the submit 
        // button click is the sole way for changes to go through.  In other words, !allowSoftSubmit
        } else {
            this._submitButton.addClass("greyedOut");
        }

        var context = this.getBaseContext();
        // TODO - this settings push, followed by the full context push,  is a weirdness 
        //        that is needed for JobManager module. 
        this.pushSettingsToChildren(this.getBaseSettings(), false, true);
        
        this._previouslyPushedContext = context;
        
        // SubmitButton's magic, incantation #2 - this looks a lot like some of the guts
        //                       of Module.pushContextToChildren
        var pushHandler = function(dispatchedContext) {
            for (var i=0; i<this._children.length; i++) {
                this._children[i].baseContext = dispatchedContext;
                this._children[i].onContextChange();
                this._children[i].pushContextToChildren();
            }
        }.bind(this);
        
        if (!context.isJobDispatched() && this.someChildrenRequireDispatch(context)) {
                
            var descendants = this.getDescendants();
            for (var i=0, len=descendants.length; i<len; i++) {
                descendants[i].onBeforeJobDispatched(context);
                if (descendants[i].baseContext &&
                    descendants[i].baseContext.isJobDispatched() &&
                    descendants[i].baseContext.job.canBeAutoCancelled()) {
                        descendants[i].baseContext.job.cancel();
                }
            }

            var failHandler = function() {
                alert('alack the failHandler of Viscount ' + this.moduleType + ' hath been called');
            }.bind(this);

            // SubmitButton's magic, incantation #3 -
            // except that instead of using Module._fireDispatch, we use the
            // dispatchJob machinery directly.
            context.dispatchJob(pushHandler, failHandler, this.getGroupName());
        } else {
            pushHandler(context);
        }
    },
    onCancelClick: function(event) {
        // SearchContext resurrection will be fairly easy and will just involve some refactoring in ModuleLoader.startResurrection.
        // however we're still waiting on view persistence architecture, so this wouldnt work to restore
        // initial settings UI.   hopefully when settings resurrection is supported it will
        // reuse the existing resurrection code path, this idea will just work.
        // var context = <getJSONContextHere>
        //Splunk.Globals["ModuleLoader"].startResurrection(context, [this]);

        // until then have a message.
        var messenger = Splunk.Messenger.System.getInstance();
        messenger.send('info', 'splunk.search', _("For the time being 'cancel' will not be able to restore settings until settings are resurrectable."));
    }
});
