//put Module in the namespace if it isnt already there.
/*
 *  TODO:  - add validation for custom date inputs (end date after start date, times correct, etc.)
 *         - improve time input interaction (add up/down arrow key scrolling to increment times, restrict input)
 *         - clean up minor IE6 visual bugs
 *
*/
Splunk.namespace("Module");

Splunk.Module.TimeRangePicker = $.klass(Splunk.Module, {

    EARLIEST_TIME_ATTR: 'earliest_time',
    LATEST_TIME_ATTR: 'latest_time',
    ALL_TIME_LABEL: _('All time'),
    DATEPICKER_CLASS: 'customDateTime',
    DATE_RANGE_TEXT: _('Date range'),
    DATE_RANGE_CLASS: 'dateRange',
    DATES_BEFORE_TEXT: _('All dates before'),
    DATES_BEFORE_CLASS: 'datesBefore',
    DATES_AFTER_TEXT: _('All dates after'),
    DATES_AFTER_CLASS: 'datesAfter',
    SPECIFIC_DATE_TEXT: _('Specific date'),
    SPECIFIC_DATE_CLASS: 'specificDate',

	// define sequence order among user-defined times before where to insert
	// the custom controls
	CUSTOM_DATE_SEAM: 500,

    _selectedRange: null,
    _menu: null,
    _activator: null,
    
    // storage for the Date() instances we made from the dates the user picked. 
    // since these are in the browser's timezone, they are DEEPY UNTRUSTWORTHY
    _customEarliestDate: null,
    _customLatestDate: null,

    _datePickerMode: false,
    
    
    /*
     * overriding initialize to set up references and event handlers.
     */
    initialize: function($super, container) {
        $super(container);
        this.logger = Splunk.Logger.getLogger("time_range_picker.js");
       
        this._activator = $('.timeRangeActivator', this.container)

        var FALLBACK_MENU_DICT = [{"style": "timeRangePreset", "attrs": {}, "label": _("Error: Failed to load configured time ranges.")}];
        
        // read in time ranges from mako-generated template
        try {
            this._menuDict = this._params.timeRangeJson;
        } catch(e) {
            this.logger.error(e + ", " + e.message);
        }
        if (!this._menuDict) {
            this._menuDict = FALLBACK_MENU_DICT;
        }
        /*
        if (Splunk.TimeRange) {
            for (var i=0,L=this._menuDict.length; i<L; i++) {
                var item = this._menuDict[i];
		        var entry = {
                    earliest: item["attrs"]["earliest_time"],
                    latest: item["attrs"]["latest_time"],
                    label: item["label"]
                }
                Splunk.TimeRange.customTimes.push(entry);
            }
        }
        */
        // create custom date items and add them to the timerange menu
        
        var customDates = [
			[this.DATE_RANGE_TEXT, this.DATE_RANGE_CLASS],
        	[this.DATES_BEFORE_TEXT, this.DATES_BEFORE_CLASS],
        	[this.DATES_AFTER_TEXT, this.DATES_AFTER_CLASS],
        	[this.SPECIFIC_DATE_TEXT, this.SPECIFIC_DATE_CLASS]
		];

		// insert the custom pickers right before the custom seam index of
		// all of the time ranges defined in times.conf
		for (var i=0,L=this._menuDict.length; i<L; i++) {
			if (this._menuDict[i]['order'] >= this.CUSTOM_DATE_SEAM || i >= L-1) {		
				for (var j=0,M=customDates.length; j<M; j++) {
					var cust = this._buildCustomDateItem(customDates[j][0], customDates[j][1]);
					if (i >= L-1 && this._menuDict[i]['order'] < this.CUSTOM_DATE_SEAM)
    					this._menuDict.push(cust);
    				else
					    this._menuDict.splice(i, 0, cust);
				}
				break;
			}
		}
	
        
        // build menu
        this._menu = new Splunk.MenuBuilder({
            containerDiv: this.container,
            menuDict: this._menuDict,
            activator: this._activator.parent('.timeRangeActivatorWrapper'),
            menuClasses: 'splMenu-primary timeRangeMenu'
        });
       
        var selectedLabel = this._params['selected'];
        var menuReference = this._menu.getMenu();
        menuReference.click(this._onMenuClick.bind(this));
        
        var matchedSomePreset = false;
        if (selectedLabel) {
            var moduleInstance = this;
            menuReference
                .find('.timeRangePreset a').each(function(){
                    //var thisText = $(this).text().toLowerCase();
                    var thisText = moduleInstance._getRawLabel($(this).text()).toLowerCase();
                    if (thisText == selectedLabel.toLowerCase()) {
                        matchedSomePreset = true;
                        moduleInstance._activator.text($(this).text());
                        moduleInstance._setSelectedRangeToPreset($(this));
                    }
                });
        }
        // we didnt match anything
        if (!matchedSomePreset) {
            // but we expected to...
            if (selectedLabel) {
                this.logger.error(this.moduleType, " specified a selected timeRange in its config, (", selectedLabel, ") but that is not a valid option");
                this._activator.text(this.ALL_TIME_LABEL);
            } else {
                this._activator.text(this.ALL_TIME_LABEL);
            }
        } 

       // for each custom date item, do setup
        for (var i=0,L=customDates.length; i<L; i++) {
           this._setupCustomDate(customDates[i][0], customDates[i][1]);
        }   
    },

    /**
     * Returns the built-in english version of the translated time string
     */
    _getRawLabel: function(translatedTime) {

        for (var key in this._menuDict) {
            if (this._menuDict.hasOwnProperty(key)) {

                // if this item is a submenu, we go inside and iterate over all it's items.
                if (this._menuDict[key].hasOwnProperty("items")) {
                    var subMenu = this._menuDict[key]["items"];
                    for (var subKey in subMenu) {
                        if (translatedTime == subMenu[subKey]['label']) {
                            return subMenu[subKey]['rawLabel'];
                        }
                    }
                }
                // otherwise this is a plain old timeRangePreset, and we check for a match.
                else {
                    if (translatedTime == this._menuDict[key]['label']) {
                        return this._menuDict[key]['rawLabel'];
                    }
                }
            }
        }

        return '';
    },
    
    
    /* build json struct for a custom datepicker item */
    _buildCustomDateItem: function(label, blockClass) {
        var parentClass = 'customDateItem ' + blockClass + 'Parent';
        var customDateItem = {
                "label" : label,
                "style" : parentClass,
                "items" : [{
                    "menuType" : "htmlBlock",
                    "style" : blockClass,
                    "element" : $('.' + this.DATEPICKER_CLASS, this.container),
                    "action" : "clone"
                }]
        };

        return customDateItem;
    },
    getResults: function($super) {
        // make sure we dont silently pass back previous absolute time args in *any* error conditions. 
        // I'd rather hit the assertion in getModifiedContext
        this._selectedRange = null;
        return $super()
    },

    renderResults: function(xmlDoc) {
        responseDict = JSON.parse(xmlDoc.toString());
        var earliestEpochTime    = responseDict["earliestEpochTime"] || null;
        var latestEpochTime      = responseDict["latestEpochTime"] || null;
        var earliestOffset = null;
        var latestOffset   = null;
        if (responseDict["earliestOffset"]) {
            earliestOffset       = parseInt(responseDict["earliestOffset"][0], 10);
        }
        if (responseDict["latestOffset"]) {
            latestOffset         = parseInt(responseDict["latestOffset"][0], 10);
        }
        this._selectedRange = new Splunk.TimeRange(earliestEpochTime, latestEpochTime, earliestOffset, latestOffset);
        $('body').trigger('menuOpening');
        if (this._params["searchWhenChanged"].toLowerCase() != "false") {    
            this.baseContext = null;
            this.pushContextToChildren();
        }
    },
    getResultParams: function() {
        var f = Splunk.util.getConfigValue("SEARCH_RESULTS_TIME_FORMAT")
        var params = {};
        if (this._customEarliestDate) params["earliest"] = this._customEarliestDate.strftime(f);
        if (this._customLatestDate)   params["latest"]   = this._customLatestDate.strftime(f);

        return params;
    },
    /* set up custom date blocks, handle apply click events */
    _setupCustomDate: function(datepickerText, datepickerContainerClass) {
        
        var datepickerContainer = $('.' + datepickerContainerClass, this.container);
  
        var clickFn = function(toDate, fromDate) {
            this._datePickerMode = true;
            this._customEarliestDate = fromDate;
            this._customLatestDate = toDate;
            
            this._setActivatorText(toDate, fromDate, datepickerContainerClass);
            this.getResults();
            
        }.bind(this);

        var getDateWithTime = function(datepicker, timefield) {
            dateWithTime = $(datepicker, datepickerContainer).datepicker('getDate');
            t = $(timefield, datepickerContainer).val();

            var timeRE = /(\d+):(\d+):(\d+)\.?(\d*)/; // 1 or more numbers, a colon, one or more numbers, a colon, one or more numbers, an optional period followed by 0 or more numbers

            var timeArray = timeRE.exec(t);

            for ( i = 1; i < 4; i++ ) {
                if ( timeArray[i] == 'undefined' ) {
                    timeArray[i] = 0;
                }
            }
    
            dateWithTime.setHours(timeArray[1], timeArray[2], timeArray[3], timeArray[4]);

            return dateWithTime;
        };
        
        //set header text within each datepicker block
        if ( datepickerContainerClass == this.DATE_RANGE_CLASS ) {
            datepickerContainer.find('.customDateTimeHeader').text(_(this.DATE_RANGE_TEXT));
            datepickerContainer.find('.trpApply').click(function(evt){
                toDate = getDateWithTime('.dateRangeTo', '.dateRangeToTimeField');
                fromDate = getDateWithTime('.dateRangeFrom', '.dateRangeFromTimeField');

                clickFn(toDate, fromDate, datepickerContainerClass);
            });

            // to setup the date picker to work in the modules.html test, we need to 
            // either rethink how we the generic modules get tested, or provide a 
            // mechanism for the initial html to be set correctly during generic testing
            if (!Splunk._testHarnessMode) {
                this._setupDatePicker(datepickerContainerClass, '.dateRangeFrom', '.dateRangeFromAltField');
                this._setupDatePicker(datepickerContainerClass, '.dateRangeTo', '.dateRangeToAltField');
            }
        } 
        if ( datepickerContainerClass == this.DATES_BEFORE_CLASS ) {
            datepickerContainer.find('.customDateTimeHeader').text(_(this.DATES_BEFORE_TEXT));
            datepickerContainer.find('.trpApply').click(function(evt){
                toDate = getDateWithTime('.dateRangeTo', '.dateRangeToTimeField');
                fromDate = null;

                clickFn(toDate, fromDate, datepickerContainerClass);
            });

            if (!Splunk._testHarnessMode) {
                this._setupDatePicker(datepickerContainerClass, '.dateRangeTo', '.dateRangeToAltField');
            }
        } else if ( datepickerContainerClass == this.DATES_AFTER_CLASS ) {
            datepickerContainer.find('.customDateTimeHeader').text(_(this.DATES_AFTER_TEXT));
            datepickerContainer.find('.trpApply').click(function(evt){
                toDate = null;
                fromDate = getDateWithTime('.dateRangeFrom', '.dateRangeFromTimeField');

                clickFn(toDate, fromDate, datepickerContainerClass);
            });
            if (!Splunk._testHarnessMode) {
                this._setupDatePicker(datepickerContainerClass, '.dateRangeFrom', '.dateRangeFromAltField');
            }
        } else if ( datepickerContainerClass == this.SPECIFIC_DATE_CLASS ) {
            datepickerContainer.find('.customDateTimeHeader').text(_(this.SPECIFIC_DATE_TEXT));
            datepickerContainer.find('.trpApply').click(function(evt){
                toDate = getDateWithTime('.dateRangeFrom', '.dateRangeToTimeField');
                fromDate = getDateWithTime('.dateRangeFrom', '.dateRangeFromTimeField');

                clickFn(toDate, fromDate, datepickerContainerClass);
            });
            if (!Splunk._testHarnessMode) {
                this._setupDatePicker(datepickerContainerClass, '.dateRangeFrom', '.dateRangeFromAltField, .dateRangeToAltField');
            }
        }
    },
    /* function to set up and initialize a datepicker */
    _setupDatePicker: function(datepickerContainerClass, datepickerSelector, datepickerAltField){
        var $p = $('.' + datepickerContainerClass, this.container);
      
		$(datepickerSelector, $p).datepicker({
			changeMonth: false,
			changeYear: false,
			currentText: '',
			prevText: '',
			nextText: '',
			altField: $(datepickerAltField, $p),
			altFormat: 'M dd, yy',
            defaultDate: -7
		});

        var altDate = $(datepickerSelector, $p).datepicker('getDate');
        altDate = $.datepicker.formatDate('M dd, yy', altDate);
        $(datepickerAltField, $p).val(altDate);
    },
    /* function to display a short version of the new custom date range in the menu activator */
    _setActivatorText: function(toDate, fromDate, datepickerContainerClass) {
        var newText = 'Custom';

        if ( fromDate instanceof Date ) {
            f = format_date(fromDate, 'medium');
        }
        
        if ( toDate instanceof Date ) {
            t = format_date(toDate, 'medium');
        }

        if ( datepickerContainerClass == this.DATE_RANGE_CLASS ) {
           //date range, do MMM DD, YYYY - MMM DD, YYYY    
           newText = f + " - " + t;
        } else if ( datepickerContainerClass == this.DATES_BEFORE_CLASS ) {
           // all dates before, do All dates before MMM DD, YYYY
           newText = this.DATES_BEFORE_TEXT + " " + t;
        } else if ( datepickerContainerClass == this.DATES_AFTER_CLASS ) {
           // all dates after, do All dates after MMM DD, YYYY
           newText = this.DATES_AFTER_TEXT + " " + f;
        } else if ( datepickerContainerClass == this.SPECIFIC_DATE_CLASS ) {
           //specific date, do MMM DD, YYYY
           newText = f;
        }

        this._activator.text(newText); 
    },
    /* 
    We override getModifiedContext, because this class implements a 'selection' state. In this case
    it implements a "Selection Time" behaviour.  Children will display data from the selected time range.
    */
    getModifiedContext: function() {
        var range = null;
        if (this._selectedRange) {
            range = this._selectedRange;
        } else {
            this.logger.error("Assertion Failed - we have no selected range.  If this occurs with the calendar-pickers its a possible race condition.");
            range = new Splunk.TimeRange();
        }
        var context = this.getBaseContext();
        context.abandonJob();

        this.logger.debug("drunkenmaster TRP.getModifiedContext " + context.getSearch());
        context.setTimeRange(range);
        return context;
    },
    /* 
     * Examines the context's TimeRange and sets its own UI to duplicate that TimeRange
     * If it cannot find a match, it creates an <option> element and blindly puts the
     * earliest and latest args of the TimeRange instance into its values,
     * and uses TimeRange.toConciseString() to output the label.
     */
    applyContext: function(context) {
        var range = context.getTimeRange();
        

        var RECONCILE_TIME_INTENTIONS = false;
        if (RECONCILE_TIME_INTENTIONS) {
            var timeIntentionToString = function(context, intentionName) {
                var intention = context.getIntentionReference(intentionName);
                if (intention) {
                    var arg = intention["arg"];
                    return Splunk.TimeRange.relativeArgsToString(arg["count"], arg["unit"], arg["snapUnit"]);
                }
                return false;
            }
            var earliestIntentionTerm = timeIntentionToString(context, "earliesttime");
            var latestIntentionTerm   = timeIntentionToString(context, "latesttime");

            var needToRenormalize = earliestIntentionTerm || latestIntentionTerm;
            if (needToRenormalize) {
                var earliestArg = earliestIntentionTerm || range.getEarliestTimeTerms();
                var latestArg   = latestIntentionTerm   || range.getLatestTimeTerms();
                range = new Splunk.TimeRange(earliestArg, latestArg);
            }
        }
        var earliest = range.getEarliestTimeTerms() || null;
        var latest   = range.getLatestTimeTerms()   || null;
        
        
        if (context.isResurrected()) {
            if (earliest || latest) {
                // walk through all our existing options and if there's a match, select it.
                var moduleInstance = this;
                var foundAMatch = false;
                var self = this;
                this._menu.getMenu().find('.timeRangePreset a').each(function(){
                    var thisEarliest = $(this).attr(self.EARLIEST_TIME_ATTR) || null;
                    var thisLatest   = $(this).attr(self.LATEST_TIME_ATTR)   || null;
                    var thisRange = new Splunk.TimeRange(thisEarliest, thisLatest);
                    
                    if (range.equalToRange(thisRange)) {
                        moduleInstance._activator.text($(this).text())
                        moduleInstance._datePickerMode = false;
                        moduleInstance._selectedRange = range;
                        foundAMatch = true;
                        // this return is within an each(), so it's more like a break.
                        return true;
                    } 
                });
                if (foundAMatch) {
                    return;
                }
                else {
                    // create a new OPTION element, insert it, and select it
                    this._insertNewMenuItem(range);
                    return;
                }
            }
            this._selectedRange = new Splunk.TimeRange();
            this._activator.text(this.ALL_TIME_LABEL);
        } else if (range && !range.isAllTime()) {
            this._datePickerMode = false;
            this._insertNewMenuItem(range);
            this.baseContext = null;
            this.pushContextToChildren();
            // return true to stop the context from propagating upward any further.
            return true;
        }
        return false
    },
    _insertNewMenuItem: function(timeRange) {
        var newOption = {};
        var link = $("<a>")
            .attr(this.EARLIEST_TIME_ATTR, timeRange.getEarliestTimeTerms())
            .attr(this.LATEST_TIME_ATTR, timeRange.getLatestTimeTerms())
            .text(timeRange.toConciseString())
        
        $("<li>")
            .addClass("timeRangePreset")
            .append(link)
        .appendTo($(".innerMenuWrapper", this.container)[0]);
            
        this._selectedRange = timeRange;
        
        if (timeRange.getDuration() == -1) {
            this._activator.text(timeRange.toConciseString());
        } else {
            this._activator.text(_("Custom"));
        }
    },
    _onMenuClick: function(evt) {
        var t = evt.target;
       
        if ( $(t).parent('li').hasClass('timeRangePreset') ) {
            this._setSelectedRangeToPreset($(t));
            this.setParam('selected', this._getRawLabel($(t).text()));
            if (this._params["searchWhenChanged"].toLowerCase() != "false") {    
                this.baseContext = null;
                this.pushContextToChildren();
            }
            $('.timeRangeActivator', this.container).text($(t).text());
        }
    },
   
    /*
     * Returns an array of searchterms representing the selected timerange option
     * in the pulldown.
     */
    _setSelectedRangeToPreset: function(presetElement) {
        var earliest = presetElement.attr(this.EARLIEST_TIME_ATTR);
        var latest   = presetElement.attr(this.LATEST_TIME_ATTR);
        this._selectedRange = new Splunk.TimeRange(earliest, latest);
    }
});
