//put Module in the namespace if it isnt already there.
Splunk.namespace("Module");

Splunk.Module.ViewRedirector = $.klass(Splunk.Module, {
    initialize: function($super, container){
        $super(container);
        // kinda weird, but it can be "True", or "False", or any other string. 
        // if it's a random string we basically will pass it straight through to window.open.
        // Goal was to not have more than one config param, and avoid config params that are only applicable if 
        // other config params have certain values..
        this.popupWindowFeatures = false;
        try {
            this.openInPopup = Splunk.util.normalizeBoolean(this._params["popup"], true);
        } catch(e) {
            this.openInPopup = true;
            this.popupWindowFeatures = this._params["popup"];
        }
    },
    addChild: function($super, child) {
        this.logger.error(this.moduleType, 'module is configured with a child module. There is no reason to do this and so it is probably a mistake.');
        return $super(child);
    },
    /*
     * The most natural way to do this is to override onContextChange.
     * However that wouldnt prevent the module's descendants from getting
     * updates, and dispatching searches while the user is leaving the page.
     * Therefore, instead of pushing to children, we redirect the user
     * to the specified view.
     */
    pushContextToChildren: function($super, explicitContext) {
        var context = explicitContext || this.getBaseContext();
        if (context.isResurrected()) {
            return false;
        }
        var args = {};
        var qsDict = Splunk.util.queryStringToProp(document.location.search);
            
        if (Splunk.util.normalizeBoolean(this._params["sendBaseSID"])) {
            // second time through we'll have a base_sid which is the event search
            // and the regular sid, which we resurrected the modules from.
            if (qsDict.hasOwnProperty("base_sid")) {
                args["base_sid"] = qsDict["base_sid"];
            }
            else if (qsDict.hasOwnProperty("sid")) {
                args["base_sid"] = qsDict["sid"];
            }
            // first time through we use the regular sid we linked in on.
            else if (context.isJobDispatched()) {
                args["base_sid"] = context.job.getSearchId();
            } else {
                this.logger.error(this.moduleType, " is configured to send the underlying sid, but its context hasnt been dispatched and there are no sids in the URL");
            }
        }
        if (context.getViewStateId()) {
            args["vs"] = context.getViewStateId();
        }
        this.sendToView(args);
    },
    /**
     * largely delegates to SearchContext.sendToView(). 
     * however we also pass through relevant parameters from the module config.
     */
    sendToView: function(additionalArgs) {
        var context = this.getBaseContext();
        var options = {};
        // if the 'popup' param was neither 'True' nor 'False', we assume it was a verbatim windowFeatures string to be passed through to window.open()
        if (this.popupWindowFeatures) options["windowFeatures"] = this.popupWindowFeatures;
        context.sendToView(this._params["viewTarget"], additionalArgs, Splunk.util.normalizeBoolean(this._params["dispatchBeforeRedirect"]), this.openInPopup, options);
    }
});